-- srbennet 12/05
-- revised 3/06

DROP SCHEMA corpus CASCADE;

CREATE SCHEMA corpus;


CREATE TABLE corpus.message(

    id SERIAL PRIMARY KEY, --the id of the message

	messageid INTEGER NOT NULL, -- disassociating the primary key from the messageid because there can be more than
						-- one version of a message in the database

    time TEXT NOT NULL, --the timestamp of the e-mail, may also include "mode=backstory"

    period INTEGER NOT NULL, --currently unused

    messagegroup TEXT NOT NULL,   -- the category of the message, currently one of the following

                  --list: Query, Response, Directive, Discussion, Confirmation,

                  -- Vendor Query, Vendor Response, UC Query, UC Response

    outcome TEXT NOT NULL, --always "blank"

    thread INTEGER NOT NULL, -- what storyline this message is a part of 

    noise TEXT NOT NULL, -- 'true' if this is a noise message

    response TEXT NOT NULL, -- 'true' if this message has been responded to by the 

                   -- conference planner

    replyto_message_id INTEGER, -- the messageID this message is a reply to, 

                     -- 0 if this is not a reply to another message
 
    subject TEXT NOT NULL, --the subject line of the e-mail

    body TEXT NOT NULL -- the body of the e-mail message

);


CREATE TABLE corpus.normalmessage(
	id SERIAL PRIMARY KEY, --the id of the message
    
    from_person_id INTEGER -- a link to the person table, which contains a
	-- valid e-mail address of the person this message is from, as well as their VIP status


	-- this is commented out because the VIP information is simply gotten out of the 
	-- person linked above... reinstate this if the person table ever loses this data
    -- vip_person_id INTEGER NOT NULL-- 'true' if the sender is a VIP
);



CREATE TABLE corpus.variablesendermessage(

    id SERIAL PRIMARY KEY, --the id of the message
    
    from_variable_id INTEGER -- a variable pointing to the e-mail address this message is from
);





-- recipients table, each message is guarenteed to at least have a message of

-- type 'to', could have as many as the writer wishes

CREATE TABLE corpus.recipient_message_person(

	id SERIAL PRIMARY KEY,

    message_messageid INTEGER, --the message id this recipient is attached to

    type TEXT, -- the type of recipient this is - to, cc, or bcc

    recipient_id INTEGER, -- the e-mail address of a recipient
    
    isvariable TEXT -- the class of recipient, either person or variable

);


CREATE TABLE corpus.label(

     id SERIAL PRIMARY KEY,  --  an id for the variable, almost meaningless

     label TEXT  -- the name of the variable being referenced

);


CREATE TABLE corpus.label_message(

     id SERIAL PRIMARY KEY,  --  an id for the variable, almost meaningless

     label_id INTEGER,  -- the id of the label being referenced

     message_messageid INTEGER -- the id of the message this label should be attached to

);



CREATE TABLE corpus.variable(
     id SERIAL PRIMARY KEY,  --  an id for the variable, almost meaningless
     name TEXT,  -- the name of the variable being referenced
     value TEXT, -- the value this variable is pointing to
     dateformat TEXT -- the format of the date in strftime notation (man strftime for more information)
);

CREATE TABLE corpus.messageclass(
	id SERIAL PRIMARY KEY,
	messageclass TEXT
	);
	
CREATE TABLE corpus.message_messageclass(
	id SERIAL PRIMARY KEY,
	message_id INTEGER,
	messageclass_id INTEGER
);
