package relativelayout.matrix;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import relativelayout.equation.Equation;
import relativelayout.equation.Variable;
import Jama.LUDecomposition;
import Jama.Matrix;

/**
 * <<Class summary>>
 * @author Rachael Bennett (srbennet@andrew.cmu.edu)
 * @version 0.1
 */
public class RelativeMatrix {

	// public static void main(String[] args) {
	// RelativeMatrix b = new RelativeMatrix();
	// ConcreteEquation e = new ConcreteEquation();
	// e.setCoefficient(1, Variable.get("x"));
	// e.setCoefficient(-1, Variable.get("y"));
	// e.setRightHandSide(10);
	// b.addEquation(Variable.get("x"), e);
	// e = new ConcreteEquation();
	// e.setCoefficient(1, Variable.get("y"));
	// e.setRightHandSide(42);
	// b.addEquation(Variable.get("y"), e);
	// Map<Variable, Double> foo = b.solveMatrix();
	// System.out.println(foo);
	// }

	private HashMap<Variable, Equation> equations = new HashMap<Variable, Equation>();
	
    private static boolean debug = false;

	/**
	 * 
	 */
	public RelativeMatrix() {}

	private Matrix actuallySolveMatrix(double[][] body, double[][] sol) {

		Matrix m = new Matrix(body);
		// System.out.println("The matrix dimensions are " + m.getRowDimension() + " col "
		// + m.getColumnDimension() + " \n ");
		// m.print(5, 0);
		LUDecomposition decomp = new LUDecomposition(m);
		Matrix solution = new Matrix(sol);
		// System.out.println("The solution dimensions are " + solution.getRowDimension() + " rol "
		// + solution.getColumnDimension());
		// solution.print(5, 0);
		// printMatrix(m, solution);

		try {
			return decomp.solve(solution);
		} catch (RuntimeException e) {
			if (e.getMessage().equals("Matrix is singular.")) {
				throw new AmbiguousLayoutException();
			} else {
				throw e;
			}

		}
	}

	/**
	 * This function creates a matrix in the uninitialized variable parameters from the equations
	 * above
	 * @param body an uninitialized int array
	 * @param solution an uninitialized int array
	 */
	private void createMatrix(double[][] body, double[][] solution) {
		// Iterator<Equation> equationsList = equations.values().iterator();
		ArrayList<Variable> allVariablesList;
		List<Variable> equationVariable;
		Variable variable;

		Equation incorporating;

		allVariablesList = new ArrayList<Variable>(getAllVariables());
		// do I waste valuable time transforming the tree set into a list? or do I go with the flow?
		// System.out.println("All variables are " + allVariablesList);
		for (int i = 0; i < body.length; i++ ) {
			if (equations.containsKey(allVariablesList.get(i))) {
				incorporating = equations.get(allVariablesList.get(i));
				equationVariable = incorporating.getVariables();
				body[i][i] = 1;// incorporating.getCoefficient(allVariablesList.get(i));
				solution[i][0] = incorporating.getRightHandSide();

				for (int j = 0; j < equationVariable.size(); j++ ) {
					variable = equationVariable.get(j);
					// System.out.println(j + " The variab;e index. This variable is " + variable);
					body[i][allVariablesList.indexOf(variable)] = incorporating
						.getCoefficient(variable);
				}

				// System.out.println("The right hand side of the current equation is "
				// + incorporating.getRightHandSide() + "\n");
			} else {
				body[i][i] = 1;
				// System.out.println("The variable doesn't have a match");
			}
		}
	}

	private Set<Variable> getAllVariables() {
		Iterator<Equation> equationsList = equations.values().iterator();
		HashSet<Variable> variables = new HashSet<Variable>();
		Equation incorporating;

		for (int i = 0; i < equations.size(); i++ ) {
			incorporating = equationsList.next();
			variables.addAll(incorporating.getVariables());
			// System.out.println("I saw " + incorporating.getVariables());
		}
		return variables;
	}

	public Map<Variable, Double> solveMatrix() {
		// System.out.println("All variables, in order: " + getAllVariables());

		int numVariables = getAllVariables().size();
		double[][] body = new double[numVariables][numVariables];
		double[][] solution = new double[numVariables][1];

		createMatrix(body, solution);
		// actuallySolveMatrix(body, solution).print(5, 0);
		return convertBack(actuallySolveMatrix(body, solution));
	}

	private Map<Variable, Double> convertBack(Matrix solutionMatrix) {
		// System.out.println("Solution matrix:");
		// solutionMatrix.print(5, 0);
		Map<Variable, Double> solutionMap = new HashMap<Variable, Double>();

		ArrayList<Variable> allVariablesList = new ArrayList<Variable>(getAllVariables());

		for (int j = 0; j < allVariablesList.size(); j++ ) {
			solutionMap.put(allVariablesList.get(j), solutionMatrix.get(j, 0));
		}

		return solutionMap;
	}

	public void addEquation(Variable variable, Equation equation) {
       equations.put(variable, equation);
       //		System.out.println("Adding an equation for " + variable);
       if (debug)
       {
           solveMatrix();
       }
	}

	public void printMatrix(Matrix m, Matrix sol) {
		String line;
		ArrayList variables = new ArrayList<Variable>(getAllVariables());
		for (int i = 0; i < variables.size(); i++ ) {
			line = variables.get(i).toString();
			for (int j = 0; j < variables.size(); j++ ) {
				line += "\t" + m.get(i, j);
			}
			System.out.println(line + "\t" + sol.get(i, 0));
		}
		System.out.println("\n\n\n");
	}
	
	public static void setDebug(boolean isDebugging) {
        debug = isDebugging;
    }
	
	public static boolean getDebug() {
		return debug;
	}

}
