package airspaceGUI;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.exception.JDBCConnectionException;
import org.hibernate.exception.SQLGrammarException;

import airspaceGUI.base.BasePanel;
import airspaceGUI.base.corpus.gui.CorpusMainWindow;
import airspaceGUI.base.event.EventPanel;
import airspaceGUI.base.person.PeoplePanel;
import airspaceGUI.base.room.RoomPanel;
import airspaceGUI.expansion.MenuBarCreation;
import airspaceGUI.expansion.Splash;
import airspaceGUI.expansion.TabbedPanelMenu;
import airspaceGUI.model.AgentModel;
import airspaceGUI.model.GatheringModel;
import airspaceGUI.model.MeetingModel;
import airspaceGUI.model.PeopleModel;
import airspaceGUI.model.RoomModel;

/**
 * Main class for the corpusGuiExtension project, controls how many tabs there
 * are and what is on them.
 * 
 * @author srbennet
 * 
 */
public class CorpusTabbedPane extends JFrame {
	/**
	 * TODO factor out menubar stuff TODO refactor the program so that you can
	 * have multiple databases open at once TODO command line option that would
	 * allow you to not show the CMRadar stuff
	 */
	private static final long serialVersionUID = 1L;

	private static final JTabbedPane mainTabPane = new JTabbedPane();

	private static final Object HELPTEXT = "Useage: \n \t-cmradar \t Enables the CMRADAR fields \n --help  \t Displays this help text \n \t --? \t Displays this help text";

	protected SessionFactory sessionFactory;

	/**
	 * what forms the menuBar over all the different tabs
	 */
	private TabbedPanelMenu menuBar = new TabbedPanelMenu();

	private ComponentListener cl = new ComponentAdapter() {
		public void componentResized(ComponentEvent arg0) {
			int minWidth = Math.max(getMinTabDimension().width, getWidth());
			int minHeight = Math.max(getMinTabDimension().height, getHeight());

			CorpusTabbedPane.this.setSize(minWidth, minHeight);
		}
	};

	public FindWindowManager fwm = new FindWindowManager();

	private ActionListener findListener;

	private Session session;

	public static void main(final String[] args) {
		System.setProperty("apple.laf.useScreenMenuBar", "true");
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			e.printStackTrace();
		}

		CorpusTabbedPane ctp = null;

		if (args.length > 0) {
			if (args[0].compareToIgnoreCase("-cmradar") == 0) {
				ctp = new CorpusTabbedPane(true);
			} else {
				System.out.println(HELPTEXT);
			}
		}

		if (ctp == null) {
			ctp = new CorpusTabbedPane(false);
		}

		ctp.pack();
		ctp.setVisible(true);
	}

	/**
	 * Initializes the corpusTabbed Pane with all of the necessary panels
	 * 
	 * @param cmradar
	 *            true if the cmradar fields should show, false otherwise
	 */
	public CorpusTabbedPane(boolean cmradar) {
		this.addComponentListener(cl);
		try {
			Splash loadingSplash = new Splash();
			setUp();
			session = sessionFactory.openSession();
			session.beginTransaction();
			PeopleModel peopleModel = new PeopleModel(session);
			MeetingModel meetingModel = new MeetingModel(session);
			GatheringModel gatheringModel = new GatheringModel(session);
			RoomModel roomModel = new RoomModel(session);
			AgentModel agentModel = new AgentModel(session);
			// VendorPanel vp = new VendorPanel();

			loadingSplash.updateProgress(50);
			PeoplePanel pp = new PeoplePanel(mainTabPane, peopleModel, session);
			loadingSplash.updateProgress(10);
			RoomPanel rp = new RoomPanel(mainTabPane, roomModel, session);
			loadingSplash.updateProgress(10);
			EventPanel ep;
			// STPPanel stpp;
			CorpusMainWindow cgw;

			if (cmradar) {
				ep = new EventPanel(session, mainTabPane, agentModel,
						meetingModel, gatheringModel, peopleModel, roomModel);
			} else {
				ep = new EventPanel(session, mainTabPane, gatheringModel,
						peopleModel);
			}
			// ep.load(session);
			loadingSplash.updateProgress(15);
			cgw = new CorpusMainWindow(this, mainTabPane, session, peopleModel);
			loadingSplash.updateProgress(15);
			// stpp = new STPPanel();

			sessionFactory.close();
			loadingSplash.removeSplash();

			findListener = new ActionListener() {
				public void actionPerformed(final ActionEvent arg0) {
					fwm.setVisible(true);
				}
			};

			// add a change listener to fire whenever the tab changes
			mainTabPane.addChangeListener(new ChangeListener() {
				private int selectedIndex = -1;
				
				/**
				 * if the tab change should be ignored
				 * */
				private boolean ignoreTabChange = false;

				public void stateChanged(ChangeEvent cevt) {
					if (ignoreTabChange) {
						return;
					}
					
					JTabbedPane tabPane = (JTabbedPane) cevt.getSource();
					BasePanel selected = (BasePanel) tabPane.getSelectedComponent();
					
					if (selectedIndex != -1) {
						BasePanel lastSelected = (BasePanel) tabPane
								.getComponentAt(selectedIndex);
						
						if (lastSelected.isModified()) {
							ignoreTabChange = true;
							tabPane.setSelectedComponent(lastSelected);

							int option = JOptionPane.showConfirmDialog(null,
									"You have made some changes to the current record. "
											+ "Do you want to save them?", "",
									JOptionPane.YES_NO_CANCEL_OPTION);
							if (option == JOptionPane.YES_OPTION) {
								lastSelected.save();
							} else if (option == JOptionPane.NO_OPTION) {
								lastSelected.setModified(false);
							} else if (option == JOptionPane.CANCEL_OPTION) {
								ignoreTabChange = false;
								return;
							}
						}
						tabPane.setSelectedComponent(selected);
						ignoreTabChange = false;
					}

					menuBar.reset();
					selected.setMenuBar(menuBar);
					menuBar.getFind().addActionListener(findListener);
					menuBar.getFind().setEnabled(true);

					try {
						selected.setPageLoading(true);
						selected.fillInText(session);
						selected.setPageLoading(false);
					} catch (SQLGrammarException e) {
						// need to rollback or commit the changes before getting
						// a
						// new one or the database gets confused, I chose
						// rollback
						// since the changes the user made the user doesn't
						// necessarialy
						// want to keep unless they have explicitly saved, in
						// which case
						// those changes will already have been committed
						session.getTransaction().rollback();
						session = sessionFactory.openSession();
						session.beginTransaction();
					}
					setTitle(selected.getTitle());
					selectedIndex = tabPane.indexOfComponent(selected);
					addQuitListener();
				}

			});

			this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
			this.addWindowListener(new WindowAdapter() {
				public void windowClosing(final WindowEvent arg0) {
					exit();
				}
			});

			mainTabPane.add(pp.getPane());
			mainTabPane.add(rp.getPane());
			mainTabPane.add(ep.getPane());
			// mainWindow.add(stpp.getPane());
			// mainWindow.add(vp.getPane());
			// mainWindow.setEnabledAt(4, false);
			mainTabPane.add(cgw.getPane());

			fwm.addFindWindow(pp.getFindWindow(), pp.getName());
			fwm.addFindWindow(rp.getFindWindow(), rp.getName());
			fwm.addFindWindow(ep.getFindWindow(), ep.getName());
			fwm.addFindWindow(cgw.getFindWindow(), cgw.getName());

			getContentPane().add(mainTabPane);
			// menuBar.addArchiveListener(dumpListener);
			this.setJMenuBar(menuBar);
			loadingSplash.removeSplash();
		} catch (JDBCConnectionException e) {
			// tell the user of the problem, and then terminate the program
			JOptionPane
					.showMessageDialog(
							mainTabPane,
							"Cannot connect to the database.  \nPlease check that the database is \n running and that "
									+ "corpus.hibernate.xml \n has the correct database settings",
							"Error", JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		} catch (ExceptionInInitializerError ex) {
			// tell the user of the problem, and then terminate the program
			ex.printStackTrace();
			JOptionPane
					.showMessageDialog(
							mainTabPane,
							"Hibernate Error: "
									+ ex.getCause().getMessage()
									+ ".\nPlease check that corpus.hibernate.xml is configured correctly",
							"Error", JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		}
	}

	/**
	 * This searches all the tabs in the tab pane and returns the smallest size.
	 * 
	 * @return the smallest preferred size of the panel
	 */
	private Dimension getMinTabDimension() {
		final Component[] tabs = mainTabPane.getComponents();
		if (tabs.length != 0) {
			int width = tabs[0].getPreferredSize().width;
			int height = tabs[0].getPreferredSize().height;

			for (int i = 1; i < tabs.length; i++) {
				// System.out.println("This tab is " + tabs[i].getName() +
				// " and has a preferred size of " +
				// tabs[i].getPreferredSize());
				width = Math.min(tabs[i].getPreferredSize().width, width);
				height = Math.min(tabs[i].getPreferredSize().height, height);
				// System.out.println("The min width and height are now " +
				// width + ", " + height);
			}

			// System.out.println("Returning " + new Dimension(width, height));

			return new Dimension(width, height);
		}
		return new Dimension(0, 0);
	}

	/**
	 * Sets up the test environment for this class by initiating a Hibernate
	 * SessionFactory based on the default configuration (the file
	 * "corpus.hibernate.xml" on the classpath.
	 */
	protected final void setUp() {
		try {
			// Create the SessionFactory from hibernate.cfg.xml
			URL cfgUrl = ClassLoader.getSystemClassLoader().getResource(
					"corpus.hibernate.xml");
			// File configFile = new File(url.toURI());
			Configuration config = new Configuration().configure(cfgUrl);

			sessionFactory = config.buildSessionFactory();
		} catch (Exception ex) {
			// Make sure you log the exception, as it might be swallowed
			System.err.println("Initial SessionFactory creation failed." + ex);
			throw new ExceptionInInitializerError(ex);
		}
	}

	/**
	 * Custom exit function to exit all the panels, and then terminate the
	 * program
	 * 
	 * @return true if the program is able to terminate, false otherwise
	 */
	public boolean exit() {
		boolean stillExit = true;

		for (int exitCount = 0; exitCount < mainTabPane.getTabCount(); exitCount++) {
			// execute a custom exit function for each tab before closing
			if (stillExit
					&& ((MenuBarCreation) mainTabPane.getComponentAt(exitCount)) != null) {
				stillExit = ((MenuBarCreation) mainTabPane
						.getComponentAt(exitCount)).exit();
			}
		}
		if (stillExit) {
			sessionFactory.close();
			System.exit(0);
		}

		return stillExit;
	}

	/**
	 * This registers what happens when the Quit menu item is selected
	 * 
	 */
	private void addQuitListener() {
		// register what happens when this menu item is selected
		if (System.getProperty("mrj.version") == null) {
			menuBar.getQuit().addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					exit();
				}
			});
		} else {
			final QuitHandlerMac qhm = new QuitHandlerMac(); // get the menubar
			// to actually use the application menu on a mac
			qhm.getQuitHandler(this);
		}
	}

}
