/**
 * 
 */
package airspaceGUI.base;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.UndoManager;

/**
 * Provides the action that should be taken when the user wants to redo a change
 * 
 * @author srbennet
 * 
 */
class RedoAction extends AbstractAction
{
    /**
     * 
     */
    private transient final UndoManager undo;

    private static UndoAction undoAction;

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    /**
     * Constructs an undoAction
     * 
     * @param undo the manager of all the undo/redo edits
     */
    public RedoAction(UndoManager undo)
    {
        super("Redo");
        this.undo = undo;
        setEnabled(false);
    }

    /**
     * Defines the action for the object to perform
     * 
     * @param e
     *            the action event
     */
    public void actionPerformed(final ActionEvent e) {
        try
        {
            if (undo.canRedo())
            {
                undo.redo();
            } else
            {
                Toolkit.getDefaultToolkit().beep();
            }
            updateRedoState();
            undoAction.updateUndoState();
        } catch (CannotRedoException ex)
        {
            ex.printStackTrace();
        }
    }

    /**
     * Updates the Redo stack.
     * Called when an action has been undone
     *
     */
    protected void updateRedoState() {
        if (undo.canRedo())
        {
            setEnabled(true);
            putValue(Action.NAME, undo.getRedoPresentationName());
        } else
        {
            setEnabled(false);
            putValue(Action.NAME, "Redo");
        }
    }

    /**
     * 
     * @param undo the undoAction that this object should work off of
     */
    public void setUndoAction(final UndoAction undo) {
        RedoAction.undoAction = undo;
    }
}