package airspaceGUI.base.corpus.gui;

import java.awt.FileDialog;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.StateEdit;
import javax.swing.undo.StateEditable;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;
import org.hibernate.Session;

import airspaceGUI.base.BasePanel;
import airspaceGUI.base.corpus.CorpusGUIMessage;
import airspaceGUI.base.corpus.MessageType;
import airspaceGUI.base.corpus.Variable;
import airspaceGUI.expansion.FindWindow;
import airspaceGUI.expansion.TabbedPanelMenu;
import airspaceGUI.model.PeopleModel;
import airspaceGUI.model.VariableModel;
import edu.cmu.radar.base.Person;
import edu.cmu.radar.base.corpus.FakeMessage;
import edu.cmu.radar.base.corpus.FakeMessageClass;
import edu.cmu.radar.base.corpus.NormalFakeMessage;
import edu.cmu.radar.base.corpus.Recipient;
import edu.cmu.radar.base.corpus.VariableSenderFakeMessage;

/*
 * Created on May 27, 2005
 */
/**
 * This class generates the main window and populates all its fields when
 * necessary
 */
public class CorpusMainWindow extends BasePanel implements StateEditable {
	private static final String DATA = "data";

	/**
     * 
     */
	private static final String BCC_RECIPIENT = "bcc";

	/**
     * 
     */
	private static final String CC_RECIPIENT = "cc";

	/**
     * 
     */
	private static final String TO_RECIPIENT = "to";

	private static final long serialVersionUID = 1L;

	/**
	 * this is the opening of a file which can be redone
	 * 
	 */
	protected static final String OPENDATABASEEDIT = "Open Database Edit";

	/**
	 * this is a change in which message is being edited
	 * 
	 */
	protected static final String MESSAGEEDIT = "Message Edit";

	/**
	 * this is a change in the text of the current message being edited
	 */
	protected static final String TEXTEDIT = "Text Edit";

	/**
	 * this is a change in the times or order of the messages in the message
	 * ArrayList
	 */
	protected static final String RANDOMEDIT = "Random Edit";

	/**
	 * the current type of edit that is taking place
	 */
	protected String currentEdit;

	/**
	 * The current series of edits that have taken place
	 */
	protected StateEdit currentState;

	/**
	 * The actual visible GUI interface
	 */
	protected CorpusMainPanel mainPanel;

	/**
	 * The Menu for advanced corpus GUI functions
	 */
	private JMenu advancedMenu = new JMenu("Advanced");

	/**
	 * Menu item to export corpus data for the injector
	 */
	private JMenuItem export = new JMenuItem("Export for Injector...");

	/**
	 * Menu item to set the preferences for corpus generation
	 */
	private JMenuItem pref = new JMenuItem("Message Preferences...");

	/**
	 * Menu item to launch the graphical variable editor
	 */
	private JMenuItem vars = new JMenuItem("Variable Editor...");

	/**
	 * Menu item to apply random times to the corpus data
	 */
	protected JMenuItem randomTimes = new JMenuItem("Apply Random Times...");

	/**
	 * Menu item to undo random time operations
	 */
	protected JMenuItem defaultTimes = new JMenuItem("Apply Default Times");

	/**
	 * new RandomWindow to be launched by the "Apply Random Times" JMenuItem
	 */
	protected RandomWindow randWin = null;

	/**
	 * preferences window launched by selecting JMenuItem pref
	 */
	protected Preferences messPref = null;

	/**
	 * modal variables window launched by the "Variable Editor" JMenuItem
	 */
	protected VariableWindow varWin = null;

	/**
	 * A list of all the errors to be output on an attempt to save
	 */
	protected Map errorList = new HashMap();

	/**
	 * the max ID encountered in the file
	 */
	protected int maxID = 1;

	/**
	 * the title of the message that is showing
	 */
	protected String title;

	/**
	 * The list of data that the user has modofied this session
	 */
	private static ArrayList modifiedMessages = new ArrayList();

	/**
	 * The frame this window is in
	 */
	protected JFrame parent;

	/**
	 * Manager used to hold all the corpus variables
	 */
	private static VariableModel variableModel;

	/**
	 * This is the action that is clicked when the export menu item is clicked.
	 */
	private ActionListener exportListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			saveForInjector();
		}
	};

	/**
	 * This is the action performed when the prefreences menu item is clicked
	 */
	private ActionListener prefListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			if (messPref == null) {
				messPref = new Preferences(); // launch a preferences window
				// if one
				// has not been launched before
			}
			messPref.setVisible(true);
		}
	};

	/**
	 * This launches the variable window in response to a menu click
	 */
	private ActionListener varsListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			if (varWin == null) {
				varWin = new VariableWindow(CorpusMainWindow.this.mainPanel,
						CorpusMainWindow.variableModel);
				// launch a variable
				// window if one has not
				// been before
			}
			varWin.setVisible(true);
		}
	};

	/**
	 * This randomizes the times on the messages in response to a menu item
	 * click or shortcut key press.
	 */
	private ActionListener randomListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			saveText();
			final boolean closed;
			currentEdit = RANDOMEDIT;
			mainPanel.removeUndoListeners(undoableEditListener);
			undoMenu.setEnabled(true);

			randWin = new RandomWindow(new ArrayList(getDataList())); // launch
			// a new RandomWindow
			// TODO figure out how to figure out when the window has been closed
			// and when the window actually changed the messages because the
			// equals
			// doesn't work for some reason
			randWin.setVisible(true);
			currentState = new StateEdit(CorpusMainWindow.this, currentEdit);

			// System.out.println(getDataList().get(getIndexOfDisplayed()));

			if (!randWin.getMessages().equals(getDataList())) {
				// System.out.println("Something has changed");
				CorpusGUIMessage previous = (CorpusGUIMessage) getDataList()
						.get(getIndexOfDisplayed());
				setDataList(new ArrayList(randWin.getMessages()));// update
				// the
				// ArrayList
				// with the new
				// randomized one

				// get the index of this message in case it has changed
				int index = getDataList().indexOf(previous);
				setIndexOfDisplayed(index);

				// totally refresh the panel.to reflect the time changes
				fillInText((CorpusGUIMessage) getDataList().get(
						getIndexOfDisplayed())); // fill
				// in the text from the new message atLabel index messageOn
				// only refresh the status label
				setStatus(getIndexOfDisplayed() + 1);
			}
			currentState.end();
			undoManager.addEdit(currentState);
			mainPanel.addUndoListeners(undoableEditListener);
		}
	};

	/**
	 * This returns the randomized times to default ones.
	 */
	private ActionListener defaultTimeListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			currentEdit = RANDOMEDIT;
			mainPanel.removeUndoListeners(undoableEditListener);
			undoMenu.setEnabled(true);
			currentState = new StateEdit(CorpusMainWindow.this, currentEdit);

			setDataList(RandomWindow.setToDefault(new ArrayList(getDataList())));
			fillInText((CorpusGUIMessage) getDataList() // reload this message
					// after reseting
					.get(getIndexOfDisplayed()));
			currentState.end();
			undoManager.addEdit(currentState);
			mainPanel.addUndoListeners(undoableEditListener);
		}
	};

	/**
	 * FindListener to launch this panel's findWindow.
	 */
	ActionListener findListener;

	/**
	 * Default constructor, this initializes everything in the GUI and places
	 * them correctly on the interface. It also registers the actions that occur
	 * when objects on the interface are manipulated
	 * 
	 * @param parent
	 *            The fram on which this panel will be shown
	 * @param session
	 *            An initialized hibernate session to read the data out of.
	 * @param tab
	 *            The tabbed panel this object is in, null if it is in it's own
	 *            window
	 * @param peopleModel
	 *            All the people that exist in the world.
	 * 
	 */
	public CorpusMainWindow(JFrame parent, final JTabbedPane tab,
			Session session, PeopleModel peopleModel) {
		super();
		this.parent = parent;
		setSession(session);
		getSession().beginTransaction();
		variableModel = new VariableModel(session);
		mainPanel = new CorpusMainPanel(this.parent, session, peopleModel);
		/*
		 * add the main panel of text boxes and labels to the main window This
		 * also saves the reference to the MainPanel so that if a file is loaded
		 * the appropriate fields can be populated
		 */
		parent.setTitle("Compose a Message");

		nextButton.setText("Next Message");
		prevButton.setText("Previous Message");
		newButton.setText("New Message");
		deleteButton.setText("Delete Message");

		findWindow = new FindCorpus(CorpusMainWindow.this, getDataList(), tab);
		this.addModificationListeners(textFieldModificationListener,
				comboBoxModificationListener, checkBoxModificationListener);

		readDataFromDB();
		// System.out.println("Setting the current edit type");
		currentEdit = MESSAGEEDIT;
	}

	/**
	 * This sets the title of the window to the given string.
	 * 
	 * @param title
	 *            The title to set in the titlebar
	 */
	protected void setTitle(String title) {
		this.title = title;
		parent.setTitle(title);
	}

	/**
	 * Gets the title the parent JFrame should have
	 * 
	 * @return the title of this panel
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * goes through the list of messages and records the maximum id in the list.
	 * 
	 */
	private final void getMaxId() {
		final ArrayList mess = new ArrayList(getDataList());

		for (int i = 0; i < mess.size(); i++) {
			if (((CorpusGUIMessage) mess.get(i)).getMessageId().intValue() > maxID) {
				maxID = ((CorpusGUIMessage) mess.get(i)).getMessageId()
						.intValue();
			}
		}
	}

	/**
	 * A custom exit function. This one saves the messages to a file, calls the
	 * variable windows' exit function, if the variable window existed and
	 * terminates the program
	 * 
	 * @return true if exit is ok to proceed, false otherwise
	 * 
	 */
	public boolean exit() {
		super.exit();

		if (varWin != null) {
			varWin.exit();
		}

		return true;
	}

	/**
	 * Fills in the fields of the GUI with the information from the message
	 * 
	 * @param message
	 *            The message to show in the gui.
	 */
	public void fillInText(CorpusGUIMessage message) {
		mainPanel.fillInText(message);
		setTitle(message.getSubject());
		// update the message progress, make sure the status label is updated
		setIndexOfDisplayed(getDataList().indexOf(message));
		setStatus(getIndexOfDisplayed() + 1);
		setButtonPermissions();
		currentEdit = MESSAGEEDIT;
	}

	/**
	 * Saves the text the GUI is currently showing temporarially into a
	 * FakeMessage
	 * 
	 * @return the text in a FakeMessage
	 */
	public Object saveText() {
		CorpusGUIMessage saveIn = null;
		int id = 0;

		if (getIndexOfDisplayed() <= getDataList().size()) {
			saveIn = ((CorpusGUIMessage) getDataList().get(
					getIndexOfDisplayed()));
		}

		saveIn = getMessageBasedOnMessageClass(saveIn);
		id = saveIn.getId();

		// save the Time
		saveIn.setTime(mainPanel.datePanel.getTime());

		try {
			saveIn
					.setMessageId((new Integer(mainPanel.messageIdField
							.getText())));
		} catch (NumberFormatException eX) {
			/*
			 * save the message containing the error, so that it can be reported
			 * to the user
			 */
			errorList.put(saveIn.getMessageId(), "The \"ID\" field in message "
					+ saveIn.getMessageId() + " is invalid.\n");
			JOptionPane.showMessageDialog(null, "The \"ID\" field in message "
					+ saveIn.getMessageId() + " is invalid.\n", "Error",
					JOptionPane.ERROR_MESSAGE);
			return null;
		}

		try {
			saveIn.setPeriod(new Integer(mainPanel.periodField.getText())
					.intValue());
		} catch (NumberFormatException eX) {
			/*
			 * save the message containing the error, so that it can be reported
			 * to the user
			 */
			errorList.put(saveIn.getMessageId(),
					"The \"Period\" field in message " + saveIn.getMessageId()
							+ " is invalid.\n");
			JOptionPane.showMessageDialog(null,
					"The \"Period\" field in message " + saveIn.getMessageId()
							+ " is invalid.\n", "Error",
					JOptionPane.ERROR_MESSAGE);
			return null;
		}
		// save the group this message belongs to
		saveIn.setGroup(mainPanel.groupField.getText());
		// save the outcome of the message
		saveIn.setOutcome(mainPanel.outcomeField.getText());

		try {
			saveIn.setThread(new Integer(mainPanel.threadField.getText())
					.intValue());
		} catch (NumberFormatException eX) {
			/*
			 * save the message containing the error, so that it can be reported
			 * to the user
			 */
			errorList.put(saveIn.getMessageId(),
					"The \"Thread\" field in message " + saveIn.getMessageId()
							+ " is invalid.\n");
			JOptionPane.showMessageDialog(null,
					"The \"Thread\" field in message " + saveIn.getMessageId()
							+ " is invalid.\n", "Error",
					JOptionPane.ERROR_MESSAGE);
			return null;
		}

		try {
			saveIn.setReplyTo((new Integer(mainPanel.replyToField.getText())
					.intValue()));
		} catch (NumberFormatException eX) {
			/*
			 * save the message containing the error, so that it can be reported
			 * to the user
			 */
			errorList.put(saveIn.getMessageId(),
					"The \"Reply To:\" field in message "
							+ saveIn.getMessageId() + " is invalid.\n");
			JOptionPane.showMessageDialog(null,
					"The \"Reply To:\" field in message "
							+ saveIn.getMessageId() + " is invalid.\n",
					"Error", JOptionPane.ERROR_MESSAGE);
			return null;
		}

		try {
			// start complecated from code, hopefully to become less complecated
			// soon!
			String text = mainPanel.fromField.getText();
			if (text.matches(MessageType.EMAILVERIFICATION)) {
				if (saveIn.getMessage() instanceof NormalFakeMessage) {
					// this is how to save a normal person message when it was a
					// normal person message to begin with
					if (text.indexOf("<") > -1) {
						text = text.substring(text.indexOf("<") + 1, text
								.lastIndexOf(">"));
					}
					saveIn.setFrom(mainPanel.book.getPerson(text));
				} else {
					saveIn.switchMessageType(mainPanel.book.getPerson(text));
					if (id != 0) {
						saveIn.setId(id);
					}
				}
			} else if (Variable.isValidInsertedVariable(text)) {
				if (saveIn.getMessage() instanceof VariableSenderFakeMessage) {
					// make sure that the variable inputed actually exists
					if (variableModel.getVariable(text) != null) {
						// get the variable that was inputed and save it in the
						// from field
						saveIn.setFrom(variableModel.getVariable(text));
					} else {
						errorList.put(saveIn.getMessageId(),
								"The \"From\" field in message "
										+ saveIn.getMessageId()
										+ " is invalid.\n");
						JOptionPane.showMessageDialog(null,
								"The \"From\" field in message "
										+ saveIn.getMessageId()
										+ " is not a valid variable.\n",
								"Error", JOptionPane.ERROR_MESSAGE);
						return null;
					}
				} else {
					// make sure the variable window is initialized
					if (varWin == null) {
						varWin = new VariableWindow(mainPanel, variableModel);
						varWin.setVisible(false);
					}
					// this message is now a variableSender message
					saveIn.switchMessageType(variableModel.getVariable(text));
					if (id != 0) {
						// save the id if it has one, so that it isn't seen as a
						// new message
						saveIn.setId(id);
					}
				}
			} else {
				// the text doesn't match the variable verification regex
				System.out.println("The error is "
						+ Variable.isValidInsertedVariable(text)
						+ " the text is " + text);
				errorList.put(saveIn.getMessageId(),
						"The \"From\" field in message "
								+ saveIn.getMessageId()
								+ " is not a valid variable.\n");
				return null;
			}
		} catch (Exception ex) {
			errorList.put(saveIn.getMessageId(),
					"The \"From\" field in message " + saveIn.getMessageId()
							+ " is invalid.\n");
			ex.printStackTrace();
			return null;
		}

		saveIn.setSubject(mainPanel.subjectField.getText());
		saveIn.setBody(mainPanel.bodyField.getText());

		saveIn.setNoise(mainPanel.noiseCheck.getSelectedObjects() != null);
		saveIn
				.setResponse(mainPanel.responseCheck.getSelectedObjects() != null);

		Set recipientSet = new HashSet();
		try {
			Set toRecipients = saveRecipients(saveIn, mainPanel.toField
					.getText(), TO_RECIPIENT);
			recipientSet.addAll(toRecipients);
		} catch (NullPointerException e) {
			// TODO inform the user the to field cannot be saved
			e.printStackTrace();
			return null;
		}
		recipientSet.addAll(saveRecipients(saveIn, mainPanel.ccField.getText(),
				CC_RECIPIENT));

		recipientSet.addAll(saveRecipients(saveIn,
				mainPanel.bccField.getText(), BCC_RECIPIENT));

		saveIn.setLabels(mainPanel.getSelectedLabels());
		saveIn.setRecipients(recipientSet);

		getDataList().set(getIndexOfDisplayed(), saveIn);
		errorList.remove(saveIn);

		if (isModified()) {
			modifiedMessages.add(saveIn.getMessage());
		}

		return saveIn;
	}

	/**
	 * @param saveIn
	 *            The message the program thinks it is saving
	 * @return The message that was edited, based on the messageClass that is
	 *         showing and the message passed into this program
	 */
	private CorpusGUIMessage getMessageBasedOnMessageClass(
			CorpusGUIMessage saveIn) {
		final StringTokenizer messageClassTokenizer = new StringTokenizer(
				mainPanel.messageClassField.getText(), ",");
		String messageClassString = "";
		HashSet messageClassSet = new HashSet();
		while (messageClassTokenizer.hasMoreTokens()) {
			messageClassString = messageClassTokenizer.nextToken();
			if (messageClassString.startsWith(" ")) {
				// take off leading spaces, since there can be spaces in the
				// messageClass,
				// we can't tokenize on it.
				messageClassString = messageClassString.substring(1);
			} else if (messageClassString.endsWith(" ")) {
				messageClassString = messageClassString.substring(0,
						messageClassString.lastIndexOf(" "));
			}
			FakeMessageClass mc = new FakeMessageClass(messageClassString);
			if (saveIn == null
					&& ((CorpusGUIMessage) getDataList().get(
							getIndexOfDisplayed())).getMessageClass().contains(
							mc)) {
				saveIn = (CorpusGUIMessage) getDataList().get(
						getIndexOfDisplayed());
			} else if (saveIn == null) {
				saveIn = new CorpusGUIMessage(new NormalFakeMessage());
			}

			if (mainPanel.messageClassList.indexOf(mc) > -1) {
				messageClassSet.add(mainPanel.messageClassList
						.get(mainPanel.messageClassList.indexOf(mc)));
			} else {
				getSession().persist(mc);
				getSession().save(mc);
				mainPanel.messageClassList.add(mc);
				messageClassSet.add(mc);
				// mainPanel.messageClassList.add(mc);
			}
			// TODO figure out how to determine a new branch in the tree of the
			// root message
			// right now it does if the current message doesn't have anything
			// the same
			// and just edits the current message if there is atLabel least one
			// tag
			// the same
		}

		saveIn.setMessageClass(messageClassSet);
		return saveIn;
	}

	/**
	 * Saves the recipients from the GUI to a hashSet, and returns it
	 * 
	 * @param saveIn
	 *            the message to save the recipients in
	 * @param recipientTokenList
	 *            the list of recipients to tokenize
	 * @param recipientType
	 *            the type of recipients that are contained in the list
	 * @return a HashSet containting all the recipients of saveIn
	 */
	private Set saveRecipients(CorpusGUIMessage saveIn,
			String recipientTokenList, String recipientType) {
		Set recipientSet = new HashSet();

		String[] emails = recipientTokenList.split("\\s*,\\s*");

		if (emails.length == 0 && recipientType.equals(TO_RECIPIENT)) {
			/*
			 * save the message containing the error, so that it can be reported
			 * to the user
			 */
			errorList.put(saveIn.getMessageId(), "The " + recipientType
					+ " field in message " + saveIn.getMessageId()
					+ " is invalid.\n");
		}

		for (String email : emails) {
			if (email.matches(MessageType.EMAILVERIFICATION)) {
				if (email.indexOf("<") > -1) {
					email = email.substring(email.indexOf("<") + 1, email
							.lastIndexOf(">"));
				}
				Person recipientPerson = mainPanel.book.getPerson(email);
				Recipient recipient;
				if (recipientPerson != null) {
					recipient = new Recipient(recipientType, recipientPerson,
							saveIn.getMessage(), "false");
				} else {
					JOptionPane.showMessageDialog(null, "The \""
							+ recipientType + "\" field in message "
							+ saveIn.getMessageId() + " is invalid.\n",
							"Error", JOptionPane.ERROR_MESSAGE);
					return null;
				}

				recipientSet.add(recipient);
			} else if (Variable.isValidInsertedVariable(email)) {

				if (varWin == null) {
					varWin = new VariableWindow(mainPanel, variableModel);
				}

				Recipient recipient = new Recipient(recipientType,
						variableModel.getVariable(email), saveIn.getMessage(),
						Boolean.toString(true));
				recipientSet.add(recipient);
			} else {
				errorList.put(saveIn.getMessageId(), "The \"" + recipientType
						+ ":\" field in message " + saveIn.getId()
						+ " is invalid.\n");
			}
		}
		return recipientSet;
	}

	/**
	 * Clears the text off of the display
	 */
	public void createNewInstance() {
		makeBlank(maxID);
		getDataList().add(getIndexOfDisplayed(), new CorpusGUIMessage());
		mainPanel.messageClassField.requestFocusInWindow();
	}

	/**
	 * This function blanks out the fields in preparation for a new message to
	 * be written. If the preferences window has been used, it keeps the fields
	 * checked within the preferences window
	 * 
	 * @param id
	 *            - the next un-used id to assign to the new message
	 */
	public void makeBlank(int id) {
		// only clear the following fields if the preference to
		// keep them doesn't exist or is false
		if (messPref == null || !messPref.getKeepTime()) {
			mainPanel.datePanel.clearTime();
		}
		if (messPref == null || !messPref.getKeepPeriod()) {
			mainPanel.periodField.setText("");
		}
		if (messPref == null || !messPref.getKeepGroup()) {
			mainPanel.groupField.setText("");
		}
		if (messPref == null || !messPref.getKeepOutcome()) {
			mainPanel.outcomeField.setText("");
		}
		if (messPref == null || !messPref.getKeepThread()) {
			mainPanel.threadField.setText("");
		}
		if (id >= 0) {
			mainPanel.messageIdField.setText(String.valueOf(++id));
		}
		if (messPref == null || !messPref.getKeepReplyTo()) {
			mainPanel.replyToField.setText("");
		}
		if (messPref == null || !messPref.getKeepSubject()) {
			mainPanel.subjectField.setText("");
			parent.setTitle("Compose a Message");
		}
		if (messPref == null || !messPref.getKeepFrom()) {
			mainPanel.fromField.setText("");
		}
		if (messPref == null || !messPref.getKeepTo()) {
			mainPanel.toField.setText("");
		}
		if (messPref == null || !messPref.getKeepCC()) {
			mainPanel.ccField.setText("");
		}
		if (messPref == null || !messPref.getKeepBCC()) {
			mainPanel.bccField.setText("");
		}
		if (messPref == null || !messPref.getKeepBody()) {
			mainPanel.bodyField.setText("");
		}

		if (messPref == null || !messPref.getKeepVIP()) {
			mainPanel.vipCheck.setSelected(false);
		}
		if (messPref == null || !messPref.getKeepNoise()) {
			mainPanel.noiseCheck.setSelected(false);
		}
		if (messPref == null || !messPref.getKeepResponse()) {
			mainPanel.responseCheck.setSelected(false);
		}
		if (messPref == null || !messPref.getKeepLabels()) {
			mainPanel.clearSelectedLabels();
		}

		if (messPref == null || !messPref.getKeepMessageClass()) {
			mainPanel.messageClassField.setText("Root Message");
		}
	}

	/**
	 * Loads the data from the database
	 * 
	 * @param session
	 *            An initialized session connected to the correct database
	 * @return true if the load was successful, false otherwise
	 * 
	 */
	public boolean load(Session session) {
		setSession(session);
		this.getSession().beginTransaction();

		if (this.getDataList() == null) {
			readDataFromDB();
		}
		return true;
	}

	/**
	 * Deletes the currently shown message from the database and shows the next
	 * message in the List
	 * 
	 * @return true if the deletion was successful, false otherwise
	 */
	public boolean deleteDisplayedObject() {
		try {
			List data = getDataList();
			FakeMessage object = ((CorpusGUIMessage) data
					.get(getIndexOfDisplayed())).getMessage();
			if (getSession().contains(object)) {
				getSession().delete(object);
			}
			getDataList().remove(data.get(getIndexOfDisplayed()));
			fillInText(getSession());
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	/**
	 * Saves the data back to the database
	 * 
	 * @return true if the save was successful, false otherwise
	 */
	public boolean save() {
		CorpusGUIMessage saved = (CorpusGUIMessage) saveText();
		persist(saved.getMessage());
		/*
		 * re-enable the next and previous buttons
		 */
		setButtonPermissions();

		// Set the message counter
		setStatus(getIndexOfDisplayed() + 1);
		return true;
	}

	/**
	 * Function used to save the messages out to a file that the injector can
	 * read (aka broken xml)
	 * 
	 */
	public void saveForInjector() {
		final CorpusGUIMessage saved = (CorpusGUIMessage) saveText();
		// save the current message in case it has been edited

		if (!errorList.isEmpty()) {
			String allErrors = "\t";
			final Iterator errors = errorList.values().iterator();
			while (errors.hasNext()) {
				allErrors += ("\t" + ((String) errors.next()));
			}
			JOptionPane.showMessageDialog(null,
					"The messages cannot be saved to a file because the "
							+ "following messages are invalid:\n" + allErrors,
					"Error", JOptionPane.ERROR_MESSAGE);
			return;
		}

		if (saved == null) {
			// something went wrong
			return;
		}
		if (getDataList() != null
				&& getIndexOfDisplayed() < getDataList().size()) {
			getDataList().set(getIndexOfDisplayed(), saved);
		} else if (getDataList() != null) {
			getDataList().add(saved);
		} else {
			setDataList(new ArrayList());
			getDataList().add(saved);
		}
		final ExportOptions optionsPane = new ExportOptions(parent,
				getSession());
		final FileDialog fd = new FileDialog(parent, "Save to an XML file",
				FileDialog.SAVE);
		String saveHere = null;
		fd.setVisible(true);
		if (fd.getFile() != null) {
			/*
			 * if (fd.getFile().lastIndexOf("xml") >= 0) { buttonPanel.saveHere
			 * = fd.getDirectory() + fd.getFile(); } else { buttonPanel.saveHere
			 * = fd.getDirectory() + fd.getFile() + ".xml"; }
			 */
			saveHere = fd.getDirectory() + fd.getFile();
		}

		if (saveHere != null)
			try {
				// write out all the messages to the file
				FileWriter normalOut = new FileWriter(saveHere
						+ ".messages.xml");
				FileWriter backstoryOut = new FileWriter(saveHere
						+ ".backstory.xml");

				List normalMessages = new ArrayList();
				List backstoryMessages = new ArrayList();

				for (int i = 0; i < getDataList().size(); i++) {
					if (!((CorpusGUIMessage) getDataList().get(i))
							.getBackstory()
							&& optionsPane
									.isWantedMessageClass(((CorpusGUIMessage) getDataList()
											.get(i)))) {
						/*
						 * normalOut.write(((CorpusGUIMessage)getDataList().get(i
						 * )) .getBrokenXML(getSession())); normalOut.flush();
						 */
						normalMessages.add(getDataList().get(i));
					} else if (optionsPane
							.isWantedMessageClass(((CorpusGUIMessage) getDataList()
									.get(i)))) {
						/*
						 * backstoryOut.write(((CorpusGUIMessage)getDataList().get
						 * (i)) .getBrokenXML(getSession()));
						 * backstoryOut.flush();
						 */
						backstoryMessages.add(getDataList().get(i));
					}
				}

				writeBrokenXMLMessages(normalMessages, normalOut);
				writeBrokenXMLMessages(backstoryMessages, backstoryOut);

				normalOut.close();
				backstoryOut.close();

				if (optionsPane.exportVariables()) {
					exportVariables(saveHere);
				}

				System.out.println("exported");

				// saving for the injector shouldn't change the isModified field
				// mainPanel.isModified = false;
			} catch (IOException except) {
				except.printStackTrace();
			}

		/*
		 * re-enable the next and previous buttons
		 */
		setButtonPermissions();

		// Set the message counter
		setStatus(getIndexOfDisplayed() + 1);
	}

	private void writeBrokenXMLMessages(List messages, FileWriter writer)
			throws IOException {
		/*
		 * StringBuffer xml = new StringBuffer(); xml.append("<emails>\n");
		 * for(int i = 0; i < messages.size(); i++) {
		 * xml.append(((CorpusGUIMessage
		 * )getDataList().get(i)).getBrokenXML(getSession())); }
		 * xml.append("</emails>\n"); return xml.toString();
		 */
		// outputs the xml with Dom4J
		Document document = DocumentHelper.createDocument();
		Element emailsNode = document.addElement("emails");

		for (int i = 0; i < messages.size(); i++) {
			((CorpusGUIMessage) getDataList().get(i)).getBrokenXML(
					getSession(), emailsNode);
		}

		XMLWriter xmlWriter = new XMLWriter(writer, OutputFormat
				.createPrettyPrint());
		xmlWriter.write(document);
	}

	/**
	 * Exports the variables to a file for submission into the injector
	 * 
	 * @param saveHere
	 *            the path to save the variables file in
	 */
	private void exportVariables(String saveHere) {
		FileWriter writer = null;
		try {
			writer = new FileWriter(saveHere + ".variables.xml");

			Document document = DocumentHelper.createDocument();
			Element varsNode = document.addElement("variables");
			for (int i = 0, size = variableModel.size(); i < size; i++) {
				edu.cmu.radar.base.corpus.Variable writing = (edu.cmu.radar.base.corpus.Variable) variableModel
						.get(i);
				Element varNode = varsNode.addElement("variable");
				varNode.addAttribute(writing.getName(), writing.getValue());
			}

			XMLWriter xmlWriter = new XMLWriter(writer, OutputFormat
					.createPrettyPrint());
			xmlWriter.write(document);

			/*
			 * edu.cmu.radar.base.corpus.Variable writing;
			 * variableOut.write("<variables>\n"); for (int i = 0; i <
			 * variableModel.size(); i++) { writing =
			 * (edu.cmu.radar.base.corpus.Variable) variableModel .get(i);
			 * variableOut.write("<variable " + writing.getName() + "=\"" +
			 * writing.getValue() + "\"/>\n"); }
			 * variableOut.write("</variables>"); variableOut.close();
			 */
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (writer != null) {
				try {
					writer.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

	}

	/**
	 * This function is used to store the state of the GUI so that it can be
	 * reverted later to this saved state. Inherited from the StateEditable
	 * interface
	 * 
	 * @param state
	 *            a hashtable to store the state in
	 */
	public void storeState(final Hashtable state) {
		if (currentEdit.equals(MESSAGEEDIT)) {
			state.put(currentEdit, new Integer(getIndexOfDisplayed()));
			state.put(DATA, this.getDataList());
		} else if (currentEdit.equals(RANDOMEDIT)) {
			final ArrayList messageCopy = new ArrayList();

			for (int i = 0; i < getDataList().size(); i++) {
				messageCopy.add((getDataList().get(i)));
			}

			state.put(currentEdit, messageCopy);
		}
		/*
		 * Unknown if this is needed for undoManager, uncomment if undoManager
		 * stops working else if(currentEdit.equals(OPENDATABASEEDIT)) {
		 * ArrayList messageCopy = new ArrayList();
		 * 
		 * if (getDataList() != null) { for (int i=0; i < getDataList().size();
		 * i++) { messageCopy.add(((FakeMessage)getDataList().get(i)).copy()); }
		 * 
		 * if (!messageCopy.isEmpty()) state.put(currentEdit, messageCopy);
		 * if(loadedFilename != null) state.put("filename", loadedFilename); }
		 * // if the messages ArrayList is null, don't add anything to the list
		 * with the currentEdit key state.put("messageOn", new
		 * Integer(getIndexOfDisplayed())); }
		 */
	}

	/**
	 * This function is used to restore the GUI to a previous state
	 * 
	 * @param state
	 *            the state to restore the GUI to
	 */
	public void restoreState(final Hashtable state) {
		final Object edit = state.get(currentEdit);
		mainPanel.removeUndoListeners(undoableEditListener);
		if (currentEdit.equals(MESSAGEEDIT)) {
			if (edit != null) {
				this.setDataList((List) state.get(DATA));
				setIndexOfDisplayed(((Integer) edit).intValue());
				fillInText((CorpusGUIMessage) getDataList().get(
						getIndexOfDisplayed()));
				setStatus(getIndexOfDisplayed() + 1);
			}
		} else if (currentEdit.equals(RANDOMEDIT)) {
			setDataList((ArrayList) edit);
			fillInText((CorpusGUIMessage) getDataList().get(
					getIndexOfDisplayed()));
		} else if (currentEdit.equals(OPENDATABASEEDIT)) {
			if (edit == null) {
				setIndexOfDisplayed(0);
				setStatus(0);
				makeBlank(-1);
				redoMenu.setEnabled(false);
			} else {
				setDataList((ArrayList) edit);
				setIndexOfDisplayed(((Integer) state.get("messageOn"))
						.intValue());
				setStatus(getIndexOfDisplayed() + 1);
				fillInText((CorpusGUIMessage) getDataList().get(
						getIndexOfDisplayed()));
			}
			setButtonPermissions();
		}
		mainPanel.addUndoListeners(undoableEditListener);
	}

	/**
	 * Switches the current message to the message atLabel index messageNum
	 * 
	 * @param messageNum
	 *            the index of the message wished to be switched to.
	 */
	public void gotoNumber(int messageNum) {
		currentEdit = CorpusMainWindow.MESSAGEEDIT;
		mainPanel.removeUndoListeners(undoableEditListener);
		undoMenu.setEnabled(true);
		currentState = new StateEdit(this);
		// modifiedMessages.add(new Integer(getIndexOfDisplayed()));

		if (getDataList() != null && (messageNum < getDataList().size())) {
			setIndexOfDisplayed(messageNum);

			CorpusGUIMessage current = (CorpusGUIMessage) getDataList().get(
					getIndexOfDisplayed());
			fillInText(current); // load the new e-mail read
			// in into the
			// text fields
		}

		setButtonPermissions();
		currentState.end();
		undoManager.addEdit(currentState);
		mainPanel.addUndoListeners(undoableEditListener);
	}

	/**
	 * Reads the data out of the database, using the session that was passed to
	 * the constructor.
	 * 
	 */
	private void readDataFromDB() {
		/*
		 * if (mainPanel.isModified) { // only uncomment if there are going to
		 * be multiple databases open atLabel once save(false); }
		 */
		undoMenu.setEnabled(true);
		mainPanel.removeUndoListeners(undoableEditListener);

		currentEdit = OPENDATABASEEDIT; // undoing the loading of data from the
		// database
		// System.out.println("Database edit");

		currentState = new StateEdit(CorpusMainWindow.this, currentEdit);

		// if (fd.getFile() != null)
		{
			setIndexOfDisplayed(0); // reset the message
			// count when a new
			// file is loaded

			try {
				ArrayList rawMessages = new ArrayList(load(getSession(),
						FakeMessage.class, "messageId"));
				setDataList(new ArrayList());
				CorpusGUIMessage tempCGM;
				for (int i = 0; i < rawMessages.size(); i++) {
					tempCGM = new CorpusGUIMessage((FakeMessage) rawMessages
							.get(i));
					getDataList().add(tempCGM);
				}

				// load the messages from the file into
				// a ArrayList
				/*
				 * final CorpusGUIMessage current =
				 * ((CorpusGUIMessage)getDataList()
				 * .get(getIndexOfDisplayed()));
				 */

				getMaxId(); // save the maxID of all the e-mails just read in
			} catch (Exception ex) {
				ex.printStackTrace();
			}

			currentState.end(); // handle undoManager-ing of this action
			undoManager.addEdit(currentState);
			mainPanel.addUndoListeners(undoableEditListener);
		}
	}

	/**
	 * This function creates/initializes the implementing JPanel, and returns it
	 * to be added to the JTabbedPane
	 * 
	 * @return the JPanel to be added to the JTabPane.
	 */
	public JPanel getPane() {
		GridBagConstraints gbc = new GridBagConstraints();
		this.setLayout(new GridBagLayout());
		this.setName("Corpus");

		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.BOTH;
		this.add(mainPanel, gbc);

		gbc.gridy = 1;
		gbc.weighty = 0;
		gbc.weightx = 1;
		gbc.insets = new Insets(0, 0, 12, 0);
		this.add(getButtonPanel(), gbc);
		// add the button pannel to the main window

		return this;
	}

	/**
	 * This sets the progress text field.
	 * 
	 * @param numOn
	 *            the number of the message we're on
	 */
	public void setStatus(int numOn) {
		// System.out.println("setting status to " + numOn);
		statusLabel.setText("On message " + numOn + " of "
				+ getDataList().size());
	}

	/**
	 * Loads the data from the session if it isn't loaded already, and fils in
	 * the text for the next message
	 * 
	 * @param session
	 *            The session to possibly load from
	 * 
	 */
	public void fillInText(Session session) {
		if (getDataList() == null || getDataList().isEmpty()) {
			setSession(session);
			getSession().beginTransaction();
			readDataFromDB();
		}
		mainPanel.removeUndoListeners(undoableEditListener);
		fillInText(((CorpusGUIMessage) getDataList().get(getIndexOfDisplayed())));
		mainPanel.addUndoListeners(undoableEditListener);
	}

	public void fillInText(Object object) {
		if (object instanceof CorpusGUIMessage) {
			fillInText((CorpusGUIMessage) object);
		}
	}

	/**
	 * This affects what is shown in the program's menu bar, setting up all
	 * necessary listeners and menu items.
	 * 
	 * @param menuBar
	 *            The window's menu bar
	 * 
	 */
	public void setMenuBar(final TabbedPanelMenu menuBar) {
		/* Start Menu code here */
		/*
		 * Initializes the titlebar menu, key shortcuts, and the actions that
		 * occur when a menu item is selected
		 */
		final JMenuItem save = menuBar.getSave();
		final JMenuItem cut = menuBar.getCut();
		final JMenuItem copy = menuBar.getCopy();
		final JMenuItem newMenu = menuBar.getNewMenu();
		final JMenuItem delete = menuBar.getDelete();
		next = menuBar.getNext();
		prev = menuBar.getPrev();
		undoMenu = menuBar.getUndo();
		redoMenu = menuBar.getRedo();
		paste = menuBar.getPaste();
		advancedMenu.removeAll();
		export.removeActionListener(exportListener);
		pref.removeActionListener(prefListener);
		vars.removeActionListener(varsListener);
		randomTimes.removeActionListener(randomListener);
		defaultTimes.removeActionListener(defaultTimeListener);

		final int keyMask = menuBar.keyMask;

		// register what happens when this menu item is selected
		save.addActionListener(saveListener);

		/*
		 * to transform the format to such that the injector won't choke
		 */
		export.addActionListener(exportListener);

		// register what happens when this menu item is selected
		next.addActionListener(nextListener);

		// register what happens when this menu item is selected
		prev.addActionListener(prevListener);

		// register what happens when this menu item is selected
		newMenu.addActionListener(newListener);

		// register what happens when this menu item is selected
		delete.addActionListener(deleteListener);

		// set up hotkey shortcut for this menu item
		pref.setAccelerator(KeyStroke.getKeyStroke((KeyEvent.VK_P),
				(ActionEvent.ALT_MASK | keyMask)));

		// register what happens when this menu item is selected
		pref.addActionListener(prefListener);

		// set up hotkey shortcut for this menu item
		vars.setAccelerator(KeyStroke.getKeyStroke((KeyEvent.VK_V),
				(ActionEvent.ALT_MASK | keyMask)));

		// register what happens when this menu item is selected
		vars.addActionListener(varsListener);

		// register what happens when this menu item is selected
		undoMenu.addActionListener(undoListener);

		// register what happens when this menu item is selected
		redoMenu.addActionListener(redoListener);

		// register what happens when this menu item is selected
		cut.addActionListener(cutListener);

		// register what happens when this menu item is selected
		copy.addActionListener(copyListener);

		// register what happens when this menu item is selected
		paste.addActionListener(pasteListener);

		// set up hitkey shortcut for this menu item
		randomTimes.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
				keyMask));
		// register what happens when this menu item is selected
		randomTimes.addActionListener(randomListener);

		// set up hotkey shortcut for this menu item
		defaultTimes.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
				keyMask));
		// register what happens when this menu item is selected
		defaultTimes.addActionListener(defaultTimeListener);

		save.setEnabled(true);
		undoMenu.setEnabled(false);
		redoMenu.setEnabled(false);
		cut.setEnabled(true);
		copy.setEnabled(true);
		paste.setEnabled(false);
		prev.setEnabled(false); // disable the prev menu Item until a file is
		// loaded or a new message is created

		next.setEnabled(false); // disable the next menu item until a file is
		// loaded

		// add the appropriate variables to the file menu
		menuBar.file.addSeparator();
		menuBar.file.add(export);

		// add the appropriate variables to the utility menu
		menuBar.util.addSeparator();
		menuBar.util.add(pref);
		menuBar.util.add(vars);

		// add the appropriate variables to the advanced menu
		advancedMenu.add(randomTimes);
		advancedMenu.add(defaultTimes);

		// add all the panel-specific menus to the menu bar
		// menuBar.add(messMenu);
		menuBar.add(advancedMenu);

		/* End Menu Code */
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * airspaceGUI.base.BasePanel#addModificationListeners(javax.swing.event
	 * .DocumentListener, java.awt.event.ItemListener,
	 * javax.swing.event.ChangeListener)
	 */
	protected void addModificationListeners(
			DocumentListener docModificationListener,
			ItemListener comboBoxModificationListener,
			ChangeListener checkBoxModificationListener) {
		mainPanel.addModificationListeners(docModificationListener,
				comboBoxModificationListener, checkBoxModificationListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * airspaceGUI.base.BasePanel#addUndoableEditListeners(javax.swing.event
	 * .UndoableEditListener)
	 */
	protected void addUndoableEditListeners(UndoableEditListener undoListener) {
		mainPanel.addUndoListeners(undoListener);
	}

	public FindWindow getFindWindow() {
		findWindow.updateSearchList(getDataList());
		return findWindow;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#clearPanel()
	 */
	public void clearPanel() {

	}

}