package airspaceGUI.base.corpus.gui;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Iterator;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

import airspaceGUI.base.corpus.CorpusGUIMessage;
import edu.cmu.radar.base.corpus.FakeMessageClass;

public class ExportOptions extends JDialog
{
    /**
     * the label for the messageclass check boxes
     */
    private JLabel messageClassLabel = new JLabel(
            "Export Which Message Classes?");

    /**
     * the message class check boxes
     */
    private JCheckBox[] messageClasses;

    /**
     * the list of message classes that are listed in the check boxes
     */
    private List messageClassList;

    /**
     * export label
     */
    private JLabel exportWhichLabel = new JLabel("Export which files?");

    /**
     * checkbox to check if the backstory messages should be exported
     */
    private JCheckBox backstoryCheck = new JCheckBox("Backstory Message File");

    /**
     * checkbox to check if the normal messages should be exported
     */
    private JCheckBox normalCheck = new JCheckBox(
            "Normal and Noise Message File");

    /**
     * checkbox to check if the variable messages should be exported
     */
    private JCheckBox variableCheck = new JCheckBox("Variables File");

    /**
     * go to the next panel in the exporting process
     */
    private JButton nextButton = new JButton("Next");

    /**
     * listener to configure what the nextButton does
     */
    private ActionListener nextListener = new ActionListener()
    {

        public void actionPerformed(ActionEvent arg0) {
            ExportOptions.this.setVisible(false);
        }

    };

    /**
     * Initalizes the ExportOptions dialog box
     * @param owner the owner of this dialog
     * @param session the session to read any necessary data out of
     */
    public ExportOptions(JFrame owner, Session session)
    {
        super(owner, "Export Files for the Injector", true);
        messageClassList = session.createCriteria(FakeMessageClass.class)
                .list();
        this.messageClasses = new JCheckBox[messageClassList.size()];
        // initialize the check boxes needed to choose what type of messages to
        // save
        // also need to be able to choose what filename to export the files to
        // need to export backstory, normal, and variables files

        for (int i = 0; i < messageClassList.size(); i++)
        {
            this.messageClasses[i] = new JCheckBox(
                    ((FakeMessageClass)messageClassList.get(i))
                            .getMessageClass());
        }
        
        layoutWindow();
        this.setVisible(true);
    }

    /**
     * Layout the options for exporting the corpus to file(s)
     * 
     */
    private void layoutWindow() {
        Container panel = this.getContentPane();
        GridBagConstraints gbc = new GridBagConstraints();
        Insets tabInset = new Insets(4, 20, 0, 4);
        Insets labelInset = new Insets(10, 0, 4, 0);
        int y = 0;
        panel.setLayout(new GridBagLayout());

        gbc.gridy = y++;
        gbc.gridwidth = 2;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.BOTH;
        panel.add(messageClassLabel, gbc);

        gbc.gridx = 1;
        gbc.gridwidth = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = tabInset;
        for (int i = 0; i < messageClasses.length; i++)
        {
            // add all the checkboxes
            gbc.gridy = y++;
            panel.add(messageClasses[i], gbc);
        }

        y += 2;
        gbc.gridx = 0;
        gbc.gridy = y;
        gbc.gridwidth = 2;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets = labelInset;
        panel.add(exportWhichLabel, gbc);

        // add more check boxes
        gbc.gridx = 1;
        gbc.gridwidth = 1;
        gbc.gridy = ++y;
        gbc.insets = tabInset;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        panel.add(backstoryCheck, gbc);
        gbc.gridy = ++y;
        panel.add(normalCheck, gbc);
        gbc.gridy = ++y;
        panel.add(variableCheck, gbc);
        gbc.gridy = ++y;
        gbc.gridx = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.insets = new Insets(0, 0, 0, 0);
        nextButton.addActionListener(nextListener);
        panel.add(nextButton, gbc);

        this.setResizable(false);

        this.pack();

    }

    /**
     * Sets up the test environment for this class by initiating a Hibernate
     * SessionFactory based on the default configuration (the file
     * "corpus.hibernate.xml" on the classpath.
     * 
     * @return a Hibernate session object
     */
    protected final static Session setUp() {
        try
        {
            // Create the SessionFactory from hibernate.cfg.xml

            final File configFile = new File("corpus.hibernate.xml");
            final Configuration config = new Configuration()
                    .configure(configFile);

            SessionFactory sessionFactory = config.buildSessionFactory();
            return sessionFactory.openSession();
        } catch (Exception ex)
        {
            // Make sure you log the exception, as it might be swallowed
            System.err.println("Initial SessionFactory creation failed." + ex);
            throw new ExceptionInInitializerError(ex);
        }
    }

    /**
     * Returns true if the message is of the correct messageClass
     * 
     * @param message
     *            the message to test
     * @return true if the message is of the correct messageClass, false
     *         otherwise
     */
    public boolean isWantedMessageClass(CorpusGUIMessage message) {
        Iterator messageClasses = message.getMessageClass().iterator();

        for (int i = 0; i < message.getMessageClass().size(); i++)
        {
            if (this.messageClasses[messageClassList.indexOf(messageClasses
                    .next())].isSelected())
            {
                return true;
            }
        }
        return false;
    }

    /**
     * 
     * @return true if the backstory is to be exported
     */
    public boolean exportBackstory() {
        return backstoryCheck.isSelected();
    }

    /**
     * 
     * @return true if the variables are to be exported
     */
    public boolean exportVariables() {
        return variableCheck.isSelected();
    }

    /**
     * 
     * @return true if the normal corpus messages are to be exported
     */
    public boolean exportNormal() {
        return normalCheck.isSelected();
    }

}
