package airspaceGUI.base.corpus.gui;

import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;

import airspaceGUI.base.corpus.CorpusGUIMessage;
import airspaceGUI.expansion.FindWindow;

import edu.cmu.radar.base.corpus.FakeMessageClass;

public class FindCorpus extends FindWindow
{
    private static final String SUBJECT = "Subject";

    private static final String FROM = "From";

    private static final String MESSAGEID = "Message id";

    private static final String BODY = "Body";

    private static final String MESSAGECLASS = "Message Class";

    private List searchList;

    private CorpusMainWindow corpusPanel;

    private static Object[] keywordList = {MESSAGEID, MESSAGECLASS, FROM,
            SUBJECT, BODY};

    /**
     * 
     * @param corpusPanel
     *            The panel that initiated this obeject
     * @param searchList
     *            the list to search over
     */
    public FindCorpus(CorpusMainWindow corpusPanel, List searchList)
    {
        super(keywordList);
        // setTitle("Find a Message");
        // setLocationRelativeTo(corpusPanel);
        this.searchList = searchList;
        this.corpusPanel = corpusPanel;
        parent = corpusPanel;
    }

    /**
     * Creates a findCorpus window
     * 
     * @param mainPanel
     *            the panel that initiated this object
     * @param dataList
     *            the list to search over
     * @param tab
     *            the tabbedPanel to pass to the superclass
     */
    public FindCorpus(CorpusMainWindow mainPanel, List dataList, JTabbedPane tab)
    {
        super(keywordList, tab);
        this.searchList = dataList;
        this.corpusPanel = mainPanel;
        parent = corpusPanel;
    }

    /*
     * (non-Javadoc)
     * 
     * @see airspaceGUI.expansion.FindWindow#updatePanel(java.lang.Object)
     */
    protected void updatePanel(Object object) {
        try
        {
            CorpusGUIMessage message = (CorpusGUIMessage)object;
            corpusPanel.fillInText(message);
            if (tabs != null)
            {
                tabs.setSelectedComponent(corpusPanel);
            }
        } catch (ClassCastException e)
        {
            e.printStackTrace();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see airspaceGUI.expansion.FindWindow#search(java.util.Map)
     */
    protected ArrayList search(Map searchTerms) {
        ArrayList results = new ArrayList();
        CorpusGUIMessage messageSearching;
        Integer searchedMessageId = null;
        final String searchedFrom = ((String)searchTerms.get(FROM));
        final String lowerCaseSearchedFrom = (searchedFrom == null ? null
                : searchedFrom.toLowerCase());
        final String searchedSubject = ((String)searchTerms.get(SUBJECT));
        final String lowerCaseSearchedSubject = (searchedSubject == null ? null
                : searchedSubject.toLowerCase());
        final String searchedBody = ((String)searchTerms.get(BODY));
        final String lowerCaseSearchedBody = (searchedBody == null ? null
                : searchedBody.toLowerCase());
        final String searchedMessageClass = ((String)searchTerms
                .get(MESSAGECLASS));
        final String lowerCaseSearchedMessageClass = (searchedMessageClass == null ? null
                : searchedMessageClass.toLowerCase());
        boolean isResult = true;

        try
        {
            // make sure that this doesn't throw an error if the user isn't
            // looking for a messageid
            searchedMessageId = (searchTerms.get(MESSAGEID) == null ? null
                    : new Integer((String)searchTerms.get(MESSAGEID)));
        } catch (NumberFormatException nfe)
        {
            // put the information message into the main thread so it'll
            // actually show
            SwingUtilities.invokeLater(new Thread()
            {
                public void run() {
                    JOptionPane
                            .showMessageDialog(
                                    null,
                                    "The Message id you are searching for is not an Integer, please supply an integer",
                                    "Note", JOptionPane.ERROR_MESSAGE);
                }
            });
            return null;
        }

        Iterator messageClasses;

        for (int i = 0; i < searchList.size(); i++)
        {
            messageSearching = (CorpusGUIMessage)searchList.get(i);
            messageClasses = messageSearching.getMessageClass().iterator();
            if (searchedMessageId != null
                    && !searchedMessageId.equals("")
                    && !messageSearching.getMessageId().equals(
                            searchedMessageId))
            {
                isResult = false;
            }

            if (lowerCaseSearchedFrom != null
                    && !(messageSearching.getFrom()).toLowerCase().contains(
                            lowerCaseSearchedFrom))
            {
                isResult = false;
            }

            if (lowerCaseSearchedSubject != null
                    && !(messageSearching).getSubject().toLowerCase().contains(
                            lowerCaseSearchedSubject))
            {
                isResult = false;
            }

            if (lowerCaseSearchedBody != null
                    && !(messageSearching).getBody().toLowerCase().contains(
                            lowerCaseSearchedBody))
            {
                isResult = false;
            }
            String messageClass;
            while (messageClasses.hasNext())
            {
                messageClass = ((FakeMessageClass)messageClasses.next())
                        .getMessageClass().toLowerCase();
                if (lowerCaseSearchedMessageClass != null
                        && !messageClass
                                .contains(lowerCaseSearchedMessageClass))
                {
                    isResult = false;
                }
            }

            if (isResult)
            {
                results.add(searchList.get(i));
            }
            // reset the result flag
            isResult = true;
        }

        return results;
    }

    /**
     * This is used to specify a title to display in the JList, used when the 
     * object doesn't have a toString function, or the developer wants to display
     * something other than what the toString function does.
     * 
     * @param list the list these objects are being displayed in
     * @param value the value to display
     * @param index the index of the cell this object is supposed to be in
     * @param isSelected tells whether this cell is selected or not
     * @param cellHasFocus tells whether this cell has focus
     * @return the label to display in the list
     * 
     */
    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList,
     *      java.lang.Object, int, boolean, boolean)
     */
    public Component getListCellRendererComponent(JList list, Object value, // value
            // to
            // display
            int index, // cell index
            boolean isSelected, // is the cell selected
            boolean cellHasFocus) {
        String labelText;
        JLabel label;

        CorpusGUIMessage cgm = (CorpusGUIMessage)value;
        labelText = ("MessageId: " + cgm.getMessageId() + "\n MessageClass:"
                + cgm.getMessageClass() + "\n From: " + cgm.getFrom()
                + "\n Subject:" + cgm.getSubject());
        /* } */

        label = new JLabel(labelText);

        if (isSelected)
        {
            label.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY));
            label.setOpaque(true);
            label.setBackground(list.getSelectionBackground());
            label.setForeground(list.getSelectionForeground());
        } else
        {
            label.setBorder(BorderFactory
                    .createLineBorder(list.getBackground()));
            label.setOpaque(false);
            label.setBackground(list.getBackground());
            label.setForeground(list.getForeground());
        }
        label.setEnabled(list.isEnabled());
        label.setFont(list.getFont());

        return label;
    }

    /**
     * Updates the search list with the list in the parameter
     * 
     * @param search
     *            the new list to search over
     */
    public void updateSearchList(List search) {
        searchList = search;
    }

}
