/**
 * 
 */
package airspaceGUI.base.corpus.gui;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * This class is used to explicitly add a new MessageClass to the list of known
 * messageClasses. It is a simple modal dialog that stores the new messageClass
 * for retrieval by the calling object
 * 
 * @author srbennet
 * 
 */
public class NewMessageClassWindow extends JDialog
{
    /**
     * The messageClass field into which the user types the new MessageClass
     */
    private JTextField messageClassField = new JTextField();

    /**
     * Done button action. This dismisses the window, preserving what the user
     * typed
     */
    private ActionListener doneListener = new ActionListener()
    {

        public void actionPerformed(ActionEvent arg0) {
            NewMessageClassWindow.this.dispose();
        }

    };

    /**
     * This is the cancel listener, it is execuited when the used clicks on the
     * close button, clears out the text field since the user changed their
     * mind.
     */
    private ActionListener cancelListener = new ActionListener()
    {

        public void actionPerformed(ActionEvent arg0) {
            // clear out the text since the user changed their mind
            messageClassField.setText("");
            NewMessageClassWindow.this.dispose();
        }

    };

    /**
     * @param arg0
     *            The Dialog this should be on top of
     * @param arg1
     *            true if this window is modal, false otherwise
     * @throws HeadlessException
     */
    public NewMessageClassWindow(Frame arg0, boolean arg1)
            throws HeadlessException
    {
        super(arg0, "New Message Class", arg1);
        this.setLocationRelativeTo(arg0);
        this.setLocationByPlatform(true);
        setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        JLabel messageClassLabel = new JLabel("Message Class");
        JButton okButton = new JButton("Done");
        JButton cancelButton = new JButton("Cancel");
        okButton.addActionListener(doneListener);
        cancelButton.addActionListener(cancelListener);
        getRootPane().setDefaultButton(okButton);

        gbc.insets = new Insets(8, 8, 8, 8);
        add(messageClassLabel, gbc);
        gbc.gridx = 1;
        gbc.gridwidth = 3;
        gbc.fill = GridBagConstraints.BOTH;
        add(messageClassField, gbc);

        gbc.gridy = 1;
        gbc.gridx = 2;
        gbc.gridwidth = 1;
        gbc.insets = new Insets(8, 0, 8, 0);
        gbc.fill = GridBagConstraints.NONE;
        add(cancelButton, gbc);
        gbc.gridx = 3;
        add(okButton, gbc);

        this.setResizable(false);
        this.pack();
    }

    /**
     * @param arg0
     *            The Dialog this should be on top of
     * @param arg1
     *            true if this window is modal, false otherwise
     * @throws HeadlessException
     */
    public NewMessageClassWindow(Dialog arg0, boolean arg1)
            throws HeadlessException
    {
        super(arg0, arg1);
        // TODO Auto-generated constructor stub
    }

    /**
     * Used to retrieve the name of the new MessageClass that the user entered
     * into this window
     * 
     * @return The name of the new messageclass the user just created
     */
    public String getNewMessageClass() {
        return messageClassField.getText();
    }

}
