package airspaceGUI.base.corpus.gui;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Random;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import airspaceGUI.base.corpus.CorpusGUIMessage;


/**
 * A class to randomize the times, and possibly orders of Messages. This also
 * includes the gui that is shown whewn the user wants to execute randomizations
 * 
 * @author srbennet
 * 
 */
public class RandomWindow extends JDialog
{

    private static final long serialVersionUID = 1L;

    private transient int maxTime = 1;

    private transient ArrayList currentMessages = null; // the list of messages

    // to be randomized

    // initialize checkboxes
    private final transient JCheckBox injectionCheck = new JCheckBox(
            "Injection");

    private final transient JCheckBox backstoryCheck = new JCheckBox(
            "Backstory");

    private final transient JCheckBox reorderCheck = new JCheckBox(
            "Reorder Messages");

    // labels for the fields
    private transient final JLabel timeLabel = new JLabel("Max Time:");

    private transient final JLabel typeLabel = new JLabel("FakeMessage Type:");

    // field for a time
    private final transient JTextField timeField = new JTextField();

    private final transient JButton doneButton = new JButton("Randomize");
    

    /**
     * Initializes the window that is needed to set various parameters of the
     * randomizing process
     * 
     * @param messages
     *            the messages to be randomized
     */
    public RandomWindow(ArrayList messages)
    {
        super();
        dialogInit();
        GridBagConstraints gbc = new GridBagConstraints();
        Container panel = this.getContentPane();
        panel.setLayout(new GridBagLayout());
        this.setResizable(false);
        this.setModal(true);
        this.setTitle("Randomize Message Times");

        this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        
        /*
         * Set up the Constraints to layout the window correctly
         */
        gbc.insets = new Insets(8, 8, 8, 8);
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.EAST;

        /*
         * add the injection/backstory checkboxes to the window in order
         */
        gbc.gridx = 0;
        gbc.gridy = 0;
        panel.add(typeLabel, gbc);

        gbc.gridx = 1;
        panel.add(injectionCheck, gbc);

        gbc.gridx = 2;
        backstoryCheck.setEnabled(false);
        panel.add(backstoryCheck, gbc);

        /*
         * add in the time label and field to the interface below the checkboxes
         */
        gbc.gridy = 1;
        gbc.gridx = 0;
        panel.add(timeLabel, gbc);

        gbc.gridx = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.BOTH;
        panel.add(timeField, gbc);

        /*
         * add in the reorder checkbox
         */
        gbc.gridx = 1;
        gbc.gridy = 2;
        gbc.gridwidth = 2;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor = GridBagConstraints.EAST;
        panel.add(reorderCheck, gbc);

        /*
         * add in the done button on the window, to start the randomizing
         * process also add a listener so it does something when the button is
         * clicked.
         */
        gbc.gridy = 3;
        gbc.gridx = 2;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.insets = new Insets(5, 0, 5, 5);
        gbc.fill = GridBagConstraints.NONE;
        panel.add(doneButton, gbc);

        doneButton.addActionListener(new ActionListener()
        {

            public void actionPerformed(final ActionEvent click) {
                // TODO Auto-generated method stub
                if (injectionCheck.getSelectedObjects() != null)
                {
                    try
                    {
                        maxTime = (new Integer(timeField.getText())).intValue();
                        if (maxTime < 1)
                        {
                            throw new Exception();
                        }
                    } catch (Exception e)
                    {
                        JOptionPane
                                .showMessageDialog(
                                        RandomWindow.this,
                                        "The max time is not valid. /n "
                                                + "Please enter a number greater than 1",
                                        "Invalid Field",
                                        JOptionPane.WARNING_MESSAGE);
                    }

                    if (reorderCheck.getSelectedObjects() == null) // The
                    // checkbox
                    // has not
                    // been
                    // checked
                    /*
                     * if the reorder checkbox has not been checked, get and
                     * sort the random numbers and assign them to the messages
                     * in order
                     */
                    {
                        sortTimes();
                    } else
                    /*
                     * if the reorder checkbox has been checked, then assign
                     * numbers and reorder the messages appropriately
                     */
                    {
                        sortMessages();
                    }
                } else
                {
                    JOptionPane.showMessageDialog(RandomWindow.this,
                            "Please select a FakeMessage Type", "Error",
                            JOptionPane.INFORMATION_MESSAGE);
                }
            }

        });

        this.currentMessages = messages;
        this.setSize(350, 200);
    }

    /**
     * Assigns new random times to the messages in the ArrayList passed into the
     * constructor of this object. This will not change the order of the
     * messages in the ArrayList
     */
    protected void sortTimes() {
        final Random randNum = new Random();
        final ArrayList times = new ArrayList();
        Integer temp;

        for (int i = 0; i < currentMessages.size(); i++)
        {
            temp = new Integer(randNum.nextInt(maxTime) + 1);
            times.add(temp);
            // System.out.println(temp);
        }
        Collections.sort(times);

        for (int i = 0; i < currentMessages.size(); i++)
        {
            final CorpusGUIMessage current = (CorpusGUIMessage)currentMessages.get(i);
            String holder, newTime = new String();
            final StringTokenizer st = new StringTokenizer(current.getTime());
            holder = st.nextToken();

            while (st.hasMoreTokens())
            {
                newTime += holder + " ";
                holder = st.nextToken();
            }
            newTime += times.get(i);
            current.setTime(newTime);
        }
        RandomWindow.this.setVisible(false);
    }

    /**
     * Assigns new random times to the messages in the ArrayList passed into the
     * constructor of this object. This will change the order of the messages in
     * the ArrayList
     */
    protected void sortMessages() {
        final ArrayList newMessages = new ArrayList(); // temprary
        // datastructure to hold
        // messages to be sorted
        final Random randNum = new Random();
        Integer temp;

        for (int i = 0; i < currentMessages.size(); i++)
        {
            final CorpusGUIMessage current = (CorpusGUIMessage)currentMessages.get(i);
            String holder, newTime = new String();
            final StringTokenizer st = new StringTokenizer(current.getTime());

            temp = new Integer(randNum.nextInt(maxTime) + 1);
            holder = st.nextToken();

            while (st.hasMoreTokens())
            {
                newTime += holder + " ";
                holder = st.nextToken();
            }
            newTime += temp;
            current.setTime(newTime);
            newMessages.add(new MessageKeys(temp, current));
        }
        Collections.sort(newMessages, new MessageKeys());
        currentMessages = new ArrayList();
        for (int i = 0; i < newMessages.size(); i++)
        {
            currentMessages.add(i, ((MessageKeys)newMessages.get(i)).message);
        }
        RandomWindow.this.setVisible(false);
    }

    /**
     * Sets the message times for all the messages in the ArrayList to the
     * default of 1
     * 
     * @param messages
     *            a list of messages whose times need to be reset to the default
     * @return the list of messages with their times reset
     */
    public static ArrayList setToDefault(final ArrayList messages) {
        final ArrayList temp = new ArrayList();

        for (int i = 0; i < messages.size(); i++)
        {
            final CorpusGUIMessage current = (CorpusGUIMessage)messages.get(i);
            String holder, newTime = new String();
            final StringTokenizer st = new StringTokenizer(current.getTime());
            holder = st.nextToken();
            while (st.hasMoreTokens())
            {
                newTime += holder + " ";
                holder = st.nextToken();
            }
            newTime += "1";
            current.setTime(newTime);
            temp.add(current);
        }
        return temp;
    }

    /**
     * Returns the current list of messages
     * 
     * @return the current list of messages, may not be the same as the list
     *         that was passed in
     */
    public ArrayList getMessages() {
        return currentMessages;
    }

    /***************************************************************************
     * /
     * 
     * @param args
     */
    /*
     * public static void main(String[] args) { // TODO Auto-generated method
     * stub RandomWindow rw = new RandomWindow(null); rw.show(); }
     */

    /**
     * A class constructed to enable the sorting of messages into a random order
     * based on the key
     */
    private class MessageKeys implements Comparator
    {
        protected transient Integer key;

        private transient CorpusGUIMessage message;

        /**
         * Constructor
         * 
         * @param key -
         *            a key for these messages to be sorted by
         * @param message -
         *            the message the key should eb associated to
         */
        MessageKeys(Integer key, CorpusGUIMessage message)
        {
            super();
            this.key = key;
            this.message = message;
        }

        /**
         * Default constructor
         * 
         */
        public MessageKeys()
        {
            super();
        }

        /**
         * Function to implement the comparable interface Defines a "natural
         * ordering" based on the keys
         * 
         * @param k1
         *            one object to compare
         * @param k2
         *            the other object to compare
         * @return an integer measure of how different the two objects are
         */
        public int compare(final Object k1, final Object k2) {
            if (k1 instanceof MessageKeys && k2 instanceof MessageKeys)
            {
                return ((MessageKeys)k1).key.intValue()
                        - ((MessageKeys)k2).key.intValue();
            } else
            {
                return -1;
            }
        }

    }
}
