package airspaceGUI.base.event;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;

import org.hibernate.Session;

import airspaceGUI.base.SubPanel;
import airspaceGUI.expansion.ConstraintFactory;
import airspaceGUI.expansion.FakeComboBox;
import airspaceGUI.expansion.TimePanel;
import airspaceGUI.model.MeetingModel;
import airspaceGUI.model.RoomModel;
import edu.cmu.radar.base.Location;
import edu.cmu.radar.base.cmradar.ConcreteMeeting;

public class ConcreteMeetingPanel extends SubPanel {
	private FakeComboBox locationField;

	private TimePanel dateField = new TimePanel();

	private List locationList;

	private AbstractMeetingPanel absMeetingPane;

	// Listener to get adding a new Location events
	private ActionListener locationListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			int i = locationList.indexOf(getLocation(e.getActionCommand()));

			locationField
					.setText(((Location) locationList.get(i)).getCaption());
		}
	};

	public ConcreteMeetingPanel(Session session, RoomModel roomModel,
			MeetingModel eventModel) {
		absMeetingPane = new AbstractMeetingPanel(session, eventModel);

		locationList = roomModel;
		setDataList(eventModel);
		loadSupports(session);
		setSession(session);
	}

	/**
	 * Loads all the supporting data for the panel
	 * 
	 * @param session
	 *            The sesssion to load from
	 */
	private void loadSupports(Session session) {
		List locationStrings = new ArrayList(), agentStrings = new ArrayList();
		if (locationList != null) { // if the location list is null, either
			// we're not showing this panel
			// or we have bigger problems

			for (int i = 0; i < locationList.size(); i++) {
				locationStrings.add(((Location) locationList.get(i))
						.getCaption());
			}

			locationField = new FakeComboBox(locationStrings,
					"Add Location \u25bc");
			locationField.setSingleMode(true);
		}
	}

	protected void addModificationListeners(
			DocumentListener textFieldModificationListener,
			ItemListener comboBoxModificationListener,
			ChangeListener spinnerModificationListener) {
		absMeetingPane.addModificationListeners(textFieldModificationListener,
				comboBoxModificationListener, spinnerModificationListener);

		if (locationField != null) {
			locationField.addDocumentListener(textFieldModificationListener);
		}
		dateField.addChangeListener(spinnerModificationListener);
	}

	protected void addUndoableEditListeners(UndoableEditListener undoListener) {
		absMeetingPane.addUndoableEditListeners(undoListener);

		if (locationField != null) {
			locationField.addUndoableEditListener(undoListener);
		}
	}

	protected void addFocusListeners() {
		locationField.addFocusListener(this);
	}

	public Object saveText() {
		Object meeting = getDataList().get(getIndexOfDisplayed());

		if (meeting instanceof ConcreteMeeting) {
			ConcreteMeeting cmeeting = (ConcreteMeeting) meeting;

			absMeetingPane.saveAbstract(cmeeting);
			saveConcrete(cmeeting);

			if (cmeeting.getLocation() != null) {
				return meeting;
			}
		}

		getDataList().remove(meeting);
		return null;
	}

	private void saveConcrete(ConcreteMeeting cm) {
		for (int i = 0, size = locationList.size(); i < size; i++) {
			Location location = (Location) locationList.get(i);
			if (location.getCaption().equals(locationField.getText())) {
				cm.setLocation(location);
			}
		}

		cm.setMeetingdate(dateField.getDate());
		cm.setStarttime(dateField.getTime());
	}

	/**
	 * clears off the ConcreteMeeting panel
	 * 
	 */
	public void createNewInstance() {
		absMeetingPane.clearPanel();

		clearPanel();
		getDataList().add(getIndexOfDisplayed(), new ConcreteMeeting());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.SubPanel#clearPanel()
	 */
	public void clearPanel() {
		absMeetingPane.clearPanel();

		locationField.setText("");
		dateField.clearTime();
		locationField.requestFocusInWindow();
	}

	public boolean load(Session session) {
		setSession(session);

		if (getDataList() == null) {
			setDataList(load(session, ConcreteMeeting.class));
		}

		return true;
	}

	public boolean save() {
		Object meeting = saveText();
		super.persist(meeting);
		return true;
	}

	public JPanel getPane() {
		this.setLayout(new BorderLayout());
		add(absMeetingPane.getPane(), BorderLayout.CENTER);

		JPanel concreteMeetingPanel = new JPanel();
		add(concreteMeetingPanel, BorderLayout.SOUTH);

		JLabel locationLabel = new JLabel("Location: ");
		JLabel meetingDateLabel = new JLabel("Date: ");
		int row = 0;
		int col = 0;
		ConstraintFactory cf = new ConstraintFactory(8, 8, 8, 8, 2);
		GridBagConstraints gbc = new GridBagConstraints();
		concreteMeetingPanel.setLayout(new GridBagLayout());

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		concreteMeetingPanel.add(locationLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		concreteMeetingPanel.add(locationField, gbc);
		// private Location location;

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		concreteMeetingPanel.add(meetingDateLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		concreteMeetingPanel.add(dateField, gbc);

		return this;
	}

	public void fillInText(Session session) {
		if (getDataList() == null) {
			load(session);
		}

		if (getDataList().get(getIndexOfDisplayed()) instanceof ConcreteMeeting) {
			fillInText((ConcreteMeeting) getDataList().get(
					getIndexOfDisplayed()));
		}
	}

	public void fillInText(Object object) {
		fillInText((ConcreteMeeting) object);
	}

	public void fillInText() {
		Object obj = getDataList().get(getIndexOfDisplayed());

		if (obj instanceof ConcreteMeeting) {
			fillInText((ConcreteMeeting) obj);
		}
	}

	/**
	 * This fills in the fields with the data in the object
	 * 
	 * @param meeting
	 *            the object whose data we should fill in the panel with
	 */
	private void fillInText(ConcreteMeeting meeting) {
		absMeetingPane.fillInText(meeting);

		if (meeting.getLocation() != null) {
			locationField.setText(meeting.getLocation().getCaption());
		}

		if (meeting.getMeetingdate() != null) {
			dateField.setDate(meeting.getMeetingdate());
		}

		if (meeting.getStarttime() != null) {
			dateField.setTime(meeting.getStarttime());
		}

		setIndexOfDisplayed(getDataList().indexOf(meeting));
	}

	public void gotoNumber(int number) {
		Object obj = getDataList().get(number);

		fillInText((ConcreteMeeting) obj);
		setIndexOfDisplayed(number);
	}

	public String getTitle() {
		return "Concrete Meeting";
	}

	private Location getLocation(final String name) {
		for (int i = 0; i < locationList.size(); i++) {
			if (((Location) locationList.get(i)).getCaption().equals(name)) {
				return (Location) locationList.get(i);
			}
		}
		return null;
	}

	public String getStatusText() {
		return "On concrete meeting " + (getIndexOfDisplayed() + 1) + " of "
				+ getDataList().size() + " concrete meetings";
	}
}
