/**
 * 
 */
package airspaceGUI.base.event;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;

import org.hibernate.Session;

import airspaceGUI.base.BasePanel;
import airspaceGUI.base.SubPanel;
import airspaceGUI.expansion.FindWindow;
import airspaceGUI.model.AgentModel;
import airspaceGUI.model.GatheringModel;
import airspaceGUI.model.MeetingModel;
import airspaceGUI.model.PeopleModel;
import airspaceGUI.model.RoomModel;

/**
 * This panel edits STP and cmRadar events
 * 
 * @author srbennet
 * 
 */
public class EventPanel extends BasePanel {
	private FindEvent findWindow;

	private JTabbedPane tabs = new JTabbedPane();

	private ChangeListener tabListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			setStatus(((SubPanel) tabs.getSelectedComponent())
					.getIndexOfDisplayed());
			setButtonPermissions();
		}
	};

	private MeetingPanel meetingPanel;

	private GatheringPanel gatheringPanel;

	private boolean cmradar;

	/**
	 * @param session
	 *            A session the auxilary data can be loaded from
	 * @param tab
	 *            the tab panel this panel is on, null if it is in it's own
	 *            window
	 * @param agentModel
	 *            A model containing all the agent data
	 * @param meetingModel
	 *            A model containing all the meeting data
	 * @param gatheringModel
	 *            A model containing all the gathering data
	 * @param peopleModel
	 *            A model containing all the people data
	 * @param roomModel
	 *            A model containing all the room data
	 */
	public EventPanel(Session session, final JTabbedPane tab,
			AgentModel agentModel, MeetingModel meetingModel,
			GatheringModel gatheringModel, PeopleModel peopleModel,
			RoomModel roomModel) {
		nextButton.setText("Next Event");
		prevButton.setText("Previous Event");
		newButton.setText("New Event");
		deleteButton.setText("Delete Event");
		// disable the delete button since there is an error in base.
		deleteButton.setEnabled(false);

		// locationList = rm;
		ArrayList searchData = new ArrayList();

		for (int i = 0; i < meetingModel.size(); i++) {
			searchData.add(meetingModel.get(i));
		}

		for (int j = 0; j < gatheringModel.size(); j++) {
			searchData.add(gatheringModel.get(j));
		}

		for (int k = 0; k < agentModel.size(); k++) {
			searchData.add(agentModel.get(k));
		}
		findWindow = new FindEvent(EventPanel.this, searchData, tab);

		this.setName("Event");
		meetingPanel = new MeetingPanel(session, meetingModel, roomModel,
				agentModel);
		gatheringPanel = new GatheringPanel(session, gatheringModel,
				peopleModel);
		tabs.add(meetingPanel.getPane());
		// System.out.println(gatheringPanel.getPane());
		tabs.add(gatheringPanel.getPane());

		// add a change listener to fire whenever the tab changes
		tabs.addChangeListener(new ChangeListener() {
			private int selectedIndex = -1;

			public void stateChanged(final ChangeEvent cevt) {
				boolean alreadyLoading = isPageLoading();
				setPageLoading(true);

				JTabbedPane changed = (JTabbedPane) cevt.getSource();
				SubPanel selected = (SubPanel) changed.getSelectedComponent();
				if (selectedIndex >= 0) {
					final SubPanel lastPanel = (SubPanel) changed
							.getComponentAt(selectedIndex);
					lastPanel.saveText();
				}

				selected.fillInText(getSession());
				selectedIndex = changed.indexOfComponent(selected);

				if (!alreadyLoading) {
					setPageLoading(false);
				}
			}

		});
		ArrayList allData = new ArrayList();
		// we don't care what this call returns...
		allData.addAll(agentModel);
		allData.addAll(meetingModel);
		allData.addAll(gatheringModel);

		this.setSession(session);
		setDataList(searchData);
		this.cmradar = true;
	}

	/**
	 * @param session
	 *            A session the auxilary data can be loaded from
	 * @param tab
	 *            the tab panel this panel is on, null if it is in it's own
	 *            window
	 * @param agentModel
	 *            A model containing all the agent data
	 * @param meetingModel
	 *            A model containing all the meeting data
	 * @param gatheringModel
	 *            A model containing all the gathering data
	 * @param peopleModel
	 *            A model containing all the people data
	 * @param roomModel
	 *            A model containing all the room data
	 */
	public EventPanel(Session session, final JTabbedPane tab,
			GatheringModel gatheringModel, PeopleModel peopleModel) {
		super();
		meetingPanel = null;
		nextButton.setText("Next Event");
		prevButton.setText("Previous Event");
		newButton.setText("New Event");
		deleteButton.setText("Delete Event");
		// disable the delete button since there is an error in base.
		deleteButton.setEnabled(false);

		// locationList = rm;
		ArrayList searchData = new ArrayList();

		for (int j = 0; j < gatheringModel.size(); j++) {
			searchData.add(gatheringModel.get(j));
		}

		findWindow = new FindEvent(EventPanel.this, searchData, tab);

		this.setName("Event");
		gatheringPanel = new GatheringPanel(session, gatheringModel,
				peopleModel);

		ArrayList allData = new ArrayList();
		// we don't care what this call returns...
		allData.addAll(gatheringModel);
		tabs.add(gatheringPanel.getPane());

		this.setSession(session);
		setDataList(searchData);
		this.cmradar = false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#getFindWindow()
	 */
	public FindWindow getFindWindow() {
		return findWindow;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#getPane()
	 */
	public JPanel getPane() {
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(new GridBagLayout());
		// only fill horizontal so that the inner panel(s) doesn't center itself
		// and instead stays top-aligned
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.weightx = 1;
		gbc.weighty = 1;

		add(tabs, gbc);
		tabs.addChangeListener(tabListener);
		if (cmradar) {
			// we want the cmradar fields
			meetingPanel.addChangeListener(tabListener);
		}

		gbc.gridx++;
		gbc.weightx = 0;
		gbc.weighty = 0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.SOUTHWEST;
		gbc.insets = new Insets(0, 0, 12, 0);
		add(getButtonPanel(), gbc);

		this.addModificationListeners(textFieldModificationListener,
				comboBoxModificationListener, checkBoxModificationListener);

		return this;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeairspaceGUI.base.BasePanel#addModificationListeners(java.awt.event.
	 * KeyListener, java.awt.event.ItemListener,
	 * javax.swing.event.ChangeListener)
	 */
	protected void addModificationListeners(
			DocumentListener textFieldModificationListener,
			ItemListener comboBoxModificationListener,
			ChangeListener checkBoxModificationListener) {
		if (cmradar) {
			meetingPanel.addModificationListeners(
					textFieldModificationListener,
					comboBoxModificationListener, checkBoxModificationListener);
		}
		gatheringPanel.addModificationListeners(textFieldModificationListener,
				comboBoxModificationListener, checkBoxModificationListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * airspaceGUI.base.BasePanel#addUndoableEditListeners(javax.swing.event
	 * .UndoableEditListener)
	 */
	protected void addUndoableEditListeners(
			final UndoableEditListener undoListener) {
		if (cmradar) {
			meetingPanel.addUndoableEditListeners(undoListener);
		}
		gatheringPanel.addUndoableEditListeners(undoListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#setStatus(int)
	 */
	protected void setStatus(int numberOn) {
		// ((SubPanel) tabs.getSelectedComponent()).gotoNumber(numberOn);
		statusLabel.setText(((SubPanel) tabs.getSelectedComponent())
				.getStatusText());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#getStatusText()
	 */
	public String getStatusText() {
		return ((SubPanel) tabs.getSelectedComponent()).getStatusText();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#saveText()
	 */
	public Object saveText() {
		save();

		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#clearText()
	 */
	public void createNewInstance() {
		((SubPanel) tabs.getSelectedComponent()).createNewInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#clearPanel()
	 */
	public void clearPanel() {
		((SubPanel) tabs.getSelectedComponent()).clearPanel();
	}

	/**
	 * Deletes the currently shown message from the database and shows the next
	 * message in the List
	 * 
	 * @return true if the deletion was successful, false otherwise
	 */
	public boolean deleteDisplayedObject() {
		boolean deleteResult = ((SubPanel) tabs.getSelectedComponent())
				.deleteDisplayedObject();
		if (!getSession().isOpen()) {
			getSession().beginTransaction();
		}
		setStatus(getIndexOfDisplayed());
		return deleteResult;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#load(org.hibernate.Session)
	 */
	public boolean load(final Session session) {
		return ((SubPanel) tabs.getSelectedComponent()).load(session);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#save()
	 */
	public boolean save() {
		boolean saved = true;

		if (meetingPanel != null) {
			saved = meetingPanel.save();
		}

		saved = gatheringPanel.save();

		return saved;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#fillInText(org.hibernate.Session)
	 */
	public void fillInText(final Session session) {
		setSession(session);
		((SubPanel) tabs.getSelectedComponent()).fillInText(getSession());
		setStatus(((SubPanel) tabs.getSelectedComponent())
				.getIndexOfDisplayed());
		setButtonPermissions();
	}

	/**
	 * Fills in the panel with the data contained in the object
	 * 
	 * @param object
	 *            the object to fill the panel with
	 */
	public void fillInText(final Object object) {
		((SubPanel) tabs.getSelectedComponent()).fillInText(object);
		setStatus(((SubPanel) tabs.getSelectedComponent())
				.getIndexOfDisplayed());
		setButtonPermissions();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#gotoNumber(int)
	 */
	public void gotoNumber(int number) {
		setIndexOfDisplayed(number);

		((SubPanel) tabs.getSelectedComponent()).gotoNumber(number);

		setStatus(((SubPanel) tabs.getSelectedComponent())
				.getIndexOfDisplayed());
		setButtonPermissions();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#getTitle()
	 */
	public String getTitle() {
		return ((SubPanel) tabs.getSelectedComponent()).getTitle();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#getMinimumSize()
	 */
	public Dimension getMinimumSize() {
		int minHeight = Math.max(meetingPanel.getPreferredSize().height,
				gatheringPanel.getPreferredSize().height);
		int minWidth = Math.max(meetingPanel.getPreferredSize().width,
				gatheringPanel.getPreferredSize().width);
		minHeight = Math.max(minHeight,
				getButtonPanel().getPreferredSize().height);
		minWidth = Math
				.max(minWidth, getButtonPanel().getPreferredSize().width);
		return new Dimension(minWidth, minHeight);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.SubPanel#maxIndex()
	 */
	public int maxIndex() {
		return ((SubPanel) tabs.getSelectedComponent()).maxIndex();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.SubPanel#isModified()
	 */
	public boolean isModified() {
		return ((SubPanel) tabs.getSelectedComponent()).isModified();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.SubPanel#setModified(boolean)
	 */
	public void setModified(boolean modified) {
		((SubPanel) tabs.getSelectedComponent()).setModified(modified);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.SubPanel#isPageLoading()
	 */
	public boolean isPageLoading() {
		return ((SubPanel) tabs.getSelectedComponent()).isPageLoading();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.SubPanel#setPageLoading(boolean)
	 */
	public void setPageLoading(boolean pageLoading) {
		super.setPageLoading(pageLoading);

		for (int i = 0, size = tabs.getTabCount(); i < size; i++) {
			SubPanel subPane = (SubPanel) tabs.getComponentAt(i);
			subPane.setPageLoading(pageLoading);
		}
	}

}
