/**
 * 
 */
package airspaceGUI.base.event;

import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTabbedPane;

import airspaceGUI.expansion.FindWindow;

import edu.cmu.radar.base.Gathering;
import edu.cmu.radar.base.cmradar.AbstractMeeting;
import edu.cmu.radar.base.cmradar.Agent;
import edu.cmu.radar.base.cmradar.ConcreteMeeting;

/**
 * @author srbennet
 * 
 */
public class FindEvent extends FindWindow
{
    //TODO spin off/ add a tabbed interface to the find window for the events panel
    
    //Fields to search on for events
    private static final String EVENTTYPE = "Type";

    private static final String LOCATION = "Location";

    //fields to search for a Gathering
    private static final String CAPTION = "Caption";
    
    private static final String TITLE = "Title";
    
    //fields to search for an Agent
    private static final String USERNAME = "Username";
    
    private static final String USERID = "User Id";
    
    private static final String NAME = "Name";
    
    //Reuse the eventType constant
    //private static final String TYPE = "Type";
    

    private transient final EventPanel eventPanel;

    private transient ArrayList searchList;

    private static Object[] meetingKeywordList = {EVENTTYPE,LOCATION};
    private static Object[] gatheringKeywordList = {CAPTION, TITLE};
    private static Object[] agentKeywordList = {USERNAME, USERID, NAME/*, TYPE*/};
    private static Object[] keywordList = {EVENTTYPE, LOCATION, CAPTION, 
        TITLE, USERNAME, USERID, NAME/*, TYPE*/};
    

    /**
     * @param eventPanel
     *            the panel this field should load things onto when they are
     *            double clicked
     * @param searchList
     *            the list of meetings to search to come up with possibilities 
     *            for what the user is looking for
     */
    public FindEvent(EventPanel eventPanel, ArrayList searchList)
    {
        super(keywordList);
        this.eventPanel = eventPanel;
        parent = eventPanel;
        this.searchList = searchList;
    }

    /**
     * @param eventPanel
     *            the panel this field should load things onto when they are
     *            double clicked
     * @param searchList
     *            the list to search to come up with possibilities for what the
     *            user is looking for
     * @param tab
     *            The tabbed panel the eventPanel is on, null if it is not in a
     *            tabbed pane
     * 
     */
    public FindEvent(EventPanel eventPanel, ArrayList searchList,
            JTabbedPane tab)
    {
        super(keywordList, tab);
        this.eventPanel = eventPanel;
        parent = eventPanel;
        this.searchList = searchList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see airspaceGUI.expansion.FindWindow#updatePanel(java.lang.Object)
     */
    protected void updatePanel(final Object object) {
       // System.out.println(object);
        eventPanel.fillInText(object);
        if (tabs != null)
        {
            //System.out.println(eventPanel);
            //System.out.println(this.tabs.getComponentCount() + this.tabs.getComponentAt(tabs.getSelectedIndex()).getName());
            //System.out.println(super.tabs.getComponentCount() + super.tabs.getComponentAt(tabs.getSelectedIndex()).getName());

            super.tabs.setSelectedComponent(eventPanel);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see airspaceGUI.expansion.FindWindow#search(java.util.Map)
     */
    protected ArrayList search(final Map searchTerms) {
        final ArrayList results = new ArrayList();
        boolean isResult = true;
        Object workingOn;

        for (int i = 0; i < searchList.size(); i++)
        {
            workingOn = searchList.get(i);
            try
            {

                if (searchTerms.get(EVENTTYPE) != null)
                {   
                      if(workingOn instanceof AbstractMeeting 
                        && ((AbstractMeeting)workingOn).getType() != null
                        && (!((String)searchTerms.get(EVENTTYPE)).toLowerCase()
                                .contains(
                                        ((AbstractMeeting)workingOn)
                                                .getType().toLowerCase()) 
                                && !((AbstractMeeting)workingOn)
                                .getType().toLowerCase().contains(
                                        ((String)searchTerms.get(EVENTTYPE)).toLowerCase())))
                      {
                          isResult = false;
                      } else if (workingOn instanceof Agent
                              && ((Agent)workingOn).getAgenttype() != null
                              && !((String)searchTerms.get(EVENTTYPE)).toLowerCase()
                                  .contains(
                                      ((Agent)workingOn)
                                              .getAgenttype().toLowerCase()))
                      {
                          isResult = false;
                      }
                }

                if (searchTerms.get(CAPTION) != null 
                        && workingOn instanceof Gathering
                        && workingOn instanceof Gathering
                        && !((Gathering)workingOn).getCaption()
                                .toLowerCase().contains(
                                        ((String)searchTerms.get(CAPTION))
                                                .toLowerCase()))
                {
                    isResult = false;
                }
                if (searchTerms.get(LOCATION) != null 
                        && workingOn instanceof ConcreteMeeting
                        && !((ConcreteMeeting)workingOn).getLocation()
                                .getCaption().toLowerCase().contains(
                                        ((String)searchTerms.get(LOCATION))
                                                .toLowerCase()))
                {
                    isResult = false;
                }

                if (isResult 
                        && workingOn instanceof Gathering
                        && searchTerms.get(TITLE) != null
                        && !(((Gathering)workingOn).getTitle()
                                .contains(((String)searchTerms.get(TITLE))) || 
                                ((String)searchTerms.get(TITLE))
                                .compareToIgnoreCase(((Gathering)workingOn).getTitle()) == 0))
                {
                    isResult = false;
                }
                
                if( searchTerms.get(USERNAME) != null 
                        && workingOn instanceof Agent
                        && !((String)searchTerms.get(USERNAME)).toLowerCase()
                            .contains(
                                    ((Agent)workingOn)
                                    .getUsername().toLowerCase()))
                {
                    isResult = false;
                }
                
                if(searchTerms.get(USERID) != null 
                        && workingOn instanceof Agent
                        && !((String)searchTerms.get(USERID)).toLowerCase()
                            .contains(
                                ((Agent)workingOn)
                                .getUserid().toLowerCase()))
                {
                    isResult = false;
                }
                
                
                if(searchTerms.get(NAME) != null 
                        && !(workingOn instanceof Agent)
                        && ((!((String)searchTerms.get(NAME)).toLowerCase()
                                .contains(
                                        ((Agent)workingOn)
                                        .getAgentname().toLowerCase()))
                                || !((Agent)workingOn)
                                    .getAgentname().toLowerCase()
                                    .contains(((String)searchTerms
                                            .get(NAME)).toLowerCase())))
                {
                    isResult = false;
                } 
                
            } catch (ClassCastException e)
            {
                // e.printStackTrace();
                // ignore class cast exceptions in this case since we can't know
                // really what kind of event each is
                isResult = false;
                //System.out.println("Class Cast Exception " + e.getMessage());
            } finally
            {
                if (isResult)
                {
                    results.add(workingOn);
                }
            }
            isResult = true;
        }

        return results;
    }

    public Component getListCellRendererComponent(final JList list, final Object value, // value
            // to
            // display
            final int index, // cell index
            boolean isSelected, // is the cell selected
            boolean cellHasFocus) {
        final JLabel label = new JLabel();
        if (value instanceof Gathering)
        {
            label.setText("Title: " + ((Gathering)value).getTitle()
                    + " Caption: " + ((Gathering)value).getCaption());
        } else if (value instanceof ConcreteMeeting)
        {
            label.setText("Type: " + ((AbstractMeeting)value).getType()
                    + " in "
                    + ((ConcreteMeeting)value).getLocation().getCaption());
        } else if (value instanceof AbstractMeeting)
        {
            label.setText("Type: " + ((AbstractMeeting)value).getType());
        } else
        {
            label.setText("Agent Name: " + ((Agent)value).getAgentname());
        }

        if (isSelected)
        {
            label.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY));
            label.setOpaque(true);
            label.setBackground(list.getSelectionBackground());
            label.setForeground(list.getSelectionForeground());
        } else
        {
            label.setBorder(BorderFactory
                    .createLineBorder(list.getBackground()));
            label.setOpaque(false);
            label.setBackground(list.getBackground());
            label.setForeground(list.getForeground());
        }
        label.setEnabled(list.isEnabled());
        label.setFont(list.getFont());

        return label;
    }

    public void updateSearchList(List search) {
        searchList = (ArrayList)search;
    }

}
