package airspaceGUI.base.event;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;

import org.hibernate.HibernateException;
import org.hibernate.Session;

import airspaceGUI.base.SubPanel;
import airspaceGUI.expansion.ConstraintFactory;
import airspaceGUI.expansion.FakeComboBox;
import airspaceGUI.model.GatheringModel;
import airspaceGUI.model.PeopleModel;
import edu.cmu.radar.base.Gathering;
import edu.cmu.radar.base.Person;

public class GatheringPanel extends SubPanel {
	/**
	 * fields related to a gathering
	 */
	private final transient JTextField captionField = new JTextField();

	private final transient JTextField titleField = new JTextField();

	private final transient JTextField purposeField = new JTextField();

	private transient FakeComboBox personField; // only one!

	private transient FakeComboBox speakerField;

	private transient final ArrayList personList;

	// Listener to get adding new People events
	private final transient ActionListener personListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			int i = personList.indexOf(getPerson(e.getActionCommand()));

			if (i != -1) {
				String name;
				final Person p = (Person) personList.get(i);

				if (p.getMiddle() == null) {
					name = p.getFirst() + " " + p.getLast();
				} else {
					name = p.getFirst() + " " + p.getMiddle() + " "
							+ p.getLast();
				}
				personField.setText(name);
			}
		}
	};

	public GatheringPanel(Session session, GatheringModel gm,
			PeopleModel personModel) {
		super();
		personList = personModel;
		setDataList(gm);
		setName("Conference Sessions");
		// setStatus(getIndexOfDisplayed() +1);
		setSession(session);
		loadSupports();
	}

	private ArrayList getPeopleNames() {
		Person workingOn;
		final ArrayList nameList = new ArrayList();
		String name;

		for (int i = 0; i < personList.size(); i++) {
			workingOn = (Person) personList.get(i);

			if (workingOn.getMiddle() == null) {
				name = workingOn.getFirst() + " " + workingOn.getLast();
			} else {
				name = workingOn.getFirst() + " " + workingOn.getMiddle() + " "
						+ workingOn.getLast();
			}
			nameList.add(name);
		}

		return nameList;
	}

	private void loadSupports() {
		final ArrayList personStrings = new ArrayList();

		Person current;
		for (int i = 0; i < personList.size(); i++) {
			current = (Person) personList.get(i);
			if (current.getMiddle() == null) {
				personStrings.add(current.getFirst() + " " + current.getLast());
			} else {
				personStrings.add(current.getFirst() + " "
						+ current.getMiddle() + " " + current.getLast());
			}

			if (current.getAkafirst() != null
					&& !current.getAkafirst().equals("")) {
				if (current.getMiddle() == null) {
					personStrings.add(current.getAkafirst() + " "
							+ current.getLast());
				} else {
					personStrings.add(current.getFirst() + " "
							+ current.getMiddle() + " " + current.getLast());
				}
			}
		}

		personField = new FakeComboBox(personStrings, "Add Person \u25bc");
		speakerField = new FakeComboBox(personStrings, "Add Speaker \u25bc");
		personField.addSelectionListener(personListener);
	}

	public JPanel getPane() {
		int row = 0, col = 0;
		JPanel gatheringPanel = new JPanel();
		GridBagConstraints gbc = new GridBagConstraints();
		gatheringPanel.setLayout(new GridBagLayout());
		setLayout(new GridBagLayout());
		ConstraintFactory cf = new ConstraintFactory(8, 8, 8, 8, 2);

		/**
		 * labels for fields related to a gathering
		 */
		final JLabel captionLabel = new JLabel("Caption: ");
		final JLabel titleLabel = new JLabel("Title: ");
		final JLabel purposeLabel = new JLabel("Purpose: ");
		final JLabel personLabel = new JLabel("Person: ");
		final JLabel speakerLabel = new JLabel("Speakers: ");

		/**
		 * fields related to a gathering
		 */
		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		gbc.weighty = 0;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gatheringPanel.add(captionLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		gbc.weighty = 0.0;
		gbc.weightx = 1.0;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gatheringPanel.add(captionField, gbc);
		// private String caption;

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 0;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gatheringPanel.add(titleLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 0;
		gbc.weightx = 1.0;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gatheringPanel.add(titleField, gbc);
		// private String title;

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 0;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gatheringPanel.add(purposeLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 0;
		gbc.weightx = 1.0;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gatheringPanel.add(purposeField, gbc);
		// private String purpose;

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 0;
		gbc.anchor = GridBagConstraints.EAST;
		gatheringPanel.add(personLabel, gbc);
		gbc = cf.getConstraint(col--, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 0;
		gbc.weightx = 1.0;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gatheringPanel.add(personField, gbc); // only one!
		// private Person person;

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.BOTTOMMOST);
		gbc.weighty = 1;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gbc.insets = new Insets(9, 8, 8, 0);
		gatheringPanel.add(speakerLabel, gbc);
		gbc = cf.getConstraint(col--, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.BOTTOMMOST);
		gbc.weighty = 1;
		gbc.weightx = 1.0;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gatheringPanel.add(speakerField, gbc); // only one!

		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.00;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		this.add(gatheringPanel, gbc);

		return this;
	}

	private boolean saveGathering(final Gathering gather) {
		boolean isEmpty = true;
		gather.setCaption(captionField.getText());
		isEmpty = isEmpty && "".equals(captionField);

		gather.setEid(captionField.getText().replaceAll(" ", ""));

		gather.setTitle(titleField.getText());
		isEmpty = isEmpty && "".equals(titleField.getText());

		try {
			gather.setPurpose(new Integer(purposeField.getText()));
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(null,
					"The \"Purpose\" field in Gathering " + gather.getTitle()
							+ " is invalid.\n", "Error",
					JOptionPane.ERROR_MESSAGE);
			// there was an error, so we should return null
			e.printStackTrace();
			return true;
		}

		if (getPerson(personField.getText()) != null) {
			gather.setPerson(getPerson(personField.getText()));
			isEmpty = false;
		}

		final StringTokenizer speakerTokens = new StringTokenizer(speakerField
				.getText(), ",");
		String speaker;
		while (speakerTokens.hasMoreTokens()) {
			speaker = speakerTokens.nextToken().trim();
			if (getPerson(speaker) != null) {
				gather.addSpeaker(getPerson(speaker));
				isEmpty = false;
			}
		}
		return isEmpty;
	}

	/**
	 * Clears off the gathering panel
	 * 
	 */
	public void clearPanel() {
		captionField.setText("");
		titleField.setText("");
		purposeField.setText("");
		personField.setText("");
		speakerField.setText("");

		captionField.requestFocusInWindow();
	}

	/**
	 * This fills in the fields with the data in the object
	 * 
	 * @param object
	 *            the object whose data we should fill in the panel with
	 */
	private void fillInText(final Gathering object) {
		captionField.setText(object.getCaption());
		// private String caption;

		titleField.setText(object.getTitle());
		// private String title;

		if (object.getPurpose() != null) {
			purposeField.setText(object.getPurpose().toString());
			// private String purpose;
		}
		Person person;

		if (object.getPerson() != null) {
			person = object.getPerson();

			if (person.getMiddle() == null) {
				personField.setText(person.getFirst() + " " + person.getLast());
			} else {
				personField.setText(person.getFirst() + " "
						+ person.getMiddle() + " " + person.getLast());
			}
		}

		try {
			final Iterator itr = object.getSpeakers().iterator();

			if (itr.hasNext()) {
				person = (Person) itr.next();
				if (person != null) {
					if (person.getMiddle() == null) {
						speakerField.setText(person.getFirst() + " "
								+ person.getLast());
					} else {
						speakerField.setText(person.getFirst() + " "
								+ person.getMiddle() + " " + person.getLast());
					}

					while (itr.hasNext()) {
						person = (Person) itr.next();
						if (person.getMiddle() == null) {
							speakerField.setText(speakerField.getText() + ", "
									+ person.getFirst() + " "
									+ person.getLast());
						} else {
							speakerField.setText(speakerField.getText() + ", "
									+ person.getFirst() + " "
									+ person.getMiddle() + " "
									+ person.getLast());
						}
					}
				}
			} else {
				speakerField.setText("");
			}
		} catch (HibernateException e) {
			JOptionPane.showMessageDialog(null,
					"The data is inconsistent.  At least one of the speakers "
							+ "listed for this event does not exist. ",
					"Error", JOptionPane.ERROR_MESSAGE);
		}

		setIndexOfDisplayed(getDataList().indexOf(object));
	}

	private Person getPerson(final String name) {
		String fullName;
		String fullWithMiddle;
		for (int i = 0; i < personList.size(); i++) {
			fullName = ((Person) personList.get(i)).getFirst() + " "
					+ ((Person) personList.get(i)).getLast();
			fullWithMiddle = ((Person) personList.get(i)).getAkafirst() + " "
					+ ((Person) personList.get(i)).getMiddle() + " "
					+ ((Person) personList.get(i)).getLast();

			if (fullName.equals(name) || fullWithMiddle.equals(name)) {
				return (Person) personList.get(i);
			}
		}
		return null;
	}

	public void addFocusListener(final FocusListener fl) {
		captionField.addFocusListener(this);
		titleField.addFocusListener(this);
		purposeField.addFocusListener(this);
		personField.addFocusListener(this);
		speakerField.addFocusListener(this);
	}

	protected void addModificationListeners(
			DocumentListener textFieldModificationListener,
			ItemListener comboBoxModificationListener,
			ChangeListener checkBoxModificationListener) {
		captionField.getDocument().addDocumentListener(
				textFieldModificationListener);
		titleField.getDocument().addDocumentListener(
				textFieldModificationListener);
		purposeField.getDocument().addDocumentListener(
				textFieldModificationListener);
		if (personField != null) {
			personField.addDocumentListener(textFieldModificationListener);
		}
		if (speakerField != null) {
			speakerField.addDocumentListener(textFieldModificationListener);
		}
	}

	protected void addUndoableEditListeners(
			final UndoableEditListener undoListener) {
		captionField.getDocument().addUndoableEditListener(undoListener);
		titleField.getDocument().addUndoableEditListener(undoListener);
		purposeField.getDocument().addUndoableEditListener(undoListener);

		if (personField != null) {
			personField.addUndoableEditListener(undoListener);
		}

		if (speakerField != null) {
			speakerField.addUndoableEditListener(undoListener);
		}
	}

	/*
	 * protected void setStatus(int numberOn) { statusLabel.setText("On
	 * gathering " + numberOn + " of " + getDataList().size() + " gatherings");
	 * }
	 */

	public Object saveText() {
		Gathering savedObject = (Gathering) getDataList().get(
				getIndexOfDisplayed());

		boolean returnNull = saveGathering(savedObject);

		if (returnNull) {
			return null;
		}

		return savedObject;
	}

	public void createNewInstance() {
		clearPanel();
		Gathering gathering = new Gathering();
		getDataList().add(getIndexOfDisplayed(), gathering);
	}

	public boolean load(Session session) {
		setSession(session);
		loadSupports();
		return true;
	}

	public boolean save() {
		if (isModified()) {
			Gathering gathering = (Gathering) getDataList().get(
					getIndexOfDisplayed());
			saveGathering(gathering);

			super.persist(gathering);
		}

		return true;
	}

	public void fillInText(final Session session) {
		if (getDataList() == null || getDataList().isEmpty()) {
			load(session);
		}
		fillInText((Gathering) getDataList().get(getIndexOfDisplayed()));
	}

	public void fillInText(Object object) {
		if (object instanceof Gathering) {
			fillInText((Gathering) object);
		}
	}

	public void gotoNumber(int number) {
		setIndexOfDisplayed(number);
		fillInText((Gathering) getDataList().get(number));
	}

	public String getTitle() {
		return "Edit an Conference Session";
	}

	public String getStatusText() {
		return "On gathering " + (getIndexOfDisplayed() + 1) + " of "
				+ getDataList().size() + " gatherings";
	}

}
