package airspaceGUI.base.event;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;

import org.hibernate.Session;

import airspaceGUI.base.SubPanel;
import airspaceGUI.expansion.ConstraintFactory;
import airspaceGUI.model.AgentModel;
import airspaceGUI.model.MeetingModel;
import airspaceGUI.model.RoomModel;
import edu.cmu.radar.base.cmradar.AbstractMeeting;
import edu.cmu.radar.base.cmradar.ConcreteMeeting;

public class MeetingPanel extends SubPanel {
	/**
	 * The panel related to an abstractMeeting
	 */
	private AbstractMeetingPanel abstractPanel;

	/**
	 * The fields related to a concrete meeting
	 */
	private ConcreteMeetingPanel concretePanel;

	/**
	 * The fields related to an agent
	 */
	private AgentPanel agentPanel;

	private SubPanel focusedPanel;

	private static JRadioButton concreteButton = new JRadioButton(
			"Concrete Meeting");

	private static JRadioButton abstractButton = new JRadioButton(
			"Abstract Meeting");

	private static JRadioButton agentButton = new JRadioButton("Agent");

	private ChangeListener cl;

	// listener to show everything but the Gathering fields
	private transient ActionListener concreteListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			boolean alreadyLoading = isPageLoading();
			setPageLoading(true);

			concreteSelected();

			if (!alreadyLoading) {
				setPageLoading(false);
			}
		}

	};

	// Listner to hide everything but the abstractGathering fields
	private transient ActionListener abstractListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			boolean alreadyLoading = isPageLoading();
			setPageLoading(true);

			abstractSelected();

			if (!alreadyLoading) {
				setPageLoading(false);
			}
		}

	};

	// Listner to hide everything but the abstractGathering fields
	private transient ActionListener agentListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			boolean alreadyLoading = isPageLoading();
			setPageLoading(true);

			agentSelected();

			if (!alreadyLoading) {
				setPageLoading(false);
			}
		}

	};

	/**
	 * @param session
	 *            A session the auxilary data can be loaded from
	 * @param eventModel
	 * @param roomModel
	 *            A model containing all the room data
	 * @param am
	 */
	public MeetingPanel(Session session, MeetingModel eventModel,
			RoomModel roomModel, AgentModel am) {
		setDataList(eventModel);

		abstractPanel = new AbstractMeetingPanel(session, eventModel);
		abstractPanel.getPane();

		concretePanel = new ConcreteMeetingPanel(session, roomModel, eventModel);
		concretePanel.getPane();

		agentPanel = new AgentPanel(session, am, eventModel);
		agentPanel.getPane();

		this.setName("Shared Calendar");
		setSession(session);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#save()
	 */
	public boolean save() {
		if (getDataList() != null) {
			if (concreteButton.isSelected()) {
				concretePanel.save();
			} else if (abstractButton.isSelected()) {
				abstractPanel.save();
			} else {
				agentPanel.save();
			}
		}
		
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#saveText()
	 */
	public Object saveText() {
		if (getDataList() != null) {
			Object savedObject = getDataList().get(getIndexOfDisplayed());

			if (concreteButton.isSelected()) {
				concretePanel.saveText();
			} else if (abstractButton.isSelected()) {
				abstractPanel.saveText();
			} else {
				agentPanel.saveText();
			}

			return savedObject;
		}

		return null;
	}

	/**
	 * transforms a concrete meeting into an abstract meeting
	 * 
	 * @param cm
	 *            the concrete meeting to transform
	 * @return the new abstract meeting
	 */
	private AbstractMeeting makeAbstract(ConcreteMeeting cm) {
		AbstractMeeting am = new AbstractMeeting();

		am.setAgentsInterested(cm.getAgentsInterested());
		am.setDuration(cm.getDuration());
		am.setId(cm.getId());
		am.setInitiator(cm.getInitiator());
		am.setLegacy_id(cm.getLegacy_id());
		am.setType(cm.getType());
		am.setWeight(cm.getWeight());

		return am;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#clearText()
	 */
	public void createNewInstance() {
		if (agentPanel.isVisible()) {
			agentPanel.createNewInstance();
		} else if (concretePanel.isVisible()) {
			concretePanel.createNewInstance();
		} else {
			abstractPanel.createNewInstance();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.SubPanel#clearPanel()
	 */
	public void clearPanel() {
		if (agentPanel.isVisible()) {
			agentPanel.clearPanel();
		} else if (concretePanel.isVisible()) {
			concretePanel.clearPanel();
		} else {
			abstractPanel.clearPanel();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#load(org.hibernate.Session)
	 */
	public boolean load(final Session session) {
		// only need to load the super-superclass, and it will get all its
		// children.
		// System.out.println("saving the session?");
		setDataList(load(session, AbstractMeeting.class, "id"));
		abstractPanel.load(session);
		concretePanel.load(session);
		agentPanel.load(session);
		setSession(session);
		return true;
	}

	/**
	 * Deletes the currently shown message from the database and shows the next
	 * message in the List
	 * 
	 * @return true if the deletion was successful, false otherwise
	 */
	public boolean deleteDisplayedObject() {
		return focusedPanel.deleteDisplayedObject();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#getPane()
	 */
	public JPanel getPane() {
		final JPanel buttonPanel = new JPanel();
		// this.setBackground(Color.BLUE);

		final ConstraintFactory cf = new ConstraintFactory(8, 8, 8, 8, 2);
		GridBagConstraints gbc = new GridBagConstraints();
		this.setLayout(new GridBagLayout());
		concretePanel.setName("concrete");
		int row = 0;

		setButtonPanel(buttonPanel, cf);

		abstractPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(), "Abstract Meeting:"));
		concretePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(), "Concrete Meeting:"));

		/**
		 * add the subpanels to the main panel
		 * 
		 */
		row = 0;
		gbc = cf.getConstraint(0, ++row, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		// gbc.weighty = 0.1;
		this.add(buttonPanel, gbc);
		gbc = cf.getConstraint(0, ++row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		// gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.BOTH;
		this.add(agentPanel, gbc);

		// gbc.weighty = 0;
		this.add(abstractPanel, gbc);

		gbc = cf.getConstraint(0, ++row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		this.add(concretePanel, gbc);

		addFocusListeners();
		concreteSelected();
		focusedPanel = concretePanel;

		return this;
	}

	/**
	 * This sets the buttonPanel of this panel
	 * 
	 * @param buttonPanel
	 *            The button panel to incorporate
	 * @param cf
	 *            The constraint factory for placement
	 */
	private void setButtonPanel(final JPanel buttonPanel,
			final ConstraintFactory cf) {
		int row = 0, col = 0;
		// final Dimension gatheringButtonSize = new Dimension(90, 20);
		final Dimension otherButtonSize = new Dimension(140, 20);
		GridBagConstraints gbc;
		/**
		 * set up the buttons to control what is showing.
		 */
		buttonPanel.setLayout(new GridBagLayout());
		gbc = cf.getConstraint(col++, row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		gbc.anchor = GridBagConstraints.EAST;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		gbc.insets = new Insets(0, 0, 0, 0);
		buttonPanel.add(abstractButton, gbc);
		gbc = cf.getConstraint(col++, row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		gbc.insets = new Insets(0, 0, 0, 0);
		buttonPanel.add(concreteButton, gbc);
		gbc = cf.getConstraint(col++, row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		gbc.anchor = GridBagConstraints.WEST;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		gbc.insets = new Insets(0, 0, 0, 0);
		buttonPanel.add(agentButton, gbc);

		abstractButton.addActionListener(abstractListener);
		abstractButton.setMaximumSize(otherButtonSize);
		abstractButton.setMinimumSize(otherButtonSize);
		abstractButton.setPreferredSize(otherButtonSize);
		concreteButton.addActionListener(concreteListener);
		concreteButton.setMaximumSize(otherButtonSize);
		concreteButton.setMinimumSize(otherButtonSize);
		concreteButton.setPreferredSize(otherButtonSize);
		agentButton.addActionListener(agentListener);
		agentButton.setMaximumSize(otherButtonSize);
		agentButton.setMinimumSize(otherButtonSize);
		agentButton.setPreferredSize(otherButtonSize);
	}

	/**
	 * Adds focus Listeners to all of the fields on this tab
	 */
	private void addFocusListeners() {
		abstractPanel.addFocusListeners(this);
		concretePanel.addFocusListeners();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#fillInText(org.hibernate.Session)
	 */
	public void fillInText(final Session session) {
		if (getDataList() == null) {
			load(session);
		}
		fillInText(getDataList().get(getIndexOfDisplayed()));
	}

	/**
	 * This fills in the fields with the data in the object
	 * 
	 * @param object
	 *            the object whose data we should fill in the panel with
	 */
	public void fillInText(final Object object) {
		if (object instanceof ConcreteMeeting) {
			concreteSelected();
			concretePanel.fillInText(object);
			abstractPanel.fillInText(object);

		} else if (object instanceof AbstractMeeting) {
			abstractSelected();
			abstractPanel.fillInText(object);
			concretePanel.createNewInstance();
		} else {
			agentSelected();
			agentPanel.fillInText(object);
		}

	}

	/**
	 * Display the concrete panel on the GUI
	 */
	protected void showConcretePanel() {
		concretePanel.setVisible(true);
	}

	/**
	 * Hide the concrete panel on the GUI
	 */
	protected void hideConcretePanel() {
		concretePanel.setVisible(false);
	}

	/**
	 * Display the concrete panel on the GUI
	 */
	protected void showAbstractPanel() {
		abstractPanel.setVisible(true);
	}

	/**
	 * Hide the concrete panel on the GUI
	 */
	protected void hideAbstractPanel() {
		abstractPanel.setVisible(false);
	}

	/**
	 * Display the concrete panel on the GUI
	 */
	protected void showAgentPanel() {
		// this is a hack to see if we can get this panel to take up the space
		// that is usually
		// taken by the other panels
		Dimension size = new Dimension(agentPanel.getWidth(), abstractPanel
				.getHeight()
				+ concretePanel.getHeight());
		agentPanel.setSize(size);
		agentPanel.setPreferredSize(size);
		agentPanel.setMinimumSize(size);
		agentPanel.setVisible(true);
	}

	/**
	 * Hide the concrete panel on the GUI
	 */
	protected void hideAgentPanel() {
		agentPanel.setVisible(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#gotoNumber(int)
	 */
	public void gotoNumber(int number) {
		setIndexOfDisplayed(number);

		// goto the number in our subpanel
		focusedPanel.gotoNumber(number);
		if (concretePanel.isVisible()) {
			// System.out.println("abstract number");
			// this is here so that both eh abstract panel gets loaded with new
			// informaiton
			abstractPanel.gotoNumber(number);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#getTitle()
	 */
	public String getTitle() {
		return "Edit an Event";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeairspaceGUI.base.BasePanel#addModificationListeners(java.awt.event.
	 * KeyListener, java.awt.event.ItemListener,
	 * javax.swing.event.ChangeListener)
	 */
	protected void addModificationListeners(
			DocumentListener textFieldModificationListener,
			ItemListener comboBoxModificationListener,
			ChangeListener spinnerModificationListener) {
		// add the key listener to the plain text fields and fake combo boxes
		abstractPanel.addModificationListeners(textFieldModificationListener,
				comboBoxModificationListener, spinnerModificationListener);
		concretePanel.addModificationListeners(textFieldModificationListener,
				comboBoxModificationListener, spinnerModificationListener);
		agentPanel.addModificationListeners(textFieldModificationListener,
				comboBoxModificationListener, spinnerModificationListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * airspaceGUI.base.BasePanel#addUndoableEditListeners(javax.swing.event
	 * .UndoableEditListener)
	 */
	protected void addUndoableEditListeners(
			final UndoableEditListener undoListener) {

		abstractPanel.addUndoableEditListeners(undoListener);
		concretePanel.addUndoableEditListeners(undoListener);
		// TODO date stuff
		// dateField
	}

	/**
	 * Display every panel but the AgentPanel, and set the concrete button
	 * selected
	 */
	private void concreteSelected() {
		showConcretePanel();
		hideAbstractPanel();
		hideAgentPanel();
		
		concreteButton.setSelected(true);
		abstractButton.setSelected(false);
		agentButton.setSelected(false);
		concretePanel.fillInText();
		focusedPanel = concretePanel;

		if (cl != null) {
			cl.stateChanged(new ChangeEvent(focusedPanel));
		}
	}

	/**
	 * Display only the AbstractPanel and set the abstract button selected
	 */
	private void abstractSelected() {
		hideConcretePanel();
		hideAgentPanel();
		showAbstractPanel();
		
		abstractButton.setSelected(true);
		concreteButton.setSelected(false);
		agentButton.setSelected(false);
		abstractPanel.fillInText();
		focusedPanel = abstractPanel;
		if (cl != null) {
			cl.stateChanged(new ChangeEvent(focusedPanel));
		}
	}

	/**
	 * Display only the AgentPanel and set the abstract button selected
	 */
	private void agentSelected() {
		hideConcretePanel();
		hideAbstractPanel();
		showAgentPanel();
		abstractButton.setSelected(false);
		concreteButton.setSelected(false);
		agentButton.setSelected(true);
		agentPanel.fillInText();
		focusedPanel = agentPanel;
		if (cl != null) {
			cl.stateChanged(new ChangeEvent(focusedPanel));
		}
	}

	public int maxIndex() {
		return focusedPanel.maxIndex();
	}

	public int getIndexOfDisplayed() {
		return focusedPanel.getIndexOfDisplayed();
	}

	public String getStatusText() {
		return focusedPanel.getStatusText();
	}

	public void addChangeListener(ChangeListener tabListener) {
		cl = tabListener;
	}
}
