/**
 * 
 */
package airspaceGUI.base.room;

import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTabbedPane;

import airspaceGUI.expansion.FindWindow;

import edu.cmu.radar.base.Location;

/**
 * This is the class that provides functionality to find any room according to
 * room type, building, or name
 * 
 * @author srbennet
 * 
 */
public class FindRoom extends FindWindow
{
    private transient ArrayList searchList = new ArrayList();

    private transient RoomPanel roomPanel;

    private static final String ROOMTYPE = "Type";

    private static final String BUILDING = "Building";

    private static final String NAME = "Name";

    private static Object[] keywordList = {NAME, ROOMTYPE, BUILDING};

    /**
     * @param roomPanel
     *            the panel this find window is searching over the data of
     * @param searchList
     *            the list of objects to search
     */
    public FindRoom(RoomPanel roomPanel, List searchList)
    {
        super(keywordList);
        initMembers(roomPanel, searchList);
    }
    
    /**
     * @param roomPanel
     *            the panel this find window is searching over the data of
     * @param searchList
     *            the list of objects to search
     * @param tab The Tabbed Pane that the roomPanel is on, null if it is not in a tabbed pane
     */
    public FindRoom(RoomPanel roomPanel, List searchList, JTabbedPane tab)
    {
        super(keywordList, tab);
        /*
         * Don't uncomment unless we're unintegrating the find windows
         * setTitle("Find a Room");
         * setLocationRelativeTo(roomPanel);
         * 
         */
        initMembers(roomPanel, searchList);
    }

    /**
     * @param roomPanel
     * @param searchList
     */
    private void initMembers(RoomPanel roomPanel, List searchList) {
        this.roomPanel = roomPanel;
        parent = roomPanel;
        /*NOTE: don't try and replace this with .addAll or this.searchList = new ArrayList, 
         * because it replaces all the objects with nulls.
         */
        for(int i = 0; i < searchList.size(); i++)
        {
            this.searchList.add(searchList.get(i));
        }
    }
    

    /*
     * (non-Javadoc)
     * 
     * @see airspaceGUI.expansion.FindWindow#updatePanel(java.lang.Object)
     */
    protected void updatePanel(final Object wanted) {
        try
        {
            final Location selected = (Location)wanted;
            roomPanel.fillInText(selected);
            if (tabs != null)
            {
                tabs.setSelectedComponent(roomPanel);
            }
        } catch (ClassCastException e)
        {
            return;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see airspaceGUI.expansion.FindWindow#search(java.util.Map)
     */
    protected ArrayList search(final Map searchTerms) {
        final ArrayList results = new ArrayList();
        boolean isResult = true;

        for (int i = 0; i < searchList.size(); i++)
        {
            // Make sure all of the strings are searched on their lower-cased
            // versions to
            // eliminate problems with matching and capitolization
            if (searchTerms.get(ROOMTYPE) != null
                    && !((Location)searchList.get(i)).getType().toLowerCase()
                            .contains(
                                    ((String)searchTerms.get(ROOMTYPE))
                                            .toLowerCase()))
            {
                isResult = false;
            }

            if (searchTerms.get(BUILDING) != null
                    && !((Location)searchList.get(i)).getBuilding()
                            .toLowerCase().contains(
                                    ((String)searchTerms.get(BUILDING))
                                            .toLowerCase()))
            {
                isResult = false;
            }

            if (searchTerms.get(NAME) != null
                    && !((Location)searchList.get(i)).getCaption().toLowerCase().contains(
                            ((String)searchTerms.get(NAME)).toLowerCase()))
            {
                isResult = false;
            }

            if (isResult)
            {
                results.add(searchList.get(i));
            }
            // reset the result flag
            isResult = true;
        }
        return results;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList,
     *      java.lang.Object, int, boolean, boolean)
     */
    public Component getListCellRendererComponent(final JList list, final Object arg1,
            final int arg2, boolean isSelected, boolean arg4) {
        final JLabel label = new JLabel();

        label.setText(arg1.toString());

        if (isSelected)
        {
            label.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY));
            label.setOpaque(true);
            label.setBackground(list.getSelectionBackground());
            label.setForeground(list.getSelectionForeground());
        } else
        {
            label.setBorder(BorderFactory.createLineBorder(list.getBackground()));
            label.setOpaque(false);
            label.setBackground(list.getBackground());
            label.setForeground(list.getForeground());
        }
        label.setEnabled(list.isEnabled());
        label.setFont(list.getFont());

        return label;
    }

    public void updateSearchList(List search) {
       searchList = (ArrayList)search;
    }

}
