package airspaceGUI.base.room;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import airspaceGUI.expansion.ConstraintFactory;
import edu.cmu.radar.base.Location;
import edu.cmu.radar.base.LocationCapacity;

/**
 * Handles the viewing and editing of the LocationCapacities of a Location(aka
 * room).
 * 
 * @author srbennet
 * 
 */
public class RoomCapacityPanel extends JPanel implements DocumentListener {
	// The list of items that should go into the configurationBox (strings only)
	private Vector configurationList = new Vector();

	// the list of all the LocationCapacities related to the currently loaded
	// location
	private ArrayList locationCapacityList;

	// this is the location that is currently being shown in this panel
	private Location location;

	// The index in configurationList of the last item selected
	private int lastSelected = 0;

	// A label for the configuration ComboBox
	private JLabel configurationLabel = new JLabel("Configuration:");

	// A JComboBox to show all the configurations currently possible for this
	// room
	private JComboBox configurationBox = new JComboBox();

	// JButton to add new configrations to the list
	private JButton addButton = new JButton("+");

	// a label for the capacity text field
	private JLabel capacityLabel = new JLabel("Capacity:");

	// a text field in which the capacity of the selected room in the selected
	// configuration is shown
	private JTextField capacityField = new JTextField();

	/**
	 * the modification listener that is passed from BasePanel
	 * */
	private DocumentListener textModificationListener;

	/**
	 * if we are loading capacity, the new values in the capacity text field are
	 * not counted as user inputs
	 * */
	private boolean loadingCapacity;

	// A listener to save and change the capacityfield when the text in the
	// comfiguration box changes
	private final ItemListener configurationSelectionListener = new ItemListener() {

		public void itemStateChanged(ItemEvent ie) {
			loadingCapacity = true;
			
			if (ie.getStateChange() == ie.DESELECTED) {
				return;
			}

			if ((lastSelected > 0 && ((LocationCapacity) locationCapacityList
					.get(lastSelected)).getConfiguration().equals(ie.getItem()))
					|| !(configurationList.contains(configurationBox
							.getSelectedItem()))) {
				savePanel();
			} else {
				lastSelected = configurationBox.getSelectedIndex();
				capacityField.setText(((LocationCapacity) locationCapacityList
						.get(lastSelected)).getCapacity());
			}
			
			loadingCapacity = false;
		}

	};

	private final ActionListener addListener = new ActionListener() {

		public void actionPerformed(ActionEvent arg0) {
			// Make a NON-Modal window with a configuration and capacity boxes,
			// then put the results in the boxes on the main panel
			JDialog capacityDialog = new JDialog((Frame) ((JButton) arg0
					.getSource()).getTopLevelAncestor(),
					"Add a new configuration", true);
			Container panel = capacityDialog.getContentPane();
			JLabel configLabel = new JLabel("Configuration:");
			JLabel capaLabel = new JLabel("Capacity:");
			final JTextField configField = new JTextField();
			final JTextField capaField = new JTextField();
			JButton okButton = new JButton("Done");
			JButton cancelButton = new JButton("Cancel");
			GridBagConstraints gbc = new GridBagConstraints();
			ConstraintFactory f = new ConstraintFactory(8, 8, 8, 8, 8);

			capacityDialog.setLayout(new GridBagLayout());

			gbc = f.getConstraint(0, 0, ConstraintFactory.LABEL,
					ConstraintFactory.TOPMOST);
			panel.add(configLabel, gbc);
			gbc = f.getConstraint(1, 0, 3, ConstraintFactory.FIELD,
					ConstraintFactory.TOPMOST);
			gbc.fill = GridBagConstraints.HORIZONTAL;
			panel.add(configField, gbc);

			gbc = f.getConstraint(0, 1, ConstraintFactory.LABEL,
					ConstraintFactory.MIDDLE);
			panel.add(capaLabel, gbc);
			gbc = f.getConstraint(1, 1, 3, ConstraintFactory.FIELD,
					ConstraintFactory.MIDDLE);
			panel.add(capaField, gbc);

			gbc = f.getConstraint(1, 2, ConstraintFactory.LABEL,
					ConstraintFactory.BOTTOMMOST);
			gbc.anchor = GridBagConstraints.NORTHEAST;
			panel.add(cancelButton, gbc);
			gbc = f.getConstraint(2, 2, ConstraintFactory.FIELD,
					ConstraintFactory.BOTTOMMOST);
			gbc.anchor = GridBagConstraints.NORTHWEST;
			panel.add(okButton, gbc);

			okButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent arg0) {
					// get the capacities and configuration from the boxes
					LocationCapacity lc = new LocationCapacity();
					lc.setCapacity(capaField.getText());
					lc.setConfiguration(configField.getText());
					lc.setLocation(location);
					locationCapacityList.add(lc);
					configurationList.add(configField.getText());
					configurationBox.addItem(configField.getText());
					((JButton) arg0.getSource()).getTopLevelAncestor()
							.setVisible(false);
					configurationBox
							.setSelectedIndex(configurationList.size() - 1);
				}

			});

			cancelButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent arg0) {
					((JButton) arg0.getSource()).getTopLevelAncestor()
							.setVisible(false);
				}

			});

			capacityDialog.pack();
			capacityDialog.setResizable(false);
			capacityDialog.setVisible(true);
		}

	};

	public RoomCapacityPanel() {
		super();
		configurationBox.addItemListener(configurationSelectionListener);
		configurationBox.setEditable(false);
		addButton.setToolTipText("Add a new Configuration");
		addButton.addActionListener(addListener);
		capacityField.getDocument().addDocumentListener(this);
	}

	/**
	 * This returns a panel to edit the room capacities.
	 * 
	 * @return The JPanel to edit room capacities
	 * 
	 */
	public JPanel getPanel() {
		GridBagConstraints gbc = new GridBagConstraints();
		ConstraintFactory f = new ConstraintFactory(8, 8, 8, 8, 8);
		Dimension buttonSize = new Dimension(20, 20);
		this.setLayout(new GridBagLayout());

		gbc = f.getConstraint(0, 0, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		gbc.insets = new Insets(4, 14, 8, 4);
		this.add(configurationLabel, gbc);
		gbc = f.getConstraint(1, 0, 4, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		// gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridwidth = 4;
		// need the large right inset in order to show the JButton right next to
		// the ComboBox.
		gbc.insets = new Insets(0, 0, 0, 25);
		this.add(configurationBox, gbc);
		gbc = f.getConstraint(4, 0, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		gbc.fill = GridBagConstraints.NONE;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.insets = new Insets(-1, 1, 0, 0);
		this.add(addButton, gbc);
		addButton.setSize(buttonSize);
		addButton.setPreferredSize(buttonSize);
		// no really, we want the button to be buttonSize
		addButton.setMaximumSize(buttonSize);
		addButton.setMinimumSize(buttonSize);
		addButton.setHorizontalTextPosition(JButton.CENTER);
		addButton.setVerticalTextPosition(JButton.BOTTOM);
		addButton.setMargin(new Insets(5, 5, 5, 5));

		gbc = f.getConstraint(1, 1, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		gbc.insets = new Insets(4, 0, 0, 0);
		this.add(capacityLabel, gbc);
		gbc = f.getConstraint(2, 1, 3, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		gbc.insets = new Insets(5, 1, 1, 1);
		this.add(capacityField, gbc);

		return this;

	}

	/**
	 * Fills in the Room Capacity panel with the values saved in the location
	 * parameter
	 * 
	 * @param location
	 *            the location to get the capacities of
	 */
	public void fillInCapacities(Location location) {
		Iterator locationCapacities = location.getCapacities().iterator();
		locationCapacityList = new ArrayList();
		configurationList = new Vector();
		configurationBox.removeAllItems();
		this.location = location;
		// reset the lastSelected counter
		lastSelected = -1;
		LocationCapacity lc;

		while (locationCapacities.hasNext()) {
			lc = (LocationCapacity) locationCapacities.next();
			configurationList.add(lc.getConfiguration());
			locationCapacityList.add(lc);
			configurationBox.addItem(lc.getConfiguration());
		}

		if (configurationBox.getModel().getSize() > 0) {
			configurationBox.setSelectedIndex(0);
			configurationBox.setEnabled(true);
			addButton.setEnabled(true);
		} else {
			configurationBox.setEnabled(false);
			addButton.setEnabled(false);
			capacityField.setText("");
		}
	}

	/**
	 * This saves the locationCapacities back to the location they should be
	 * associated with.
	 * 
	 * @param location
	 *            the location to save the LocationCapacities into.
	 */
	public void saveCapacities(Location location) {
		savePanel();
		/*
		 * HashSet locationCapacitySet = new HashSet();
		 * 
		 * for (int i = 0; i < locationCapacityList.size(); i++) { if
		 * (!locationCapacitySet.contains(locationCapacityList.get(i))) {
		 * locationCapacitySet.add(locationCapacityList.get(i)); } }
		 * 
		 * location.setCapacities(locationCapacitySet);
		 */
	}

	/**
	 * This function saves the data which is currently on this panel.
	 * 
	 */
	private void savePanel() {
		Set<LocationCapacity> capacitySet = location.getCapacities();
		if (!(configurationList.contains(configurationBox.getSelectedItem()))) {
			// LocationCapacity lc =
			// ((LocationCapacity)locationCapacityList.get(lastSelected));
			// the user has entered a new configuration category
			String configBoxText = (String) configurationBox.getSelectedItem();
			// We cant create a new instance here; otherwise Hibernate throws
			// exception for non-unique objects
			// LocationCapacity lc = new LocationCapacity(
			// location, configBoxText, capacityField.getText());
			// locationCapacityList.add(lc);
			// configurationList.add(configurationBox.getSelectedItem());
			for (LocationCapacity capacity : capacitySet) {
				if (capacity.getConfiguration().equals(configBoxText)) {
					capacity.setCapacity(capacityField.getText());
				}
			}
		} else if (lastSelected >= 0) {
			// this is the event that means this element is being de-selected,
			// so save the
			// capacity value
			((LocationCapacity) locationCapacityList.get(lastSelected))
					.setCapacity(capacityField.getText());
		}
	}

	/**
	 * adds modification listeners
	 * */
	public void addModificationListener(DocumentListener docListener) {
		textModificationListener = docListener;
	}

	public void changedUpdate(DocumentEvent e) {
		if (!loadingCapacity) {
			textModificationListener.changedUpdate(e);
		}
	}

	public void insertUpdate(DocumentEvent e) {
		if (!loadingCapacity) {
			textModificationListener.changedUpdate(e);
		}
	}

	public void removeUpdate(DocumentEvent e) {
		if (!loadingCapacity) {
			textModificationListener.changedUpdate(e);
		}
	}
}
