package airspaceGUI.base.room;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;

import org.hibernate.Session;

import airspaceGUI.base.BasePanel;
import airspaceGUI.expansion.ConstraintFactory;
import airspaceGUI.expansion.FindWindow;
import airspaceGUI.expansion.NumberSpinnerWithNull;
import airspaceGUI.model.RoomModel;
import edu.cmu.radar.base.Location;

/**
 * This is used to create, delete, and update data about rooms.
 * 
 * @author srbennet
 * 
 */
public class RoomPanel extends BasePanel {

	private static Integer negativeOne = new Integer(-1);

	/**
     * 
     */
	private static long serialVersionUID = 1L;

	private JLabel captionLabel = new JLabel("Name: ");

	private JTextField captionField = new JTextField(); // --

	// e.g.
	// WEAN
	// 5301

	// eid TEXT, -- NameWithSpacesRemoved (identifier for STP and other things)

	private JTextField xCoordField = new JTextField(); // INTEGER,

	// //Figure
	// out
	// what
	// to do
	// with
	// these

	private JLabel xCoordLabel = new JLabel("X coordinate:");

	private JTextField yCoordField = new JTextField(); // INTEGER,

	private JLabel yCoordLabel = new JLabel("Y coordinate:");

	private JTextField zCoordField = new JTextField(); // INTEGER,

	// --
	// x,y,z
	// coords
	// listed
	// in
	// CMRadar
	// example

	private JLabel zCoordLabel = new JLabel("Z coordinate:");

	private JLabel buildingLabel = new JLabel("Building:");

	private JTextField buildingField = new JTextField();// --

	// building
	// name,
	// e.g.
	// WEAN,
	// Flagstaf
	// Hotel

	private JLabel floorLabel = new JLabel("Floor:");

	private JTextField floorField = new JTextField(); // TEXT,

	// --
	// floor
	// of
	// building
	// 'CL',
	// 'G',
	// '1',
	// etc.

	// -- spacetime converts to an integer

	private JLabel sqFootageLabel = new JLabel("Square Footage:");

	private JTextField sqFootageField = new JTextField(); // INTEGER
	// ,

	// --
	// square
	// feet

	private JLabel widthLabel = new JLabel("Width (feet):");

	private JTextField widthField = new JTextField(); // INTEGER,

	// --
	// feet

	private JLabel lengthLabel = new JLabel("Length (feet):");

	private JTextField lengthField = new JTextField(); // INTEGER,

	// --
	// feet

	private JLabel heightLabel = new JLabel("Height (feet):");

	private JTextField heightField = new JTextField(); // INTEGER,

	// --
	// feet

	private JLabel typeLabel = new JLabel("Type:");

	private JTextField typeField = new JTextField(); // --

	// classroom,
	// auditorium,
	// conference,
	// reception

	// -- this determines which vague stats are
	// -- provided to STP
	// Note: figure out if there is a central location for the possible types of
	// rooms

	private JLabel costLabel = new JLabel("Cost:");

	private JTextField costField = new JTextField(); // INTEGER,

	// --
	// dollars
	// to
	// rent
	// per
	// hour

	// -- spacetime converts to non-positive int
	// -- or to a pud?
	// private final /*NumberSpinnerWithNull*/SpinnerNumberModel numberModel =
	// new /*NumberSpinnerWithNull*/SpinnerNumberModel();

	private JLabel micLabel = new JLabel("Number of Microphones:");

	private SpinnerNumberModel micModel = new SpinnerNumberModel();

	private NumberSpinnerWithNull micField = new NumberSpinnerWithNull(micModel);// INTEGER
																					// ,

	private JLabel podiumLabel = new JLabel("Number of Podiums:");

	private SpinnerNumberModel podiumModel = new SpinnerNumberModel();

	private NumberSpinnerWithNull podiumField = new NumberSpinnerWithNull(
			podiumModel);// INTEGER,

	private JLabel projectorLabel = new JLabel("Number of Projectors:");

	private SpinnerNumberModel projectorModel = new SpinnerNumberModel();

	private NumberSpinnerWithNull projectorField = new NumberSpinnerWithNull(
			projectorModel);// INTEGER,

	private JLabel stationLabel = new JLabel("Number of Stations:");

	private SpinnerNumberModel stationModel = new SpinnerNumberModel();

	private NumberSpinnerWithNull stationField = new NumberSpinnerWithNull(
			stationModel); // INTEGER,

	private JLabel computerLabel = new JLabel("Number of Computers:");

	private SpinnerNumberModel computerModel = new SpinnerNumberModel();

	private NumberSpinnerWithNull computerField = new NumberSpinnerWithNull(
			computerModel);// INTEGER

	// possibly change to a bank of spinners?

	private RoomCapacityPanel capacityPanel = new RoomCapacityPanel();

	public RoomPanel(final JTabbedPane tab, RoomModel roomModel, Session session) {
		super();
		setSession(session);
		nextButton.setText("Next Room");
		prevButton.setText("Previous Room");
		newButton.setText("New Room");
		deleteButton.setText("Delete Room");
		statusLabel = new JLabel("I am the status label");

		setDataList(roomModel);
		findWindow = new FindRoom(RoomPanel.this, getDataList(), tab);

		micField.setValue(null);
		// set the initial values of all the fields, to make sure they're
		// initialized
		podiumField.setValue(negativeOne);
		projectorField.setValue(negativeOne);
		stationField.setValue(negativeOne);
		computerField.setValue(negativeOne);
		micModel.setMinimum(negativeOne);
		podiumModel.setMinimum(negativeOne);
		projectorModel.setMinimum(negativeOne);
		stationModel.setMinimum(negativeOne);
		computerModel.setMinimum(negativeOne);
		addModificationListeners(textFieldModificationListener,
				comboBoxModificationListener, checkBoxModificationListener);
	}

	/**
	 * saves the values in the text fields to the object
	 * 
	 * @return The Location that was saved.
	 */
	public Object saveText() {
		Location loc = (Location) getDataList().get(getIndexOfDisplayed());
		saveText(loc);

		return loc;
	}

	/**
	 * This saves the text that is displayed on the GUI back to the data
	 * structure in the parameter
	 * 
	 * @param location
	 *            The location to save the data to.
	 * @return true if the object is empty, false otherwise
	 */
	protected boolean saveText(Location location) {
		boolean empty = true;
		location.setCaption(captionField.getText());
		location.setEid(location.getCaption().replaceAll(" ", ""));
		location.setBuilding(buildingField.getText());
		location.setFloor(floorField.getText());

		capacityPanel.saveCapacities(location);

		try {

			// if the field is blank, set the corresponding variable in the
			// location to null.
			if (!sqFootageField.getText().equals("")) {
				location.setFootprint(new Integer(sqFootageField.getText()));
				empty = false;
			} else {
				location.setFootprint(null);
			}

			if (!xCoordField.getText().equals("")) {
				location.setX(new Integer(xCoordField.getText()));
				empty = false;
			} else {
				location.setX(null);
			}

			if (!yCoordField.getText().equals("")) {
				location.setY(new Integer(yCoordField.getText()));
				empty = false;
			} else {
				location.setY(null);
			}

			if (!zCoordField.getText().equals("")) {
				location.setZ(new Integer(zCoordField.getText()));
				empty = false;
			} else {
				location.setZ(null);
			}

			if (!widthField.getText().equals("")) {
				location.setWidth(new Integer(widthField.getText()));
				empty = false;
			} else {
				location.setWidth(null);
			}

			if (!lengthField.getText().equals("")) {
				location.setLength(new Integer(lengthField.getText()));
				empty = false;
			} else {
				location.setLength(null);
			}

			if (!heightField.getText().equals("")) {
				location.setHeight(new Integer(heightField.getText()));
				empty = false;
			} else {
				location.setHeight(null);
			}
			location.setType(typeField.getText());

			if (!costField.getText().equals("")) {
				location.setCost(new Integer(costField.getText()));
				empty = false;
			} else {
				location.setCost(null);
			}
		} catch (NumberFormatException e) {
			e.printStackTrace();
		}

		if (micField.getValue() instanceof Integer
				&& ((Integer) micField.getValue()).intValue() >= 0) {
			location.setMicrophones((Integer) micField.getValue());
			empty = false;
		} else {
			location.setMicrophones(null);
		}

		if (podiumField.getValue() instanceof Integer
				&& ((Integer) podiumField.getValue()).intValue() >= 0) {
			location.setPodiums((Integer) podiumField.getValue());
			empty = false;
		} else {
			location.setPodiums(null);
		}

		if (projectorField.getValue() instanceof Integer
				&& ((Integer) projectorField.getValue()).intValue() >= 0) {
			location.setProjector((Integer) projectorField.getValue());
			empty = false;
		} else {
			location.setProjector(null);
		}

		if (stationField.getValue() instanceof Integer
				&& ((Integer) stationField.getValue()).intValue() >= 0) {
			location.setStations((Integer) stationField.getValue());
			empty = false;
		} else {
			location.setStations(null);
		}

		if (computerField.getValue() instanceof Integer
				&& ((Integer) computerField.getValue()).intValue() >= 0) {
			location.setComputers((Integer) computerField.getValue());
			empty = false;
		} else {
			location.setComputers(null);
		}

		return empty;
	}

	public boolean load(Session session) {
		setDataList(load(session, Location.class, "caption"));
		return !(getDataList().isEmpty());
	}

	public boolean save() {
		if (getDataList() != null) {
			Location loc = (Location) getDataList().get(getIndexOfDisplayed());
			saveText(loc);
			super.persist(loc);
			return true;
		}

		return false;
	}

	public JPanel getPane() {
		JPanel attributesPanel = new JPanel();
		JPanel dimensionPanel = new JPanel();
		JPanel accessoryPanel = new JPanel();
		this.setName("Room");
		// set the layout of all 4 panels
		this.setLayout(new GridBagLayout());
		attributesPanel.setLayout(new GridBagLayout());
		dimensionPanel.setLayout(new GridBagLayout());
		accessoryPanel.setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		ConstraintFactory cf = new ConstraintFactory(8, 8, 8, 8, 2);
		int row = 0;
		int col = 0;

		// create borders around the individual panels to break up the page a
		// bit
		accessoryPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(), "Accessories:"));
		dimensionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(), "Dimensions:"));
		attributesPanel.setBorder(BorderFactory.createTitledBorder(
				BorderFactory.createEtchedBorder(), "Attributes:"));

		// fill the attcibutes panel
		gbc.fill = GridBagConstraints.BOTH;
		gbc = cf.getConstraint(col++, row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		attributesPanel.add(captionLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		attributesPanel.add(captionField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		attributesPanel.add(buildingLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		attributesPanel.add(buildingField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		attributesPanel.add(floorLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		attributesPanel.add(floorField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(sqFootageLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(sqFootageField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(xCoordLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(xCoordField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(yCoordLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(yCoordField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(zCoordLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(zCoordField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(widthLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(widthField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(lengthLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(lengthField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(heightLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(heightField, gbc);

		gbc = cf.getConstraint(col, ++row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		dimensionPanel.add(capacityPanel.getPanel(), gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		attributesPanel.add(typeLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		attributesPanel.add(typeField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		attributesPanel.add(costLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		attributesPanel.add(costField, gbc);

		// attributesPanel done
		// start adding elements to the accessory panel
		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		accessoryPanel.add(micLabel, gbc);
		gbc = cf.getConstraint(col++, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		accessoryPanel.add(micField, gbc);

		gbc = cf.getConstraint(col++, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		accessoryPanel.add(podiumLabel, gbc);
		gbc = cf.getConstraint(col++, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		accessoryPanel.add(podiumField, gbc);

		// manually reset the column so that we can have 2 items on a row.
		col = 0;
		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		accessoryPanel.add(projectorLabel, gbc);
		gbc = cf.getConstraint(col++, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		accessoryPanel.add(projectorField, gbc);

		gbc = cf.getConstraint(col++, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		accessoryPanel.add(stationLabel, gbc);
		gbc = cf.getConstraint(col++, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		accessoryPanel.add(stationField, gbc);

		// manually reset the column so that we can have 2 items on a row.
		col = 0;
		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		accessoryPanel.add(computerLabel, gbc);
		gbc = cf.getConstraint(col++, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		accessoryPanel.add(computerField, gbc);

		// add all the sub-panels to the big panel
		row = 0;
		gbc = cf.getConstraint(0, 0, 3, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		this.add(attributesPanel, gbc);
		gbc = cf.getConstraint(0, 1, 3, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		this.add(dimensionPanel, gbc);
		gbc = cf.getConstraint(0, 2, 3, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		this.add(accessoryPanel, gbc);

		gbc = cf.getConstraint(0, 3, 2, ConstraintFactory.LABEL,
				ConstraintFactory.BOTTOMMOST);
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.insets = new Insets(0, 8, 7, 0);
		this.add(getButtonPanel(), gbc);

		// add all the focus listeners to enable undo/redo
		captionField.addFocusListener(this);
		buildingField.addFocusListener(this);
		floorField.addFocusListener(this);
		sqFootageField.addFocusListener(this);
		xCoordField.addFocusListener(this);
		yCoordField.addFocusListener(this);
		zCoordField.addFocusListener(this);
		widthField.addFocusListener(this);
		lengthField.addFocusListener(this);
		heightField.addFocusListener(this);
		typeField.addFocusListener(this);
		costField.addFocusListener(this);
		podiumField.addFocusListener(this);
		projectorField.addFocusListener(this);
		stationField.addFocusListener(this);
		computerField.addFocusListener(this);

		return this;
	}

	public void fillInText(Session session) {
		if (getSession() == null) {
			load(session);
		}

		((JFrame) getTopLevelAncestor()).setTitle("Edit a room");
		if (getDataList() != null) {
			fillInText((Location) getDataList().get(getIndexOfDisplayed()));
		} else {
			return;
		}
	}

	public void fillInText(Object object) {
		if (object instanceof Location) {
			fillInText((Location) object);
		}
	}

	/**
	 * Fills in the fields on the GUI with the data contained in the parameter
	 * room.
	 * 
	 * @param room
	 *            The data to fill in the fields with
	 */
	protected void fillInText(Location room) {
		captionField.setText(room.getCaption());
		buildingField.setText(room.getBuilding());
		floorField.setText(room.getFloor());
		if (room.getFootprint() != null) {
			sqFootageField.setText(room.getFootprint().toString());
		} else {
			sqFootageField.setText("");
		}

		if (room.getX() != null) {
			xCoordField.setText(room.getX().toString());
		} else {
			xCoordField.setText("");
		}

		if (room.getY() != null) {
			yCoordField.setText(room.getY().toString());
		} else {
			yCoordField.setText("");
		}

		if (room.getZ() != null) {
			zCoordField.setText(room.getZ().toString());
		} else {
			zCoordField.setText("");
		}

		if (room.getWidth() != null) {
			widthField.setText(room.getWidth().toString());
		} else {
			widthField.setText("");
		}

		if (room.getLength() != null) {
			lengthField.setText(room.getLength().toString());
		} else {
			lengthField.setText("");
		}

		if (room.getHeight() != null) {
			heightField.setText(room.getHeight().toString());
		} else {
			heightField.setText("");
		}
		// be sure to show the capacities
		capacityPanel.fillInCapacities(room);

		typeField.setText(room.getType());

		if (room.getCost() != null) {
			costField.setText(room.getCost().toString());
		} else {
			costField.setText("");
		}

		if (room.getPodiums() != null) {
			podiumField.setValue(room.getPodiums());
		} else {
			podiumField.setValue(negativeOne);
		}

		if (room.getProjector() != null) {
			projectorField.setValue(room.getProjector());
		} else {
			projectorField.setValue(negativeOne);
		}

		if (room.getStations() != null) {
			stationField.setValue(room.getStations());
		} else {
			stationField.setValue(negativeOne);
		}

		if (room.getComputers() != null) {
			computerField.setValue(room.getComputers());
		} else {
			computerField.setValue(negativeOne);
		}

		// make sure the getIndexOfDisplayed() is set correctly for this room.
		setIndexOfDisplayed(getDataList().indexOf(room));
		setButtonPermissions();
		this.setStatus(getIndexOfDisplayed() + 1);

		// this statement causes the bug that changes in the previous pages are
		// not saved.
		// this.setIsModified(false);
	}

	/*
	 * private void displayCapacities(Location room) { // TODO Auto-generated
	 * method stub }
	 */

	public void createNewInstance() {
		clearPanel();
		
		getDataList().add(getIndexOfDisplayed(), new Location());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#clearPanel()
	 */
	public void clearPanel() {
		captionField.setText("");
		buildingField.setText("");
		floorField.setText("");
		sqFootageField.setText("");
		xCoordField.setText("");
		yCoordField.setText("");
		zCoordField.setText("");
		widthField.setText("");
		lengthField.setText("");
		heightField.setText("");
		typeField.setText("");
		costField.setText("");
		micField.setValue(null);
		podiumField.setValue(null);
		projectorField.setValue(null);
		stationField.setValue(null);
		computerField.setValue(null);

		captionField.requestFocusInWindow();
	}

	/**
	 * Sets the status label to report to the user what piece of data they're on
	 * out of the total number of pieces of data
	 * 
	 * @param numOn
	 *            The index number of the piece of data + 1
	 * 
	 */
	protected void setStatus(int numOn) {
		statusLabel.setText("On room " + numOn + " of " + getDataList().size()
				+ " rooms");
	}

	public void gotoNumber(int number) {
		saveText((Location) getDataList().get(getIndexOfDisplayed()));
		fillInText((Location) getDataList().get(number));
		setIndexOfDisplayed(number);
		setStatus(++number);
	}

	public String getTitle() {
		return "Edit Room";
	}

	protected void addModificationListeners(
			DocumentListener textFieldModificationListener,
			ItemListener checkBoxModificationListener,
			ChangeListener comboBoxModificationListener) {
		captionField.getDocument().addDocumentListener(
				textFieldModificationListener);
		buildingField.getDocument().addDocumentListener(
				textFieldModificationListener);
		floorField.getDocument().addDocumentListener(
				textFieldModificationListener);
		sqFootageField.getDocument().addDocumentListener(
				textFieldModificationListener);
		xCoordField.getDocument().addDocumentListener(
				textFieldModificationListener);
		yCoordField.getDocument().addDocumentListener(
				textFieldModificationListener);
		zCoordField.getDocument().addDocumentListener(
				textFieldModificationListener);
		widthField.getDocument().addDocumentListener(
				textFieldModificationListener);
		lengthField.getDocument().addDocumentListener(
				textFieldModificationListener);
		heightField.getDocument().addDocumentListener(
				textFieldModificationListener);
		typeField.getDocument().addDocumentListener(
				textFieldModificationListener);
		costField.getDocument().addDocumentListener(
				textFieldModificationListener);
		micField.addChangeListener(comboBoxModificationListener);
		podiumField.addChangeListener(comboBoxModificationListener);
		projectorField.addChangeListener(comboBoxModificationListener);
		stationField.addChangeListener(comboBoxModificationListener);
		computerField.addChangeListener(comboBoxModificationListener);

		capacityPanel.addModificationListener(textFieldModificationListener);
	}

	protected void addUndoableEditListeners(UndoableEditListener undoListener) {
		captionField.getDocument().addUndoableEditListener(undoListener);
		buildingField.getDocument().addUndoableEditListener(undoListener);
		floorField.getDocument().addUndoableEditListener(undoListener);
		sqFootageField.getDocument().addUndoableEditListener(undoListener);
		xCoordField.getDocument().addUndoableEditListener(undoListener);
		yCoordField.getDocument().addUndoableEditListener(undoListener);
		zCoordField.getDocument().addUndoableEditListener(undoListener);
		widthField.getDocument().addUndoableEditListener(undoListener);
		lengthField.getDocument().addUndoableEditListener(undoListener);
		heightField.getDocument().addUndoableEditListener(undoListener);
		typeField.getDocument().addUndoableEditListener(undoListener);
		costField.getDocument().addUndoableEditListener(undoListener);
		/*
		 * look into how/if we need to attach undo modificationlisteners
		 * micField.setValue(null); podiumField.setValue(null);
		 * projectorField.setValue(null); stationField.setValue(null);
		 * computerField.setValue(null);
		 */
	}

	public FindWindow getFindWindow() {
		return findWindow;
	}

}
