package airspaceGUI.base.test;

import java.awt.event.KeyEvent;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Hashtable;
import java.util.List;

import javax.swing.JFrame;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.hibernate.ReplicationMode;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import junit.framework.Test;
import junit.framework.TestSuite;
import airspaceGUI.base.corpus.CorpusGUIMessage;
import airspaceGUI.base.corpus.gui.CorpusMainWindow;
import airspaceGUI.model.PeopleModel;
import edu.cmu.radar.base.corpus.FakeMessage;
import edu.cmu.radar.base.corpus.Variable;

public class CorpusMainWindowTest extends BasePanelTest
{

    JFrame frame = new JFrame();
    
    protected void setUp() throws Exception {
        super.setUp();
        if (!session.isOpen())
        {
            session.beginTransaction();
        }
        
        
        if (thisPanel == null)
        {
            if (!this.getClass().equals(DataKeeper.lastClass))
            {
                DataKeeper.peopleModel = new PeopleModel(session);
                DataKeeper.lastClass = this.getClass();
            }
            
            thisPanel = new CorpusMainWindow(frame, null, session, DataKeeper.peopleModel);
            thisPanel.load(session);
            frame.getContentPane().add(thisPanel.getPane());
            frame.pack();
            frame.setVisible(true);
        }
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        frame.setVisible(false);
    }
    
    public static Test suite() {
        TestSuite suite= new TestSuite("CorpusGUI Tests");

        //add individual test classes
        suite.addTestSuite(CorpusMainWindowTest.class);
        
        return suite;
    }

    public void testAddModificationListeners() {
        super.testAddModificationListeners();
    }

    public void testAddUndoableEditListeners() {
        super.testAddUndoableEditListeners();
    }

    public void testDeleteDisplayedObject() {
        thisPanel.fillInText(session);
        
        CorpusGUIMessage onPanel = (CorpusGUIMessage)thisPanel.saveText();
        
        thisPanel.deleteDisplayedObject();
        
        Object differentObject = thisPanel.saveText();
        
        assertFalse(onPanel.equals(differentObject));
        
        session.replicate(onPanel.getMessage(), ReplicationMode.OVERWRITE);

        session.getTransaction().commit();
        session.beginTransaction();
    }

    public void testExit() {
        super.testExit();
    }

    public void testSaveText() {
        super.testSaveText();
    }

    public void testClearText() {
        super.testClearText();
    }

    public void testFillInTextObject() {
        super.testFillInTextObject();
    }

    public void testSetStatus() {
        super.testSetStatus();
    }

    public void testSetMenuBar() {
        super.testSetMenuBar();
    }

    public void testGetFindWindow() {
        super.testGetFindWindow();
    }


    public void testFillInTextCorpusGUIMessage() {
        try
        {
            Method method = Class.forName("airspaceGUI.base.SubPanel").getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);
            
            CorpusGUIMessage toBeDisplayed = (CorpusGUIMessage)l.get(thisPanel.getIndexOfDisplayed());
            
            thisPanel.fillInText(toBeDisplayed);
            assertTrue(thisPanel.saveText().equals(toBeDisplayed));
            
            
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } 
    }

    public void testMakeBlank() {
        super.testClearText();
    }

    public void testSave() {
        super.testSave();
    }

    public void testSaveForInjector() {
        Method getDataList;
        try
        {
            getDataList = Class.forName("airspaceGUI.base.SubPanel")
                .getDeclaredMethod("getDataList", null);
            Class[] params = {List.class};
            Method brokenXML = thisPanel.getClass().getDeclaredMethod("writeBrokenXMLMessages",
                    params);
            getDataList.setAccessible(true);
            brokenXML.setAccessible(true);
            List l = (List)getDataList.invoke(thisPanel, null);
        //CorpusMainWindow cmw = (CorpusMainWindow)thisPanel;
        
            Object[] parameters = {l};
            //write out all messages into xml
            String xml = (String)brokenXML.invoke(thisPanel, parameters);
            
            final DocumentBuilderFactory docBF = DocumentBuilderFactory.newInstance();

            final DocumentBuilder docB = docBF.newDocumentBuilder();
            Document doc = null;
            
            doc = docB.parse(new ByteArrayInputStream( xml.getBytes()));
        //figure out how to navigate the windows that are needed to actually
        //save and validate the file for the injector
        
            //ensure the xml string conforms to the injector's bugs
            assertFalse(xml.contains("&lt;"));
            assertFalse(xml.contains("&gt;"));
            
            //can't quite say this because these characters are valid in the subject and 
            //body tags... but they shouldn't be anywhere else
            //TODO figure out how to validate based on those constraints1
            //assertFalse(xml.contains("&amp;"));
        
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } catch (ParserConfigurationException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
            fail("failed to parse");
        } catch (SAXException e)
        {
            // TODO figure out the text of the expected exceptions
            e.printStackTrace();
        } catch (IOException e)
        {
            // really shouldn't get here considering we're not 
            // really dealing with files
            e.printStackTrace();
            fail("failed IO");
        } 
    }

    /**
     * tests that the storing of state is determinanistic, that
     * the same panel state will return the same Hashtable.
     *
     */
    public void testStoreState() {
        CorpusMainWindow cmw = (CorpusMainWindow)thisPanel;
        Hashtable stored = new Hashtable();
        Hashtable newStore = new Hashtable();
        
        cmw.storeState(stored);
        
        this.typeKey(KeyEvent.VK_1);
        
        this.typeKey(KeyEvent.VK_DELETE);
        
        cmw.storeState(newStore);
        
        assertEquals(stored, newStore);
        
        //fail("Not yet implemented"); // TODO
    }

    /**
     * tests that when the state is stored, it can be returned 
     *
     */
    public void testRestoreState() {
        CorpusMainWindow cmw = (CorpusMainWindow)thisPanel;
        //make sure the panel is filled with _something_
        cmw.fillInText(session);
        Hashtable stored = new Hashtable();
        Hashtable newStore = new Hashtable();
        //turn it into a string so that any later modifications to the object
        //can't effect this representation
        Object saved = cmw.saveText();
        
        cmw.storeState(stored);
        cmw.restoreState(stored);

        Object newSaved = cmw.saveText();
        
        assertEquals(saved, newSaved);
        
        // since these functions will only be used when changing from one
        //message to another, we replicate that behavior here
        this.pressNextButton();
        
        newSaved = cmw.saveText().toString();
        
        assertFalse(newSaved.equals(saved));
        
        cmw.storeState(newStore);
        
        
        assertFalse(stored.equals(newStore));
        
        //ensure it can actually restore when something has changed
        
        cmw.restoreState(stored);
        
        newSaved = cmw.saveText();
        assertEquals(saved, newSaved);
        
        
    }

    public void testGotoNumber() {
        super.testGotoNumber();
    }

    public void testGetPane() {
        // again, this might be ovbious, and will be 
        // tested multiple times on other tests
        super.testGetPane();
    }

    public void testFillInTextSession() {
        super.testFillInTextSession();
    }

    public void testGetStatusText() {
        super.testGetStatusText();
    }

    public void testGetButtonPanel() {
        // don't think this is necessary, because 
        //if you're missing a buttonpanel, it's ovbious
        super.testGetButtonPanel();
    }

    public void testSetButtonPermissions() {
        super.testSetButtonPermissions();
    }

    public void testMaxIndex() {
        super.testMaxIndex();
    }

    public void testGetDataList() {
        super.testGetDataList();
    }

    public void testGetIndexOfDisplayed() {
        super.testGetIndexOfDisplayed();
    }

    public void testGetSession() {
        super.testGetSession();
    }

    public void testGetHasFocus() {
        super.testGetHasFocus();
    }

    public void testSetHasFocus() {
        super.testSetHasFocus();
    }

    public void testSetDataList() {
        super.testSetDataList();
    }

    public void testSetIndexOfDisplayed() {
        super.testSetIndexOfDisplayed();
    }

    public void testSetSession() {
        super.testSetSession();
    }

    public void testLoadSessionClassString() {
        List list = thisPanel.load(session, FakeMessage.class, "id");
        assertNotNull(list);
        FakeMessage lastFakeMessage = (FakeMessage)list.get(0);
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof FakeMessage);
            assertTrue(lastFakeMessage.getId().intValue() < ((FakeMessage)list.get(i)).getId().intValue());
        }
        
        list = thisPanel.load(session, FakeMessage.class, "messageId");
        assertNotNull(list);
        
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof FakeMessage);
            // check that the messageid value is increasing
        }
        
        list = thisPanel.load(session, FakeMessage.class, "messageGroup");
        assertNotNull(list);
        
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof FakeMessage);
            // check that the messagegroup value is increasing
        }
        
        list = thisPanel.load(session, FakeMessage.class, "subject");
        assertNotNull(list);
        
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof FakeMessage);
            // check that the subject value is increasing
        }
        
        list = thisPanel.load(session, Variable.class, "name");
        assertNotNull(list);
        
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Variable);
            // check that the namevalue is increasing
        }
    }

    public void testLoadSessionClass() {
        List list = thisPanel.load(session, FakeMessage.class);
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof FakeMessage);
        }
        
        list = thisPanel.load(session, Variable.class);
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Variable);
        }
    }

    public void testIsModified() {
        super.testIsModified();
    }

    public void testSetIsModified() {
        super.testSetIsModified();
    }
    
    public void testSetTitle() {
        String origName = thisPanel.getTitle();
        
        String newTitle = "newTitle";
        
        try{
            Class[] classList = {String.class};
            Method method = Class.forName("airspaceGUI.base.corpus.gui.CorpusMainWindow")
                .getDeclaredMethod("setTitle", classList);
            method.setAccessible(true);
            Object[] params = {newTitle};
            method.invoke(thisPanel, params);
        
           /* System.out.println("newTitle " + newTitle);
            System.out.println("getTitle " + thisPanel.getTitle());*/
            
            assertTrue(thisPanel.getTitle().equals(newTitle));
            
            Object[] newParams = {origName};
            method.invoke(thisPanel, newParams);
        
        }catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } 
          
    }

    public void testSaveList() {
        //only here so it doesn't try the save(List) method in subPanel
        // because it doesn't understand corpusGUIMessages
        //TODO fix this
    }

}
