package airspaceGUI.base.test;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;

import org.hibernate.ReplicationMode;

import junit.framework.Test;
import junit.framework.TestSuite;
import airspaceGUI.base.event.EventPanel;
import airspaceGUI.model.AgentModel;
import airspaceGUI.model.GatheringModel;
import airspaceGUI.model.MeetingModel;
import airspaceGUI.model.PeopleModel;
import airspaceGUI.model.RoomModel;
import edu.cmu.radar.base.Gathering;
import edu.cmu.radar.base.cmradar.AbstractMeeting;
import edu.cmu.radar.base.cmradar.Agent;
import edu.cmu.radar.base.cmradar.ConcreteMeeting;


/**
 * This class's jUnit tests only validate that this panel itself is 
 * returning the correct things, with no reguard for whether subpanels
 * themselves will return the right things.
 * @author srbennet
 *
 */
public class EventPanelCmradarTest extends BasePanelTest
{

    JFrame frame = new JFrame();

//    private static FlushMode flushMode;
    
    public static Test suite() {
        TestSuite suite= new TestSuite("CorpusGUI Tests");

        //add individual test classes
        suite.addTestSuite(EventPanelCmradarTest.class);
        
        return suite;
    }

    protected void setUp() throws Exception {
        super.setUp();
        //save the flush mode to restore later
//        flushMode = session.getFlushMode();
        if (thisPanel == null)
        {
            if (!this.getClass().equals(DataKeeper.lastClass)){
                //System.out.println("new meeting");
                DataKeeper.meetingModel = new MeetingModel(session);
                //System.out.println("new gathering");
                DataKeeper.gatheringModel = new GatheringModel(session);
               // System.out.println("new people");
                DataKeeper.peopleModel = new PeopleModel(session);
               // System.out.println("new room");
                DataKeeper.roomModel = new RoomModel(session);
              // System.out.println("new agent");
                DataKeeper.agentModel = new AgentModel(session);
                
                DataKeeper.lastClass = this.getClass();
            }
            thisPanel = new EventPanel(session, null, DataKeeper.agentModel,
                    DataKeeper.meetingModel, DataKeeper.gatheringModel, DataKeeper.peopleModel, DataKeeper.roomModel);
            frame.getContentPane().add(thisPanel.getPane());
            frame.pack();
        }
        frame.setVisible(true);
//        session.setFlushMode(FlushMode.ALWAYS);
//        session.setCacheMode(CacheMode.NORMAL);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        frame.setVisible(false);
    }

    public void testAddModificationListeners() {
        super.testAddModificationListeners();
    }

    public void testAddUndoableEditListeners() {
        super.testAddUndoableEditListeners();
    }

    public void testMaxIndex() {
        //depends on what panelyou're on, so we need to test them all
        //
        assertTrue(true);
        //this test is irrelevant, you can't get something " +
         //       "\"correct\" to test the return of the maxIndex function against"
        //super.testMaxIndex();
    }

    public void testDeleteDisplayedObject() {
        //fail("This test will always fail until cmradar is fixed");
        AbstractMeeting saved = null;
        if (thisPanel.saveText() instanceof AbstractMeeting)
        {
            saved = (AbstractMeeting)thisPanel.saveText();
        }
        super.testDeleteDisplayedObject();
        
        session.replicate(saved, ReplicationMode.IGNORE);
    }

    public void testGetStatusText() {
        super.testGetStatusText();
    }

    public void testGetIndexOfDisplayed() {
        super.testGetIndexOfDisplayed();
    }

    public void testSaveText() {
        super.testSaveText();
    }

    public void testClearText() {
        super.testClearText();
    }

    public void testFillInTextObject() {
        
        try
        {
            Method method = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);
            boolean displayed = false;

            Object toBeDisplayed;
            List seenClasses = new ArrayList();

            for(int i = 0; i < l.size(); i++)
            {
                toBeDisplayed = l.get(i);
                //make sure that there are at least one object in the set that can be displayed
                //correctly
                if (!seenClasses.contains(toBeDisplayed)){
                    seenClasses.add(toBeDisplayed.getClass());
                    thisPanel.fillInText(toBeDisplayed);
                    displayed = displayed || thisPanel.saveText().equals(toBeDisplayed);
                    
                }
            }
            assertTrue(displayed);

        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }

    }

    public void testGetMinimumSize() {
        super.testGetMinimumSize();
        //fail("Not yet implemented"); // TODO
    }

    public void testSetStatus() {
        //don't do anything for this test, because it doesn't make sense
    }

    public void testGetFindWindow() {
        super.testGetFindWindow();
    }

    public void testGetPane() {
        super.testGetPane();
    }

    public void testSave() {
        super.testSave();
    }

    public void testFillInTextSession() {
        super.testFillInTextSession();
    }

    public void testGotoNumber() {
        super.testGotoNumber();
    }

    public void testGetTitle() {
        super.testGetTitle();
    }

    public void testExit() {
        super.testExit();
    }


    public void testGetButtonPanel() {
        super.testGetButtonPanel();
    }

    public void testSetMenuBar() {
        super.testSetMenuBar();
    }

    public void testSetButtonPermissions() {
        super.testSetButtonPermissions();
    }

    public void testGetDataList() {
        super.testGetDataList();
    }

    public void testGetSession() {
        super.testGetSession();
    }

    public void testGetHasFocus() {
        super.testGetHasFocus();
    }

    public void testSetHasFocus() {
        super.testSetHasFocus();
    }

    public void testSetDataList() {
        super.testSetDataList();
    }

    public void testSetIndexOfDisplayed() {
        super.testSetIndexOfDisplayed();
    }

    public void testSetSession() {
        super.testSetSession();
    }

    public void testLoadSessionClassString() {
        /*session.getTransaction().rollback();
        session.beginTransaction();*/
        session.clear();
        session.evict(DataKeeper.gatheringModel);

        List list = thisPanel.load(session, Gathering.class, "caption");
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Gathering);
        }
        session.evict(list);
        DataKeeper.gatheringModel = new GatheringModel(session);
        
        session.evict(DataKeeper.meetingModel);
        list = thisPanel.load(session, AbstractMeeting.class, "id");
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof AbstractMeeting);
        }
        
        list = thisPanel.load(session, ConcreteMeeting.class, "id");
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof ConcreteMeeting);
        }
        session.evict(list);
        DataKeeper.meetingModel = new MeetingModel(session);
        
        session.evict(DataKeeper.agentModel);
        list = thisPanel.load(session, Agent.class, "username");
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Agent);
        }
        session.evict(list);
        DataKeeper.agentModel = new AgentModel(session);
        
    }

    public void testLoadSessionClass() {
        session.flush();
        session.beginTransaction();
        
        List list = thisPanel.load(session, Gathering.class);
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Gathering);
        }
        
        list = thisPanel.load(session, AbstractMeeting.class);
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof AbstractMeeting);
        }
        
        list = thisPanel.load(session, ConcreteMeeting.class);
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof ConcreteMeeting);
        }
        
        list = thisPanel.load(session, Agent.class);
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Agent);
        }
    }

    public void testIsModified() {
        // don't do anything here, since subpanels should be the ones detecting modifications
        //TODO work on this, see if this is an error or not 
        
    }

    public void testSetIsModified() {
        super.testSetIsModified();
    }

    public void testSaveList() {
        super.testSaveList();
    }

}
