package airspaceGUI.base.test;

import java.awt.AWTException;
import java.awt.Component;
import java.awt.Container;
import java.awt.Robot;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.hibernate.Session;
import org.hibernate.cfg.Configuration;

import airspaceGUI.base.BasePanel;
import airspaceGUI.base.corpus.CorpusGUIMessage;
import airspaceGUI.base.corpus.gui.CorpusMainWindow;
import airspaceGUI.base.event.EventPanel;
import airspaceGUI.base.person.PeoplePanel;
import airspaceGUI.base.room.RoomPanel;
import airspaceGUI.model.AgentModel;
import airspaceGUI.model.GatheringModel;
import airspaceGUI.model.MeetingModel;
import airspaceGUI.model.PeopleModel;
import airspaceGUI.model.RoomModel;
import airspaceGUI.model.VariableModel;

import edu.cmu.radar.base.Person;

/**
 * This class should be used to test the interface of the swystem
 * using the ROBOT class to automate user actions.
 * @author srbennet
 *
 */
public class InterfaceTest extends TestCase
{

    private transient Robot robot;
    private static Session session;

    protected void setUp() {
        if (robot == null)
        {
            try
            {
                robot = new Robot();
            } catch (AWTException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        try
        {
            // Create the SessionFactory from hibernate.cfg.xml
            if (session == null)
            {
                final File configFile = new File("corpus.hibernate.xml");
                final Configuration config = new Configuration()
                        .configure(configFile);

                session = config.buildSessionFactory().openSession();
                session.beginTransaction();
            }
			
            if (!this.getClass().equals(DataKeeper.lastClass))
            {
                //System.out.println("The people model is null...");
                DataKeeper.peopleModel = new PeopleModel(session);
                DataKeeper.roomModel = new RoomModel(session);
                DataKeeper.variableModel = new VariableModel(session);
                DataKeeper.agentModel = new AgentModel(session);
                DataKeeper.gatheringModel = new GatheringModel(session);
                DataKeeper.meetingModel = new MeetingModel(session);
                
                DataKeeper.lastClass = this.getClass();
            }

        } catch (Exception ex)
        {
            // Make sure you log the exception, as it might be swallowed
            System.err.println("Initial SessionFactory creation failed." + ex);
            throw new ExceptionInInitializerError(ex);
        }
        
    }
    
    public static Test suite() {
        TestSuite suite= new TestSuite("CorpusGUI Tests");

        //add individual test classes
        suite.addTestSuite(InterfaceTest.class);
        
        return suite;
    }
    
    public void testPeoplePanel()
    {
        JFrame frame = new JFrame();
        PeoplePanel peoplePanel = new PeoplePanel(null, DataKeeper.peopleModel, session);
		setUpPanel(frame, peoplePanel);
        
        robot.setAutoWaitForIdle(true);
        robot.setAutoDelay(100);
        
        typeKey(KeyEvent.VK_F);
        typeKey(KeyEvent.VK_O);
        typeKey(KeyEvent.VK_O);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_B);
        typeKey(KeyEvent.VK_A);
        typeKey(KeyEvent.VK_R);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_F);
        typeKey(KeyEvent.VK_O);
        typeKey(KeyEvent.VK_O);
        robot.waitForIdle();
        robot.keyPress(KeyEvent.VK_SHIFT);
        typeKey(KeyEvent.VK_2);
        robot.keyRelease(KeyEvent.VK_SHIFT);
        typeKey(KeyEvent.VK_B);
        typeKey(KeyEvent.VK_A);
        typeKey(KeyEvent.VK_R);
        typeKey(KeyEvent.VK_PERIOD);
        typeKey(KeyEvent.VK_C);
        typeKey(KeyEvent.VK_O);
        typeKey(KeyEvent.VK_M);
        
        Person testPerson = (Person)peoplePanel.saveText();
        
        assertTrue(testPerson.getFirst().equals("foo"));
        assertTrue(testPerson.getLast().equals("bar"));
        assertTrue(testPerson.getEmail().equals("foo@bar.com"));
        
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        typeKey(KeyEvent.VK_TAB);
        
        peoplePanel.fillInText(session);
        testDeleteFromPanel(peoplePanel);
        //displayKey(KeyEvent.VK_ENTER);
        
        /*robot.keyPress(KeyEvent.VK_SHIFT);
        robot.keyPress(KeyEvent.VK_TAB);
        robot.keyRelease(KeyEvent.VK_TAB);
        robot.keyRelease(KeyEvent.VK_SHIFT);*/
        testSaveText(peoplePanel);
        testNextAndPrevButtons(peoplePanel);
        
        
        frame.dispose();
    }
    
    private void testSaveText(BasePanel basePanel) {
        JButton next = getButton(basePanel, "next");
        JButton prev = getButton(basePanel, "prev");
        Object onPanel = basePanel.saveText();
        
        int nextX = next.getLocationOnScreen().x + next.getWidth()/2;
        int nextY = next.getLocationOnScreen().y + next.getHeight()/2;
        int prevX = prev.getLocationOnScreen().x + prev.getWidth()/2;
        int prevY = prev.getLocationOnScreen().y + prev.getHeight()/2;
        
        robot.mouseMove(nextX, nextY);
        robot.mousePress(InputEvent.BUTTON1_MASK);
        robot.mouseRelease(InputEvent.BUTTON1_MASK);
        
        robot.mouseMove(prevX, prevY);

        robot.mousePress(InputEvent.BUTTON1_MASK);
        robot.mouseRelease(InputEvent.BUTTON1_MASK);
        
        assertTrue(onPanel.equals(basePanel.saveText()));
    }

    /**
     * @param basePanel
     * @param firstIndex
     */
    private void testNextAndPrevButtons(BasePanel basePanel) {
        int firstIndex = basePanel.getIndexOfDisplayed();
        robot.setAutoDelay(500);
        JButton next = getButton(basePanel, "next");
        JButton prev = getButton(basePanel, "prev");

        int nextX = next.getLocationOnScreen().x + next.getWidth()/2;
        int nextY = next.getLocationOnScreen().y + next.getHeight()/2;
        int prevX = prev.getLocationOnScreen().x + prev.getWidth()/2;
        int prevY = prev.getLocationOnScreen().y + prev.getHeight()/2;
        
        robot.mouseMove(nextX, nextY);
        robot.mousePress(InputEvent.BUTTON1_MASK);
        robot.mouseRelease(InputEvent.BUTTON1_MASK);
        
        assertTrue((firstIndex + 1) == basePanel.getIndexOfDisplayed());
        firstIndex = basePanel.getIndexOfDisplayed();
        
        robot.mousePress(InputEvent.BUTTON1_MASK);
        robot.mouseRelease(InputEvent.BUTTON1_MASK);
        assertTrue((firstIndex + 1) == basePanel.getIndexOfDisplayed());
        
        robot.mouseMove(prevX, prevY);
        firstIndex = basePanel.getIndexOfDisplayed();

        robot.mousePress(InputEvent.BUTTON1_MASK);
        robot.mouseRelease(InputEvent.BUTTON1_MASK);
        
        assertTrue((firstIndex - 1) == basePanel.getIndexOfDisplayed());
        firstIndex = basePanel.getIndexOfDisplayed();
        
        robot.mousePress(InputEvent.BUTTON1_MASK);
        robot.mouseRelease(InputEvent.BUTTON1_MASK);
        assertTrue((firstIndex - 1) == basePanel.getIndexOfDisplayed());
    }
    
    private JButton getButton(JPanel panel, String string){
        return getButton(panel.getRootPane(), string);
    }
    
    private JButton getButton(Container container, String string) {
        Component[] components = container.getComponents();
        ArrayList subContainers = new ArrayList();
        
        for(int i = 0; i < components.length; i++)
        {
            //System.out.println(components[i].getName());
            if (components[i] != null && components[i].getName() != null
                    && components[i].getName().equals(string) 
                    && components[i] instanceof JButton)
            {
                //System.out.println(components[i].getName());
                return (JButton)components[i];
            } else if (components[i] instanceof Container)
            {
                subContainers.add(components[i]);
            }
        }
        
        for(int i = 0; i < subContainers.size(); i++)
        {
            JButton result = getButton((Container)subContainers.get(i), string);
            if (result != null)
            {
                return result;
            }
        }
        
        return null;
    }

    private void testDeleteFromPanel(BasePanel basePanel) {
        Object displayed = basePanel.saveText();
        session.evict(displayed);
        
        basePanel.deleteDisplayedObject();
        
        //need to fudge the database so things actually get saved back to the database after they
        //'ve been deleted
        session.beginTransaction();
        session.save(displayed);
        session.getTransaction().commit();
        session.beginTransaction();
        session.update(displayed);
        session.getTransaction().commit();
        session.beginTransaction();
        
        Object differentPerson = basePanel.saveText();
        
        assertFalse(displayed.equals(differentPerson));
    }

    public void typeKey(int keyCode)
    {
        robot.keyPress(keyCode);
        robot.keyRelease(keyCode);
    }

    /**
     * @param frame
     * @param peoplePanel
     */
    private void setUpPanel(JFrame frame, BasePanel basePanel) {
        frame.getContentPane().add(basePanel.getPane());
        frame.pack();
		frame.setVisible(true);
        frame.validate();
    }
    
    public void testRoomPanel()
    {
        JFrame frame = new JFrame();
        RoomPanel roomPanel = new RoomPanel(null, DataKeeper.roomModel, session);
        setUpPanel(frame, roomPanel);
        roomPanel.fillInText(session);
        
        testDeleteFromPanel(roomPanel);
        testNextAndPrevButtons(roomPanel);
        testSaveText(roomPanel);
        
        frame.dispose();
    }
    
    /**
     * Tests the eventPanel without the cmradar fields
     *
     */
    public void testEventPanel()
    {
        JFrame frame = new JFrame();
        EventPanel eventPanel = new EventPanel(session, null, DataKeeper.gatheringModel, 
                DataKeeper.peopleModel);;
        testPanel(frame, eventPanel);
    }
    
    /**
     * Tests the eventPanel with the cmradar fields
     *
     */
    public void testEventPanelCmradar(){
        JFrame frame = new JFrame();
        EventPanel eventPanel = new EventPanel(session, null, DataKeeper.agentModel,
                DataKeeper.meetingModel, DataKeeper.gatheringModel, 
                DataKeeper.peopleModel, DataKeeper.roomModel);
        testPanel(frame, eventPanel);
    }

    /**
     * @param frame The frame upon which the panel will be shown
     * @param basePanel the panel that will be configured, shown, and tested
     */
    private void testPanel(JFrame frame, BasePanel basePanel) {
        setUpPanel(frame, basePanel);
        basePanel.fillInText(session);
        
        testDeleteFromPanel(basePanel);
        testNextAndPrevButtons(basePanel);
        testSaveText(basePanel);
        
        frame.dispose();
    }
    
    public void testCorpusPanel()
    {
        JFrame frame = new JFrame();
        CorpusMainWindow corpusPanel = new CorpusMainWindow(frame, null, session, DataKeeper.peopleModel);
        setUpPanel(frame, corpusPanel);
        corpusPanel.fillInText(session);
        
        
        //the delete code s copied here because os the wrapper class the 
        //corpusMainWindow uses for its data
        CorpusGUIMessage displayed = (CorpusGUIMessage)corpusPanel.saveText();
        session.evict(displayed);
        
        corpusPanel.deleteDisplayedObject();
        
        //need to fudge the database so things actually get saved back to the database after they
        //'ve been deleted
        session.beginTransaction();
        session.save(displayed.getMessage());
        session.getTransaction().commit();
        session.beginTransaction();
        session.update(displayed.getMessage());
        session.getTransaction().commit();
        session.beginTransaction();
        
        Object differentPerson = corpusPanel.saveText();
        
        assertFalse(displayed.equals(differentPerson));
        
        testNextAndPrevButtons(corpusPanel);
        testSaveText(corpusPanel);
        
        frame.dispose();
    }
    
}
