package airspaceGUI.base.test;

import java.io.File;
import java.util.ArrayList;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.hibernate.Session;
import org.hibernate.cfg.Configuration;

import airspaceGUI.model.AgentModel;
import airspaceGUI.model.GatheringModel;
import airspaceGUI.model.MeetingModel;
import airspaceGUI.model.PeopleModel;
import airspaceGUI.model.RoomModel;
import airspaceGUI.model.VariableModel;

import edu.cmu.radar.base.Gathering;
import edu.cmu.radar.base.Location;
import edu.cmu.radar.base.Person;
import edu.cmu.radar.base.cmradar.AbstractMeeting;
import edu.cmu.radar.base.cmradar.Agent;
import edu.cmu.radar.base.cmradar.ConcreteMeeting;
import edu.cmu.radar.base.corpus.Variable;
import edu.cmu.radar.base.stp.Room;

public class ModelsTest extends TestCase
{
    private static Session session;

    protected void setUp() {
        try
        {
            // Create the SessionFactory from hibernate.cfg.xml

            final File configFile = new File("corpus.hibernate.xml");
            final Configuration config = new Configuration()
                    .configure(configFile);

            session = config.buildSessionFactory().openSession();
        } catch (Exception ex)
        {
            // Make sure you log the exception, as it might be swallowed
            System.err.println("Initial SessionFactory creation failed." + ex);
            throw new ExceptionInInitializerError(ex);
        }
    }
    
    public static Test suite() {
        TestSuite suite= new TestSuite("CorpusGUI Tests");

        //add individual test classes
        suite.addTestSuite(ModelsTest.class);

        return suite;
    }
    
    public void testAgentModel(){
        AgentModel am = new AgentModel(session);
        int i;
        for(i = 0; i < am.size(); i++)
        {
            assertTrue(am.get(i) instanceof Agent);
        }
        
        Agent a = new Agent();
        
        testArrayListFunctions(am, a);   
        
    }
    
    public void testArrayListFunctions(ArrayList list, Object newObject)
    {
        list.add(newObject);
        assertTrue(list.get(list.size() - 1) == newObject);
        assertTrue(list.contains(newObject));
        
        list.remove(newObject);
        assertFalse(list.contains(newObject));
        
        ArrayList al = new ArrayList();
        al.add(newObject);
        list.addAll(al);
        assertTrue(list.contains(newObject));
        
        list.removeAll(al);
        assertFalse(list.contains(al));
        list.removeAll(list);
        assertTrue(list.isEmpty());
    }
    
    public void testGatheringModel(){
        GatheringModel gm = new GatheringModel(session);
        int i;
        for(i = 0; i < gm.size(); i++)
        {
            assertTrue(gm.get(i) instanceof Gathering);
        }
        
        Gathering g = new Gathering();
        
        testArrayListFunctions(gm, g);
    }
    
    public void testMeetingModel(){
        MeetingModel mm = new MeetingModel(session);
        int i;
        for(i = 0; i < mm.size(); i++)
        {
            assertTrue(mm.get(i) instanceof AbstractMeeting);
        }
        
        ConcreteMeeting cm = new ConcreteMeeting();
        
        testArrayListFunctions(mm, cm);
        
        AbstractMeeting am = new AbstractMeeting();
        testArrayListFunctions(mm, am);
    }
    
    public void testPeopleModel(){
        PeopleModel pm = new PeopleModel(session);
        int i;
        for(i = 0; i < pm.size(); i++)
        {
            assertTrue(pm.get(i) instanceof Person);
        }
        
        Person p = new Person();
        
        testArrayListFunctions(pm, p); 
    }
    
    public void testRoomModel(){
        RoomModel rm = new RoomModel(session);
        int i;
        for(i = 0; i < rm.size(); i++)
        {
            assertTrue(rm.get(i) instanceof Location);
        }
        
        Room r = new Room();
        
        testArrayListFunctions(rm, r); 
    }
    
    public void testVariableModel(){
        VariableModel vm = new VariableModel(session);
        int i;
        for(i = 0; i < vm.size(); i++)
        {
            assertTrue(vm.get(i) instanceof Variable);
        }
        
        Variable v = new Variable("Fake", "Variable");
        
        testArrayListFunctions(vm, v); 
    }
    
    
}
