package airspaceGUI.base.test;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;

import javax.swing.JFrame;

import org.hibernate.Session;

import airspaceGUI.base.room.RoomPanel;
import airspaceGUI.model.RoomModel;

import junit.framework.Test;
import junit.framework.TestSuite;
import edu.cmu.radar.base.Location;

public class RoomPanelTest extends BasePanelTest
{

    JFrame frame = new JFrame();
    
    protected void setUp() throws Exception {
        super.setUp();
        if (thisPanel == null)
        {
            //System.out.println("loading the roomPanel");
            if (!this.getClass().equals(DataKeeper.lastClass))
            {
                //System.out.println("getting a new room model");
                DataKeeper.roomModel = new RoomModel(session);
                DataKeeper.lastClass = this.getClass();
            }
            //System.out.println("making a new panel");
            thisPanel = new RoomPanel(null, DataKeeper.roomModel, session);
            frame.getContentPane().add(thisPanel.getPane());
            frame.pack();
            frame.setVisible(true);
        }
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        frame.setVisible(false);
        
        //only here for debugging... remove soon
        Method method = Class.forName("airspaceGUI.base.SubPanel")
            .getDeclaredMethod("getDataList", null);
        method.setAccessible(true);
        List l = (List)method.invoke(thisPanel, null);
        //System.out.println("the list is currently " + l);
    }
    
    public static Test suite() {
        TestSuite suite= new TestSuite("CorpusGUI Tests");

        //add individual test classes
        suite.addTestSuite(RoomPanelTest.class);
        
        return suite;
    }

    public void testAddModificationListeners() {
        super.testAddModificationListeners();
    }

    public void testAddUndoableEditListeners() {
        super.testAddUndoableEditListeners();
    }

    public void testSaveText() {
        super.testSaveText();
    }

    public void testClearText() {
        super.testClearText();
    }

    public void testFillInTextObject() {
        super.testFillInTextObject();
    }

    public void testSetStatus() {
        super.testSetStatus();
    }

    public void testGetFindWindow() {
        super.testGetFindWindow();
    }

    public void testSaveTextLocation() {
        thisPanel.fillInText(session);
        try{
            Class thisSuperClass = Class.forName("airspaceGUI.base.SubPanel");
            Class[] locationClass = {Location.class};
            
            Method getDataList;
            Method saveTextLocation = thisPanel.getClass().getDeclaredMethod("saveText", locationClass);
            getDataList = thisSuperClass.getDeclaredMethod("getDataList", null);
            getDataList.setAccessible(true);
            saveTextLocation.setAccessible(true);
            
            List l = (List)getDataList.invoke(thisPanel, null);
            
            Object[] parameter = {(Location)l.get(thisPanel.getIndexOfDisplayed())};
            Object result = saveTextLocation.invoke(thisPanel, parameter);
            
            assertFalse(((Boolean)result).booleanValue());
            
          }catch (SecurityException e)
          {
              e.printStackTrace();
              fail("Security exception");
          } catch (NoSuchMethodException e)
          {
              e.printStackTrace();
              fail("No such method exception");
          } catch (IllegalArgumentException e)
          {
              e.printStackTrace();
              fail("Illegal argument exception");
          } catch (IllegalAccessException e)
          {
              e.printStackTrace();
              fail("Ilegal access exception");
          } catch (InvocationTargetException e)
          {
              e.printStackTrace();
              fail("Invocation target exception");
          } catch (ClassNotFoundException e)
          {
              e.printStackTrace();
              fail("Class not found exception");
          } 
    }

    public void testSave() {
        super.testSave();
    }

    public void testFillInTextSession() {
        super.testFillInTextSession();
    }

    public void testFillInTextLocation() {
        try
        {
            Method method = Class.forName("airspaceGUI.base.SubPanel").getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);
            
            Location toBeDisplayed = (Location)l.get(thisPanel.getIndexOfDisplayed());
            
            thisPanel.fillInText(toBeDisplayed);
            assertTrue(thisPanel.saveText().equals(toBeDisplayed));
            
            
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } 
    }

    public void testGotoNumber() {
        super.testGotoNumber();
    }

    public void testGetTitle() {
        super.testGetTitle();
    }

    public void testDeleteDisplayedObject() {
        super.testDeleteDisplayedObject();    
    }

    public void testGetStatusText() {
        super.testGetStatusText();
    }

    public void testExit() {
        super.testExit();
    }

    public void testSetMenuBar() {
        super.testSetMenuBar();
    }

    public void testSetButtonPermissions() {
        super.testSetButtonPermissions();
    }

    public void testMaxIndex() {
        super.testMaxIndex();
    }

    public void testGetDataList() {
        super.testGetDataList();
    }

    public void testGetIndexOfDisplayed() {
        super.testGetIndexOfDisplayed();
    }

    public void testGetSession() {
        super.testGetSession();
    }

    public void testGetHasFocus() {
        super.testGetHasFocus();
    }

    public void testSetHasFocus() {
       super.testSetHasFocus();
    }

    public void testSetDataList() {
        super.testSetDataList();
    }

    public void testSetIndexOfDisplayed() {
        super.testSetIndexOfDisplayed();
    }

    public void testSetSession() {
        super.testSetSession();
    }

    public void testLoadSessionClassString() {
        Session newSession = session;
        List list = thisPanel.load(newSession, Location.class, "id");
        assertNotNull(list);
        Location lastLocation = (Location)list.get(0);
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Location);
            assertTrue(lastLocation.getId().intValue() < ((Location)list.get(i)).getId().intValue());
        }
        
        list = thisPanel.load(newSession, Location.class, "caption");
        assertNotNull(list);
        
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Location);
            // check that the first value is increasing
        }
        
        list = thisPanel.load(newSession, Location.class, "building");
        assertNotNull(list);
        
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Location);
            // check that the middle value is increasing
        }
        
        list = thisPanel.load(newSession, Location.class, "type");
        assertNotNull(list);
        
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Location);
            // check that the last value is increasing
        }
    }

    public void testLoadSessionClass() {
        Session newSession = session;
        List list = thisPanel.load(newSession, Location.class);
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Location);
        }
    }

    public void testIsModified() {
       // System.out.println("starting isModified");
        super.testIsModified();
        //System.out.println("ending isModified");
    }

    public void testSetIsModified() {
        super.testSetIsModified();
    }

    public void testSaveList() {
        super.testSaveList();
    }

}
