package airspaceGUI.base.test;

import java.awt.AWTException;
import java.awt.Component;
import java.awt.Container;
import java.awt.Robot;
import java.awt.TextComponent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.JTextComponent;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.hibernate.CacheMode;
import org.hibernate.FlushMode;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

import airspaceGUI.base.SubPanel;

public class SubPanelTest extends TestCase
{
    protected SubPanel thisPanel;

    protected Robot robot;

    protected static Session session;

    private static SessionFactory sessionFactory;

    public static Test suite() {
        TestSuite suite = new TestSuite("CorpusGUI Tests");

        // add individual test classes
        suite.addTestSuite(SubPanelTest.class);

        return suite;
    }

    protected void setUp() throws Exception {
        super.setUp();
        if (robot == null)
        {
            try
            {
                robot = new Robot();
            } catch (AWTException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        try
        {
            if (session == null)
            {
                // Create the SessionFactory from corpus.hibernate.xml

                final File configFile = new File("corpus.hibernate.xml");
                final Configuration config = new Configuration()
                        .configure(configFile);

                sessionFactory = config.buildSessionFactory();
                session = sessionFactory.openSession();
                session.beginTransaction();
                session.setFlushMode(FlushMode.AUTO);
                session.setCacheMode(CacheMode.REFRESH);
            }else if (!session.isOpen() || !session.isConnected())
            {
                session = sessionFactory.openSession();
                session.beginTransaction();
            }

        } catch (Exception ex)
        {
            // Make sure you log the exception, as it might be swallowed
            System.err.println("Initial SessionFactory creation failed." + ex);
            throw new ExceptionInInitializerError(ex);
        }

        robot.setAutoDelay(300);
        robot.setAutoWaitForIdle(true);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        // session.clear();
        // just in case the session gets reset/restarted...
        /*
         * Class[] classes = {Session.class}; Method method =
         * Class.forName("airspaceGUI.base.SubPanel")
         * .getDeclaredMethod("setSession", classes);
         * method.setAccessible(true); Object[] params = {session};
         * method.invoke(thisPanel, params);
         */
        //get rid of any changes that have been made this round
        synchronized(session){
            session.getTransaction().rollback();
            //session.close();
            session.beginTransaction();
        }
        
        // this is here to ensure that any unused objects generated during the tests are
        //promptly garbage collected.
        System.gc();
    }

    protected JButton getButton(JPanel panel, String string) {
        return getButton(panel.getRootPane(), string);
    }

    protected JButton getButton(Container container, String string) {
        Component[] components = container.getComponents();
        ArrayList subContainers = new ArrayList();

        for (int i = 0; i < components.length; i++)
        {
            // System.out.println(components[i].getName());
            if (components[i] != null && components[i].getName() != null
                    && components[i].getName().equals(string)
                    && components[i] instanceof JButton)
            {
                // System.out.println(components[i].getName());
                return (JButton)components[i];
            } else if (components[i] instanceof Container)
            {
                subContainers.add(components[i]);
            }
        }

        for (int i = 0; i < subContainers.size(); i++)
        {
            JButton result = getButton((Container)subContainers.get(i), string);
            if (result != null)
            {
                return result;
            }
        }

        return null;
    }

    protected void typeKey(int keyCode) {
        robot.keyPress(keyCode);
        robot.keyRelease(keyCode);
    }

    KeyListener keyListener = new KeyListener()
    {

        public void keyPressed(KeyEvent arg0) {
            // empty

        }

        public void keyReleased(KeyEvent arg0) {
            // empty
        }

        public void keyTyped(KeyEvent arg0) {
            // erase the character that was just typed to
            // show that control got here
            typeKey(KeyEvent.VK_BACK_SPACE);
        }

    };

    ItemListener itemListener = new ItemListener()
    {
        public void itemStateChanged(ItemEvent arg0) {
            ((AbstractButton)arg0.getItem()).setSelected(((AbstractButton)arg0
                    .getItem()).getSelectedObjects() != null);
        }

    };

    ChangeListener changeListener = new ChangeListener()
    {
        public void stateChanged(ChangeEvent arg0) {
            // figure out what goes here
        }

    };

    private UndoableEditListener undoableEditListener = new UndoableEditListener()
    {

        public void undoableEditHappened(UndoableEditEvent arg0) {
            // undo the edit, like it was never done.
            arg0.getEdit().undo();
        }

    };

    public void testAddModificationListeners() {
        Component[] components = thisPanel.getComponents();
        Class thisClass = thisPanel.getClass();

        Class[] parametersClasses = {KeyListener.class, ItemListener.class,
                ChangeListener.class};
        Object[] parameters = {keyListener, itemListener, changeListener};
        try
        {
            Method m = thisClass.getDeclaredMethod("addModificationListeners",
                    parametersClasses);
            m.setAccessible(true);
            m.invoke(thisPanel, parameters);

            for (int i = 0; i < components.length; i++)
            {
                // check and make sure each component has the correct listener
                // on it
                if (components[i] instanceof TextComponent)
                {
                    // type something and ensure that the keylistener fires
                    components[i].requestFocusInWindow();
                    typeKey(KeyEvent.VK_S);

                    assertTrue(((TextComponent)components[i]).getText().equals(
                            ""));

                    typeKey(KeyEvent.VK_BACK_SPACE);

                } else if (components[i] instanceof AbstractButton)
                {
                    // this is for buttons and checkboxes
                    Component component = components[i];
                    boolean selectedStartValue = ((AbstractButton)component)
                            .isSelected();

                    int xPos = component.getX() + component.getWidth() / 3;
                    int yPos = component.getY() + component.getHeight() / 2;
                    robot.mouseMove(xPos, yPos);
                    robot.mousePress(InputEvent.BUTTON1_DOWN_MASK);
                    robot.mouseRelease(InputEvent.BUTTON1_DOWN_MASK);

                    assertFalse(((AbstractButton)component).isSelected() == selectedStartValue);

                } else if (components[i] instanceof JComboBox)
                {
                    // this is i believe, for everything else
                    // this is i believe, for everything else
                    Component component = components[i];
                    Object selected = ((JComboBox)component).getSelectedItem();

                    int xPos = component.getX() + (3 * component.getWidth())
                            / 4;
                    int yPos = component.getY() + component.getHeight() / 2;
                    robot.mouseMove(xPos, yPos);
                    robot.mousePress(InputEvent.BUTTON1_DOWN_MASK);
                    robot.mouseRelease(InputEvent.BUTTON1_DOWN_MASK);

                    assertTrue(((JComboBox)component).getSelectedItem().equals(
                            selected));

                }
            }
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        }
    }

    public void testAddUndoableEditListeners() {
        try
        {
            Component[] components = thisPanel.getComponents();
            Class thisClass = thisPanel.getClass();
            Class[] parameterClasses = {UndoableEditListener.class};
            Method method = thisClass.getDeclaredMethod(
                    "addUndoableEditListeners", parameterClasses);
            Object[] parameters = {undoableEditListener};
            method.setAccessible(true);

            method.invoke(thisPanel, parameters);

            for (int i = 0; i < components.length; i++)
            {
                // check and make sure each component has the correct listener
                // on it
                if (components[i] instanceof TextComponent)
                {
                    // type something and ensure that the keylistener fires
                    components[i].requestFocusInWindow();
                    typeKey(KeyEvent.VK_S);

                    assertTrue(((TextComponent)components[i]).getText().equals(
                            ""));

                    typeKey(KeyEvent.VK_BACK_SPACE);

                } else if (components[i] instanceof AbstractButton)
                {
                    // this is for buttons and checkboxes
                    Component component = components[i];
                    boolean selectedStartValue = ((AbstractButton)component)
                            .isSelected();

                    int xPos = component.getX() + component.getWidth() / 3;
                    int yPos = component.getY() + component.getHeight() / 2;
                    robot.mouseMove(xPos, yPos);
                    robot.mousePress(InputEvent.BUTTON1_DOWN_MASK);
                    robot.mouseRelease(InputEvent.BUTTON1_DOWN_MASK);

                    assertFalse(((AbstractButton)component).isSelected() == selectedStartValue);

                } else if (components[i] instanceof JComboBox)
                {
                    // this is i believe, for everything else
                    Component component = components[i];
                    Object selected = ((JComboBox)component).getSelectedItem();

                    int xPos = component.getX() + (3 * component.getWidth())
                            / 4;
                    int yPos = component.getY() + component.getHeight() / 2;
                    robot.mouseMove(xPos, yPos);
                    robot.mousePress(InputEvent.BUTTON1_DOWN_MASK);
                    robot.mouseRelease(InputEvent.BUTTON1_DOWN_MASK);

                    assertTrue(((JComboBox)component).getSelectedItem().equals(
                            selected));
                }
            }
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        }
    }

    public void testMaxIndex() {
        Class thisClass;
        try
        {
            thisClass = Class.forName("airspaceGUI.base.SubPanel");
            Method getDataList;
            getDataList = thisClass.getDeclaredMethod("getDataList", null);
            getDataList.setAccessible(true);

            List dataList = (List)getDataList.invoke(thisPanel, null);
            int sizeOfData = dataList.size() - 1;
            /*
             * System.out.println("The size of the data list is " + sizeOfData);
             * System.out.println("The panel should be " + dataList);
             * System.out.println("The return of maxIndex is " +
             * thisPanel.maxIndex());
             */
            assertEquals(thisPanel.maxIndex(), sizeOfData);

        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }

    }

    public void testDeleteDisplayedObject() {
        thisPanel.fillInText(session);
        Object onPanel = thisPanel.saveText();

        //evict the object so the change won't be propogated to the database
        session.evict(onPanel);
        //System.out.println("On the panel is " + onPanel);
        
        thisPanel.deleteDisplayedObject();

        Object differentObject = thisPanel.saveText();

        assertFalse(onPanel.equals(differentObject));

        //a hack to make the panel reload the data (hopefully)
        DataKeeper.lastClass = null;
        
        
    }

    public void testGetDataList() {
        // TODO add more test cases
        try
        {
            Class thisClass = Class.forName("airspaceGUI.base.SubPanel");
            Method method;
            method = thisClass.getDeclaredMethod("getDataList", null);
            method.setAccessible(true);

            List dataList = (List)method.invoke(thisPanel, null);
            assertNotNull(dataList);
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testGetStatusText() {
        try
        {
            // not sure what to do here
            // the panel should be blank now
            String startingText = thisPanel.getStatusText();
            thisPanel.fillInText(session);

            // System.out.println("The original text is " + startingText);
            // System.out.println("The new text is " +
            // thisPanel.getStatusText());

            // this might not be the case if something loads the panels
            // automatically.
            // assertFalse(thisPanel.getStatusText().equals(startingText));

            Class thisClass = Class.forName("airspaceGUI.base.SubPanel");
            Method method;
            method = thisClass.getDeclaredMethod("getDataList", null);
            method.setAccessible(true);

            List dataList = (List)method.invoke(thisPanel, null);
            startingText = thisPanel.getStatusText();

            if (dataList.size() > 1)
            {
                thisPanel.gotoNumber(1);
                
                //use these for debugging purposes
                //System.out.println("The starting text is " + startingText);
                //System.out.println("The ending text is " + thisPanel.getStatusText());
                
                assertFalse(thisPanel.getStatusText().equals(startingText));
            }
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testGetIndexOfDisplayed() {
        thisPanel.fillInText(session);
        thisPanel.gotoNumber(0);
        int firstIndex = thisPanel.getIndexOfDisplayed();
        assertTrue(firstIndex == 0);

        Object onPanel = thisPanel.saveText();

        int secondIndex = thisPanel.getIndexOfDisplayed();

        assertEquals(firstIndex, secondIndex);

        try
        {
            Method method = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);
            int number = new Random().nextInt(l.size() / 2);
            // System.out.println("got random number");
            thisPanel.gotoNumber(number);
            Thread.yield();
            //System.out.println("Starting the second save");
            int saved = thisPanel.getIndexOfDisplayed();
            assertTrue(number == saved);
            //System.out.println("saved");
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }

    }

    public void testGetSession() {
        try
        {
            Method method = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getSession", null);
            // ensure that some sort of session is saved in the panel
            method.setAccessible(true);
            Object result = method.invoke(thisPanel, null);
            assertTrue(result != null);
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testGetHasFocus() {
        final Component[] components = thisPanel.getComponents();
        thisPanel.setVisible(true);
        try
        {

            final Method m = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getHasFocus", null);
            m.setAccessible(true);

            for (int i = 0; i < components.length; i++)
            {
                if (components[i] instanceof JTextComponent)
                {
//                    while (!components[i].hasFocus())
//                    {
                        // yield so that there's enough time for the component
                        // to be granted
                        // focus
                        components[i].requestFocus();
                        components[i].requestFocusInWindow();
                        // Thread.yield();
//                    }

                    final Component c = components[i];

                    SwingUtilities.invokeAndWait(new Runnable()
                    {

                        public void run() {

                            Object hasFocus = null;
                            try
                            {
                                hasFocus = m.invoke(thisPanel, null);
                            } catch (IllegalArgumentException e)
                            {
                                fail("illegal argument exception");
                            } catch (IllegalAccessException e)
                            {
                                fail("illegal access exception");
                            } catch (InvocationTargetException e)
                            {
                                fail("invocation target exception");
                            }

                            FocusListener[] f = c.getFocusListeners();

                            assertNotNull(hasFocus);

                            assertEquals(c, hasFocus);
                        }
                    });

                }
            }
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            System.out.println(e.getCause());
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } catch (InterruptedException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public void testSetHasFocus() {
        Component[] components = thisPanel.getComponents();

        try
        {
            Class subPanel = Class.forName("airspaceGUI.base.SubPanel");
            Method getFocus = subPanel.getDeclaredMethod("getHasFocus", null);
            Class[] setFocusParameterClass = {JTextComponent.class};
            Method setFocus = subPanel.getDeclaredMethod("setHasFocus",
                    setFocusParameterClass);
            getFocus.setAccessible(true);
            setFocus.setAccessible(true);

            for (int i = 1; i < components.length; i++)
            {
                if (components[i] instanceof JTextComponent)
                {
                    Object[] parameters = {components[i]};
                    // System.out.println("before setting focus");
                    setFocus.invoke(thisPanel, parameters);
                    // System.out.println(components[i]);
                    assertEquals(components[i], getFocus
                            .invoke(thisPanel, null));
                }
            }
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testSetDataList() {
        try
        {
            ArrayList list = new ArrayList();
            Class thisClass = Class.forName("airspaceGUI.base.SubPanel");
            Class[] setParametersClasses = {List.class};
            Object[] setParameters = {list};
            Method getDataList = thisClass.getDeclaredMethod("getDataList",
                    null);
            Method setDataList = thisClass.getDeclaredMethod("setDataList",
                    setParametersClasses);

            getDataList.setAccessible(true);
            setDataList.setAccessible(true);

            List prevList = (List)getDataList.invoke(thisPanel, null);
            setDataList.invoke(thisPanel, setParameters);
            List newList = (List)getDataList.invoke(thisPanel, null);

            assertEquals(newList, list);
            assertFalse(prevList.equals(newList));

            // restore the previous datalist
            Object[] setNewParameters = {prevList};
            setDataList.invoke(thisPanel, setNewParameters);
            newList = (List)getDataList.invoke(thisPanel, null);
            assertEquals(prevList, newList);
            assertFalse(list.equals(newList));

        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }

    }

    public void testSetIndexOfDisplayed() {
        try
        {
            Class thisClass = Class.forName("airspaceGUI.base.SubPanel");
            Class[] parameterClasses = {Integer.TYPE};
            Method setIndex = thisClass.getDeclaredMethod(
                    "setIndexOfDisplayed", parameterClasses);
            Method getIndex = thisClass.getDeclaredMethod(
                    "getIndexOfDisplayed", null);
            Method maxIndex = thisClass.getDeclaredMethod("maxIndex", null);

            setIndex.setAccessible(true);
            getIndex.setAccessible(true);
            maxIndex.setAccessible(true);

            // List dataList = (List)maxIndex.invoke(thisPanel, null);
            int result = ((Integer)maxIndex.invoke(thisPanel, null)).intValue();
            Object[] parameters = {new Integer(result)};

            for(int i = 0; i <= result; i++){
                parameters[0] = new Integer(i);
                setIndex.invoke(thisPanel, parameters);
                assertEquals(
                        ((Integer)getIndex.invoke(thisPanel, null)).intValue(),
                        i);
            }
            
            parameters[0] = new Integer(0);
            setIndex.invoke(thisPanel, parameters);
            assertTrue(getIndex
                    .invoke(thisPanel, null).equals(parameters[0]));
            
            parameters[0] = new Integer(result);
            setIndex.invoke(thisPanel, parameters);
            assertTrue(getIndex
                    .invoke(thisPanel, null).equals(parameters[0]));
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testSetSession() {
        try
        {
            Class[] parameterClasses = {Session.class};
            Method setSession = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("setSession", parameterClasses);
            Method getSession = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getSession", null);
            Object[] parameters = {session};
            // ensure that some sort of session is saved in the panel
            getSession.setAccessible(true);
            setSession.setAccessible(true);

            setSession.invoke(thisPanel, parameters);

            Object result = getSession.invoke(thisPanel, null);
            assertTrue(result.equals(session));
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testIsModified() {
        try
        {
            Method isModified = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("isModified", null);

            // in this class we can only check if it is modified to begin with
            // need to hop to the basePanel to get actual modification listening
            assertFalse("The panel is modified before we do anything",
                    ((Boolean)isModified.invoke(thisPanel, null))
                            .booleanValue());

            thisPanel.fillInText(session);
            isModified.setAccessible(true);

            // in this class we can only check if it is modified to begin with
            // need to hop to the basePanel to get actual modification listening
            assertFalse("The panel is modified before we do anything",
                    ((Boolean)isModified.invoke(thisPanel, null))
                            .booleanValue());
            // the above is not necessarially true, since loading something onto
            // the panel is modifying it.

        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testSetIsModified() {
        try
        {
            Class[] parameterClasses = {Boolean.TYPE};
            Method setIsModified = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("setIsModified", parameterClasses);
            Method isModified = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("isModified", null);
            Object[] trueParameters = {new Boolean(true)};
            // ensure that some sort of session is saved in the panel
            isModified.setAccessible(true);
            setIsModified.setAccessible(true);

            setIsModified.invoke(thisPanel, trueParameters);

            Object result = isModified.invoke(thisPanel, null);
            assertTrue(((Boolean)result).booleanValue());

            Object[] falseParameters = {new Boolean(false)};
            setIsModified.invoke(thisPanel, falseParameters);
            result = isModified.invoke(thisPanel, null);

            assertFalse(((Boolean)result).booleanValue());

        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testSaveList() {
        Method method;
        try
        {
            thisPanel.fillInText(session);
            typeKey(KeyEvent.VK_1);
            thisPanel.saveText();

            method = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);
            
            List origCopy = new ArrayList();
            
            for(int i = 0; i < l.size(); i++)
            {
                origCopy.add(l.get(i));
            }
            

            thisPanel.save(l);

            thisPanel.load(session);

            List newList = (List)method.invoke(thisPanel, null);

            //for debugging purposes
             /*System.out.println("orig box " + l); 
             System.out.println("new list " + newList);*/

            assertTrue(l.equals(newList));
            
            typeKey(KeyEvent.VK_BACK_SPACE);

        } catch (SecurityException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (NoSuchMethodException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ClassNotFoundException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IllegalArgumentException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IllegalAccessException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InvocationTargetException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public void testSave() {
        Method method;
        try
        {
            thisPanel.fillInText(session);
            typeKey(KeyEvent.VK_1);
            thisPanel.saveText();

            method = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);

            thisPanel.save();

            thisPanel.load(session);

            List newList = (List)method.invoke(thisPanel, null);

            assertTrue(l.equals(newList));
            
            if (session.getTransaction().wasCommitted() || session.getTransaction().wasRolledBack())
            {
                //System.out.println("restarting transaction");
                session.beginTransaction();
            }

        } catch (SecurityException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (NoSuchMethodException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ClassNotFoundException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IllegalArgumentException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IllegalAccessException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InvocationTargetException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    public void testExit() {
        // make sure it can exit when changes aren't being made
        // fail("this is commented out because of concurency problems");

        assertTrue(thisPanel.exit());
        // session = sessionFactory.openSession();
        // make sure the session and transaction are still open
        session.beginTransaction();
        // TODO need to test other conditions.
    }

    public void testSaveText() {
        try
        {
            thisPanel.fillInText(session);
            Object saved = thisPanel.saveText();

            Method method = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);
            Object original = null;

            if (l.size() > thisPanel.getIndexOfDisplayed())
            {
                original = l.get(thisPanel.getIndexOfDisplayed());
            } else
            {
                fail("The indexOfDisplayed is larger than possible");
            }
            /*
             * System.out .println("The original message that the save should
             * match is " + original);
             */
            boolean result = saved.equals(original);
            assertTrue(result);
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testClearText() {
        thisPanel.fillInText(session);
        Object putBack = thisPanel.saveText();
        // fill in the text on the panel so we have something to clear off
        thisPanel.createNewInstance();
        Object result = thisPanel.saveText();

        if (result != null)
        {
            // here goes things we need to clean up even if the test fails
            // System.out.println("the test should be clear " + result);
            // System.out.println(result.getClass());
            // if it isn't null, then something that shouldn't be in the
            // datalist
            // may have gotten saved
            thisPanel.deleteDisplayedObject();

        }
        assertNull(result);
        thisPanel.deleteDisplayedObject();
        thisPanel.fillInText(putBack);

    }

    public void testFillInTextObject() {
        try
        {
            Method method = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);

            Object toBeDisplayed = l.get(thisPanel.getIndexOfDisplayed());

            thisPanel.fillInText(toBeDisplayed);
            assertTrue(thisPanel.saveText().equals(toBeDisplayed));

        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testGotoNumber() {
        try
        {
            thisPanel.fillInText(session);
            Method method = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);
            int number = new Random().nextInt(l.size());
            Object toBeDisplayed = l.get(number);

            thisPanel.gotoNumber(number);
            Thread.yield();
            // System.out.println("Starting the second save");
            Object saved = thisPanel.saveText();
            assertTrue(saved.equals(toBeDisplayed));

        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testGetPane() {
        JPanel panel = thisPanel.getPane();
        assertTrue(panel != null);
        // don't really care what's on it, as long as it's not null.
        // TODO not sure how to automatically test if the getPane works or
        // not...
    }

    public void testFillInTextSession() {
        try
        {
            Method method = Class.forName("airspaceGUI.base.SubPanel")
                    .getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);

            Object toBeDisplayed = null;

            if (l.size() >= thisPanel.getIndexOfDisplayed())
            {
                toBeDisplayed = l.get(thisPanel.getIndexOfDisplayed());
            } else
            {
                fail("The indexOfDisplayed shouldn't be possible");
            }

            thisPanel.fillInText(session);
            Object shown = thisPanel.saveText();
            assertTrue(shown.equals(toBeDisplayed));

        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    public void testGetTitle() {
        /** TODO figure out how this is supposed to work */
    }

}
