package airspaceGUI.expansion;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JSpinner;
import javax.swing.SpinnerListModel;
import javax.swing.SpinnerNumberModel;

/**
 * A subclass of JSpinner which allows null to be displayed in addition to integers
 * @author srbennet
 *
 */
public class NumberSpinnerWithNull extends JSpinner
{
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private SpinnerNumberModel model;
    
    /**
     * The constant that represents null
     */
    public static final String UNAVAILABLE = "NULL";

    /**
     * Initializes the number spinner
     */
    public NumberSpinnerWithNull()
    {
        super(new SpinnerListModel());
        model = new SpinnerNumberModel();
        //pretend the spinner has a list model so that we can include null.
        ArrayList list = new ArrayList();
        list.add(UNAVAILABLE);
        ((SpinnerListModel)getModel()).setList(list);
    }
    
    public NumberSpinnerWithNull(SpinnerNumberModel model)
    {
        super(new SpinnerListModel());
        this.model = model;
        ArrayList list = new ArrayList();
        list.add(UNAVAILABLE);
        ((SpinnerListModel)getModel()).setList(list);
    }

    /**
     * Returns the current value displayed in the spinner
     * @return the current value of the spinner
     */
    public Object getValue() {
        if (model == null || model.getValue().equals(new Integer(-1)))
        {
            return UNAVAILABLE;
        } else
        {
            return model.getValue();
        }
    }

    /**
     * sets the value the spinner is showing.
     * @param arg0 the value to display to the user in the field.
     */
    public void setValue(Object arg0) {
        //warning, this is a bit messy to pull off our little null scam.
        if (arg0 != null && !(arg0 instanceof Integer))
        {
            if (!arg0.equals("Unavailable"))
            {
                throw new IllegalArgumentException();
            } else
            {
                model.setValue(new Integer(-1));
            }
        } else if (arg0 == null)
        {
            model.setValue(new Integer(-1));
            /*if (!((SpinnerListModel)getModel()).getList().contains(new Integer(-1)))
            {
                ((SpinnerListModel)getModel()).getList().add(new Integer(-1));
            }*/
            getModel().setValue(UNAVAILABLE);
        } else
        {
            model.setValue(arg0);
            if (!((SpinnerListModel)getModel()).getList().contains(arg0))
            {
            	List list = ((SpinnerListModel)getModel()).getList();
                list.add(arg0);
            }
            getModel().setValue(arg0);
        }
    }

    
    /**
     * Get the next value the spinner has shown
     * @return the next value the spinner showed
     * 
     */
    public Object getNextValue() {
        return model.getNextValue();
    }

    /**
     * Get the previous value the spinner has shown
     * @return the previous value the spinner showed
     * 
     */
    public Object getPreviousValue() {
        if (getModel().getValue().equals(new Integer(-1)))
        {
            return UNAVAILABLE;
        } else
        {
            return model.getPreviousValue();
        }
    }

}
