/**
 * 
 */
package airspaceGUI.expansion;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

/**
 * This is the inner panel for a find window that controls how many terms are
 * currently be searched for and how many are possible to search on in total for
 * this panel
 * 
 * @author srbennet
 * 
 */
class ParameterPanel extends JPanel
{
    /**
     * 
     */
    ArrayList searchFields = new ArrayList();

    ArrayList searchKeywordsComboBoxes = new ArrayList();

    // the list that is shown in the comboBox
    List searchKeywordChoices;

    /**
     * sets what the combo box is showing after the user selects a new item
     */
    ItemListener comboBoxListener = new ItemListener()
    {

        Object deselectedItem;

        public void itemStateChanged(ItemEvent arg0) {
            JComboBox source = (JComboBox)arg0.getSource();
            if (arg0.getStateChange() == ItemEvent.DESELECTED)
            {
                deselectedItem = arg0.getItem();
            } else
            {
                for (int i = 0; i < searchKeywordsComboBoxes.size(); i++)
                {
                    if (!searchKeywordsComboBoxes.get(i).equals(source)
                            && ((JComboBox)searchKeywordsComboBoxes.get(i))
                                    .getSelectedIndex() == source
                                    .getSelectedIndex())
                    {
                        ((JComboBox)searchKeywordsComboBoxes.get(i))
                                .setSelectedItem(deselectedItem);
                    }
                }
            }
        }

    };

    private final ArrayList addButtons = new ArrayList();

    private final ArrayList removeButtons = new ArrayList();

    /**
     * adds a new parameter to the panel to specify more parameters to search
     * over
     */
    ActionListener addListener = new ActionListener()
    {

        public void actionPerformed(ActionEvent arg0) {
            int y = addButtons.indexOf(arg0.getSource()) + 1;
            ((JTextField)searchFields.get(y)).setVisible(true);
            ((JComboBox)searchKeywordsComboBoxes.get(y)).setVisible(true);
            ((JButton)addButtons.get(y)).setVisible(true);
            ((JButton)removeButtons.get(y)).setVisible(true);
            /*System.out.println(((JButton)arg0.getSource())
                    .getTopLevelAncestor());*/
            ((JFrame)((JButton)arg0.getSource()).getTopLevelAncestor()).pack();
            ((JButton)arg0.getSource()).getTopLevelAncestor().validate();
            ((JButton)arg0.getSource()).getTopLevelAncestor().repaint();
        }

    };

    /**
     * removes a parameter from view, and removes whatever constraints that are
     * contained therein from the search
     */
    ActionListener removeListener = new ActionListener()
    {

        public void actionPerformed(ActionEvent arg0) {
            //System.out.println("removing");
            int y = removeButtons.indexOf(arg0.getSource());
            ((JTextField)searchFields.get(y)).setVisible(false);
            ((JComboBox)searchKeywordsComboBoxes.get(y)).setVisible(false);
            ((JButton)addButtons.get(y)).setVisible(false);
            ((JButton)removeButtons.get(y)).setVisible(false);
            ((JFrame)((JButton)arg0.getSource()).getTopLevelAncestor()).pack();
            ((JButton)arg0.getSource()).getTopLevelAncestor().validate();
            ((JButton)arg0.getSource()).getTopLevelAncestor().repaint();
        }

    };

    protected ParameterPanel(boolean canBeRemoved, List searchByChoices)
    {
        searchKeywordChoices = searchByChoices;
        Dimension buttonSize = new Dimension(20, 20);
        // add all the fields and buttons to the panel, but hide all but the top
        // one
        for (int i = 0; i < searchByChoices.size(); i++)
        {
            searchFields.add(new JTextField());
            searchKeywordsComboBoxes.add(new JComboBox(searchByChoices
                    .toArray()));
            addButtons.add(new JButton("+"));
            removeButtons.add(new JButton("-"));
            ((JTextField)searchFields.get(i))
                    .setPreferredSize(new Dimension(
                            ((JComboBox)searchKeywordsComboBoxes.get(i))
                                    .getPreferredSize().width,
                            ((JTextField)searchFields.get(i))
                                    .getPreferredSize().height));
            ((JComboBox)searchKeywordsComboBoxes.get(i))
                    .addItemListener(comboBoxListener);
            if (((JComboBox)searchKeywordsComboBoxes.get(i)).getModel()
                    .getSize() > i)
            {
                ((JComboBox)searchKeywordsComboBoxes.get(i))
                        .setSelectedIndex(i);
            }
            ((JButton)addButtons.get(i))
                    .setHorizontalTextPosition(JButton.CENTER);
            ((JButton)addButtons.get(i))
                    .setVerticalTextPosition(JButton.BOTTOM);
            ((JButton)addButtons.get(i)).setMargin(new Insets(5, 5, 5, 5));
            ((JButton)addButtons.get(i)).addActionListener(addListener);
            ((JButton)addButtons.get(i)).setPreferredSize(buttonSize);
            ((JButton)addButtons.get(i)).setMinimumSize(buttonSize);
            ((JButton)addButtons.get(i)).setMaximumSize(buttonSize);
            ((JButton)removeButtons.get(i)).addActionListener(removeListener);
            ((JButton)removeButtons.get(i))
                    .setHorizontalTextPosition(JButton.CENTER);
            ((JButton)removeButtons.get(i))
                    .setVerticalTextPosition(JButton.BOTTOM);
            ((JButton)removeButtons.get(i)).setMargin(new Insets(5, 5, 5, 5));
            ((JButton)removeButtons.get(i)).setPreferredSize(buttonSize);
            ((JButton)removeButtons.get(i)).setMinimumSize(buttonSize);
            ((JButton)removeButtons.get(i)).setMaximumSize(buttonSize);
        }
        addNewRow(canBeRemoved, 0);

        /*
         * for (int i = 1; i < searchByChoices.size(); i++) { addNewRow(true,
         * i); }
         */

        // repaint both this panel and the frame that is above it
        this.validate();
        this.repaint();
        this.validate();
    }

    /**
     * Adds a row to the panel
     * 
     * @param canBeRemoved
     *            true if it can be removed from the panel, false otherwise
     * @param rowNum
     *            the row the new row should be added to
     */
    private void addNewRow(boolean canBeRemoved, int rowNum) {
        Dimension buttonSize = new Dimension(20, 20);
        GridBagConstraints gbc = new GridBagConstraints();
        int x = 0;
        this.setLayout(new GridBagLayout());
        boolean hide = false;

        for (rowNum = rowNum; rowNum < searchFields.size(); rowNum++)
        {
            x = 0;
            gbc.insets = new Insets(4, 0, 4, 0);
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.fill = GridBagConstraints.HORIZONTAL;
            gbc.gridwidth = 1;
            gbc.gridx = x++;
            gbc.gridy = rowNum;
            gbc.weightx = 1.0;
            this.add((JTextField)searchFields.get(rowNum), gbc);

            gbc.gridx = x++;
            gbc.insets = new Insets(2, 0, 4, 0);
            gbc.gridy = rowNum;
            gbc.weightx = 0;
            gbc.gridwidth = 1;
            this.add((JComboBox)searchKeywordsComboBoxes.get(rowNum), gbc);

            gbc.insets = new Insets(4, 0, 4, 0);
            if (rowNum != (searchKeywordChoices.size() - 1))
            {
                // only add the add button if there are more choices in the
                // listbox
                gbc.gridx = x++;
                gbc.gridy = rowNum;
                gbc.fill = GridBagConstraints.NONE;
                this.add((JButton)addButtons.get(rowNum), gbc);
            }
            if (canBeRemoved)
            {
                gbc.gridx = x;
                gbc.gridy = rowNum;
                this.add((JButton)removeButtons.get(rowNum), gbc);
            }
            ((JTextField)searchFields.get(rowNum)).setVisible(!hide);
            ((JComboBox)searchKeywordsComboBoxes.get(rowNum)).setVisible(!hide);
            ((JButton)addButtons.get(rowNum)).setVisible(!hide);
            ((JButton)removeButtons.get(rowNum)).setVisible(!hide);
            ((JButton)addButtons.get(rowNum)).setPreferredSize(buttonSize);
            ((JButton)removeButtons.get(rowNum)).setPreferredSize(buttonSize);
            hide = canBeRemoved = true;
        }
    }

    /**
     * Retrieves the search terms from the fields on this panel
     * 
     * @return a map containing the search parameters. The key is the type of
     *         object that is being searched
     */
    public Map getSearchTerms() {
        HashMap searchTermSet = new HashMap();

        for (int i = 0; i < searchKeywordChoices.size(); i++)
        {
            if (((JTextField)searchFields.get(i)).isVisible())
            {
                searchTermSet.put(((JComboBox)searchKeywordsComboBoxes.get(i))
                        .getSelectedItem(), ((JTextField)searchFields.get(i))
                        .getText());
            }
        }

        return searchTermSet;
    }

    /**
     * highlights the text in the first field. This is usually used when there
     * were no results to the search, and the interface wants to be helpful.
     * 
     */
    public void highlightFirst() {
        /*
         * System.out.println("about to try and highlight");
         * System.out.println(searchFields.get(0));
         * System.out.println(((JTextField)searchFields .get(0)).getText());
         * System.out.println(((JTextField)searchFields
         * .get(0)).getText().length());
         */
        SwingUtilities.invokeLater(new Thread()
        {
            public void run() {
                ((JTextField)searchFields.get(0)).select(0,
                        ((JTextField)searchFields.get(0)).getText().length());
                return;
            }
        });
    }

}