/**
 * 
 */
package airspaceGUI.expansion;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

/**
 * This defines common items in the menubar to all panels.  This simplifies menubar managing.
 * 
 * @author srbennet
 * 
 */
public final class TabbedPanelMenu extends JMenuBar
{
    private static final long serialVersionUID = 1L;

    /**
     * The file menu
     */
    public final transient JMenu file = new JMenu("File");

    /**
     * The editing menu
     */
    public final transient JMenu edit = new JMenu("Edit");
    
    /**
     * The utilities menu.
     */
    public final transient JMenu util = new JMenu("Utilities");

    /**
     * Creates a new window
     */
    //TODO figure out how to lock the ActionListener from airspace to this item
    //private final transient JMenuItem newWindow = new JMenuItem("New Window");
    
    /**
     * Allows saving of data
     */
    private final transient JMenuItem save = new JMenuItem("Save");

    /**
     * Quits the program
     */
    private transient final JMenuItem quit = new JMenuItem("Quit");

    /**
     * Changes the last edit to the previous value
     */
    private final transient JMenuItem undo = new JMenuItem("Undo");

    /**
     * Changes the edit back
     */
    private final transient JMenuItem redo = new JMenuItem("Redo");

    /**
     * Removes the selected object and places it on the system clipboard
     */
    private final transient JMenuItem cut = new JMenuItem("Cut");

    /**
     * Places the selected object on the system clipboard
     */
    private final transient JMenuItem copy = new JMenuItem("Copy");

    /**
     * Puts the text from the clipboard into the selected area, or where the
     * cursor currently is
     */
    private final transient JMenuItem paste = new JMenuItem("Paste");

    private final transient JMenuItem next = new JMenuItem("Next");
    
    private final transient JMenuItem prev = new JMenuItem("Prev");
    
    private final transient JMenuItem newMenu = new JMenuItem("New");
    
    private final transient JMenuItem delete = new JMenuItem("Delete");
    
    private final transient JMenuItem find = new JMenuItem("Find...");
    
    /**
     * The mask used to determine the correct shortcuts
     */
    public transient int keyMask = ActionEvent.CTRL_MASK;

    /**
     * Constructs a TabbedPanelMenu with all its items greyed out
     */
    public TabbedPanelMenu()
    {
        super();

        if (System.getProperty("mrj.version") != null){ // set up the right
                                                        // sequence according to
                                                        // what type of computer
                                                        // the program is being
                                                        // run on
            keyMask = ActionEvent.META_MASK;
        }
        // assign all the menuItems to a shortcut set if it is enabled, and add
        // them to the menu bar
        
        // set up hotkey shortcut for this menu item
        save.setAccelerator(KeyStroke.getKeyStroke((KeyEvent.VK_S), keyMask));
        // set up hotkey shortcut for this menu item
        undo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, keyMask));
        // set up hotkey shortcut for this menu item
        redo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, keyMask
                | ActionEvent.SHIFT_MASK));
        // set up hotkey shortcut for this menu item
        cut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, keyMask));
        // set up hotkey shortcut for this menu item
        copy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, keyMask));
        // set up hotkey shortcut for this menu item
        paste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, keyMask));
        
//      set up hotkey shortcut for this menu item
        newMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, keyMask));
//      set up hotkey shortcut for this menu item
        delete.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE, keyMask));
        // set up hotkey shortcut for this menu item
        next.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, keyMask + KeyEvent.SHIFT_DOWN_MASK));
        // set up hotkey shortcut for this menu item
        prev.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, keyMask));
        // set up hotkey shortcut for this menu item
        find.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, keyMask));

        // set up hotkey shortcut for this menu item
        quit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, keyMask));

        reset();
    }

    /**
     * Resets the menuBar to its default configuration. It is used to
     * re-initialize the menuBar without going to the trouble of instanciating a
     * new object
     * 
     */
    public void reset() {
        this.removeAll();
        file.removeAll();
        edit.removeAll();
        util.removeAll();
        util.setText("Utilities");
        

        file.add(save);

        if (System.getProperty("mrj.version") == null)
        {
            file.addSeparator();
            file.add(quit);
        }

        edit.add(undo);
        edit.add(redo);
        edit.addSeparator();
        edit.add(cut);
        edit.add(copy);
        edit.add(paste);
        
        util.add(newMenu);
        util.add(delete);
        util.addSeparator();
        util.add(next);
        util.add(prev);
        util.addSeparator();
        util.add(find);
        
        //remove any actionListeners that may have been added to these menuItems

        removeAllActionListeners(save);
        removeAllActionListeners(undo);
        removeAllActionListeners(redo);
        removeAllActionListeners(cut);
        removeAllActionListeners(copy);
        removeAllActionListeners(paste);
        removeAllActionListeners(find);
        removeAllActionListeners(next);
        removeAllActionListeners(prev);
        removeAllActionListeners(delete);
        removeAllActionListeners(newMenu);
        
        save.setEnabled(false);
        undo.setEnabled(false);
        redo.setEnabled(false);
        cut.setEnabled(false);
        copy.setEnabled(false);
        paste.setEnabled(false);
        find.setEnabled(false);
        next.setEnabled(false);
        prev.setEnabled(false);
        delete.setEnabled(false);
        newMenu.setEnabled(false);
        
        save.setVisible(true);
        undo.setVisible(true);
        redo.setVisible(true);
        cut.setVisible(true);
        copy.setVisible(true);
        paste.setVisible(true);
        find.setVisible(true);
        next.setVisible(true);
        prev.setVisible(true);
        delete.setEnabled(true);
        newMenu.setEnabled(true);

        this.add(file);
        this.add(edit);
        this.add(util);
    }
    
    private void removeAllActionListeners(JMenuItem item)
    {
        ActionListener[] actions = item.getActionListeners();
        
        for (int i = 0; i < actions.length; i++)
        {
            item.removeActionListener(actions[i]);
        }
    }

    /**
     * 
     * @return the JMenuBar to which new menus can be added
     */
    public JMenuBar getMenuBar() {
        return this;
    }

    
    /*
     * 
     * @return the "New Window" JMenuItem Used to get a reference to the item in the
     *         menu to add an action listener to
     *
    public JMenuItem getNewWindow() {
        return newWindow;
    }*/
    
    /**
     * 
     * @return the "copy" JMenuItem Used to get a reference to the item in the
     *         menu to add an action listener to
     */
    public JMenuItem getCopy() {
        return copy;
    }

    /**
     * 
     * @return the "cut" JMenuItem Used to get a reference to the item in the
     *         menu to add an action listener to
     */
    public JMenuItem getCut() {
        return cut;
    }


    /**
     * 
     * @return the "paste" JMenuItem Used to get a reference to the item in the
     *         menu to add an action listener to
     */
    public JMenuItem getPaste() {
        return paste;
    }

    /**
     * 
     * @return the "redo" JMenuItem Used to get a reference to the item in the
     *         menu to add an action listener to
     */
    public JMenuItem getRedo() {
        return redo;
    }

    /**
     * 
     * @return the "save" JMenuItem Used to get a reference to the item in the
     *         menu to add an action listener to
     */
    public JMenuItem getSave() {
        return save;
    }


    /**
     * 
     * @return the "undoManager" JMenuItem Used to get a reference to the item in the
     *         menu to add an action listener to
     */
    public JMenuItem getUndo() {
        return undo;
    }

    /**
     * 
     * @return the "undoManager" JMenuItem Used to get a reference to the item in the
     *         menu to add an action listener to
     */
    public JMenuItem getQuit() {
        return quit;
    }

    /**
     * @return Returns the find.
     */
    public JMenuItem getFind() {
        return find;
    }

    /**
     * @return Returns the next.
     */
    public JMenuItem getNext() {
        return next;
    }

    /**
     * @return Returns the prev.
     */
    public JMenuItem getPrev() {
        return prev;
    }

    /**
     * @return the delete
     */
    public JMenuItem getDelete() {
        return delete;
    }

    /**
     * @return the newMenu
     */
    public JMenuItem getNewMenu() {
        return newMenu;
    }

}
